using namespace std;

const int Ncuts=39;
const int Nrun=13;
int RunNumber[Nrun]={209, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222};
//const int Nrun=1;
//int RunNumber[Nrun]={209};

void MakeSpline();
void ApplySpline_perTCutG();
void ApplySpline();

void ChioSpline()
{

  TFile *fcut=new TFile("Chio_Cuts_Lucas.root");
  TCutG *cutZ[Ncuts];
  for(int i=0;i<Ncuts;i++) cutZ[i]=(TCutG*)fcut->Get(Form("ZGates%d",i+29));

  double Chio_DE, Chio_Ebis;
  TChain *chain=new TChain("PhysicsTree");
  for(int i=0;i<Nrun;i++) 
  {
    chain->Add(Form("../root/analysis/run_%d.root",RunNumber[i]));
    cout << "RunNumber=" << RunNumber[i] << endl;
  }
  chain->SetBranchStatus("Chio_DE",true);
  chain->SetBranchAddress("Chio_DE",&Chio_DE);
  chain->SetBranchStatus("Chio_Ebis",true);
  chain->SetBranchAddress("Chio_Ebis",&Chio_Ebis);

  TH2F *hChioDE_E_all = new TH2F("hChioDE_E","hChioDE_E",1000,1000,41000,500,1000,26000);
  TH2F *hChioDE_E[Ncuts];
  for(int i=0;i<Ncuts;i++) hChioDE_E[i]=new TH2F(Form("hChioDE_E_%d",i+1),Form("hChioDE_E_%d",i+1),2000,1000,41000,1250,1000,26000);

  int Nentries=chain->GetEntries();
  for(int e=0;e<Nentries;++e)
  {
    if(e%100000==0) cout << e*100./Nentries << "% \r" << flush;

    chain->GetEntry(e);
    hChioDE_E_all->Fill(Chio_Ebis,Chio_DE); 

    //for(int i=0;i<Ncuts;i++) if(cutZ[i]->IsInside(Chio_E,Chio_DE)) {hChioDE_E[i]->Fill(Chio_E,Chio_DE); break;}
    for(int i=0;i<Ncuts;i++) if(cutZ[i]->IsInside(Chio_Ebis,Chio_DE)) {hChioDE_E[i]->Fill(Chio_Ebis,Chio_DE); break;}
  }

  TFile *outFile=new TFile("hist/SingleZ_ChioDE_E.root","recreate");
  for(int i=0;i<Ncuts;i++) hChioDE_E[i]->Write();
  outFile->Close();

  TCanvas * can = new TCanvas(Form("ChioEbis_vs_ChioDE_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                              Form("ChioEbis_vs_ChiodE_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                              0,0,2000,1000);
  can->cd(); gPad-> SetLogz();
  hChioDE_E_all->Draw("colz");
  for(int i = 0 ; i < Ncuts ; i++) cutZ[i]->Draw("same");
}

void MakeSpline()
{
  TFile *inFile=new TFile("hist/SingleZ_ChioDE_E.root");

  TSpline3 *gspline[Ncuts];  

  TH2F* h2 [Ncuts];
  TH1F* hProfile[Ncuts];
  TH1F* pfx[Ncuts]; // extended hProfile 
  TFile *fspline=new TFile("spline_Chio_2024.root","recreate");

  TCanvas * canfit = new TCanvas("canfit","canfit",0,0,2000,1500);

  for(int i=0;i<Ncuts;i++)
    {
      // Get the 2D plot in TCutG
      h2[i]=(TH2F*)inFile->Get(Form("hChioDE_E_%d",i+1));

      // Create the TProfile
      hProfile[i]=(TH1F*)h2[i]->ProfileX();
      hProfile[i]->SetLineWidth(2);
      hProfile[i]->SetDirectory(0);
      canfit->cd();
      if (i==0) {
        hProfile[i]->GetYaxis()->SetRangeUser(5000,26000);
        hProfile[i]->Draw();
      }
      else hProfile[i]->Draw("same");

      // Get the range of the TProfile
      int FirstBin, LastBin;
      double parpol3[4];
      for(int bin=1; bin<hProfile[i]->GetNbinsX(); bin++){
        FirstBin = bin;
        if (hProfile[i]->GetBinContent(bin)>6000) break;
      }
      double parpol1[2];
      for(int bin=hProfile[i]->GetNbinsX(); bin>1 ; bin--){
        LastBin = bin;
        if (hProfile[i]->GetBinContent(bin)>6000) break;
      }

      // Create the extended TProfile
      pfx[i] = new TH1F(Form("pfx_%02d",i+1),Form("pfx_%02d",i+1),hProfile[i]->GetNbinsX(),hProfile[i]->GetBinLowEdge(1),
                             hProfile[i]->GetBinLowEdge(hProfile[i]->GetNbinsX())+hProfile[i]->GetBinWidth(hProfile[i]->GetNbinsX()));
      pfx[i]->SetLineColor(8);
      pfx[i]->SetDirectory(0);

      // find the function to extend the TProfile on the lower range
      TF1 * fitpol3 = new TF1(Form("FitPol3_pfx_%02d",i+1),"pol3",hProfile[i]->GetBinLowEdge(FirstBin),hProfile[i]->GetBinLowEdge(FirstBin)+10000);
      hProfile[i]->Fit(fitpol3,"R");
      fitpol3->GetParameters(parpol3);
      
      // find the function to extend the TProfile on the higher range
      TF1 * fitpol1 = new TF1(Form("FitPol1_pfx_%02d",i+1),"pol1",hProfile[i]->GetBinLowEdge(LastBin)-10000,hProfile[i]->GetBinLowEdge(LastBin));
      hProfile[i]->Fit(fitpol1,"R");
      fitpol1->GetParameters(parpol1);

      // fill the extended TProfile
      float newval,lastval,lasterr;
      for(int bin=1; bin<=FirstBin; bin++){
        newval=0;
        if(i < (Ncuts-4) && i!=0 && i!=1)
          for(int par=0; par<4; par++) newval += parpol3[par]*pow(hProfile[i]->GetBinCenter(bin),par);
        else if (i >= (Ncuts-4))
          newval = hProfile[i]->GetBinContent(FirstBin);
        else{
          newval=0;
          for(int par=0; par<2; par++) newval += parpol1[par]*pow(hProfile[i]->GetBinCenter(bin),par);
          pfx[i]->SetBinContent(bin,newval);
        }
        pfx[i]->SetBinContent(bin,newval);
      }
      for(int bin=FirstBin; bin<=LastBin; bin++){
        newval = hProfile[i]->GetBinContent(bin);
        if (newval!=0){
          pfx[i]->SetBinContent(bin,newval);
          pfx[i]->SetBinError(bin,hProfile[i]->GetBinError(bin));
          lastval = newval;
          lasterr = hProfile[i]->GetBinError(bin);
        }
        else{
          pfx[i]->SetBinContent(bin,lastval);
          pfx[i]->SetBinError(bin,lasterr);
        }
      }
      for(int bin=LastBin; bin<=hProfile[i]->GetNbinsX(); bin++){
        newval=0;
        for(int par=0; par<2; par++) newval += parpol1[par]*pow(hProfile[i]->GetBinCenter(bin),par);
        pfx[i]->SetBinContent(bin,newval);
      }
      pfx[i]->Draw("same");

      gspline[i]=new TSpline3(pfx[i]);
      gspline[i]->SetName(Form("fspline_%d",i+1));
      fspline->cd();
      gspline[i]->Write();

    }

  fspline->Close();

  TGraph * gr = new TGraph();
  gr->SetName("DEvsZ");

  TCanvas * can = new TCanvas("ChioEcorr_vs_Ebis_for_TSplines","ChioEcorr_vs_Ebis_for_TSplines",0,0,2500,2000);
  can->cd();
  cout << "Float_t Chio_DEcorr[" << Ncuts << "] = { " << endl; 
  for(int i=0;i<Ncuts;i++){
    can->cd();
    if(i==0) h2[i]->Draw("col");
    else     h2[i]->Draw("col,same");
    cout << h2[i]->GetMean(2) << ", " << endl;
    gr->SetPoint(i,sqrt(h2[i]->GetMean(2)),i+29);
    //gr->SetPoint(i,h2[i]->GetMean(2),i+29);
    //hProfile[i]->Draw("same");
    pfx[i]->Draw("same");
    gspline[i]->SetLineColor(kBlue);
    gspline[i]->SetLineWidth(3);
    gspline[i]->Draw("lsame");
  }

  TCanvas * cangr = new TCanvas("ToGetRoughCal","ToGetRoughCal",200,0,1200,1000);
  cangr->cd();
  gr->SetMarkerStyle(20);
  gr->Draw("AP");
}


void ApplySpline_perTCutG()
{

  // TCutG from Lucas
  TFile *fcut=new TFile("Chio_Cuts_Lucas.root");
  TCutG *cutZ[Ncuts];

  // TSpline
  TFile *fspline=new TFile("spline_Chio_2024.root","read");
  TSpline3 *gspline[Ncuts];  
  
  for(int i=0;i<Ncuts;i++){ 
    cutZ[i]=(TCutG*)fcut->Get(Form("ZGates%d",i+29));
    gspline[i] = (TSpline3*)fspline->FindObjectAny(Form("fspline_%d",i+1));
  }

  // input data
  TChain *chain=new TChain("PhysicsTree");
  for(int i=0;i<Nrun;i++) 
  {
    chain->Add(Form("../root/analysis/run_%d.root",RunNumber[i]));
    cout << "RunNumber=" << RunNumber[i] << endl;
  }
  double Chio_DE, Chio_Ebis;
  chain->SetBranchStatus("Chio_DE",true);
  chain->SetBranchAddress("Chio_DE",&Chio_DE);
  chain->SetBranchStatus("Chio_Ebis",true);
  chain->SetBranchAddress("Chio_Ebis",&Chio_Ebis);

  // variable after applying TSpline
  double Chio_DEcorr;
  double Chio_DEcorr0[Ncuts] = {
    6003.61,  6407.03,  7288.66,  7553.76, 
    7813.05,  8041.28,  8362.7 ,  8688   , 
    9066.38,  9447.99,  9833.3 ,  10248.3, 
    10674.6,  11120.6,  11552.1,  12036.4, 
    12441.9,  12927.1,  13417.2,  13850.7, 
    14294.9,  14686.5,  15143  ,  15503.7, 
    15875  ,  16296.6,  16618.1,  16978.3, 
    17371  ,  17696.9,  18093.9,  18522.9, 
    18888.3,  19253  ,  19643.1,  19988.4, 
    20330.3,  20716.8,  21052.9 
  };

  TH2F *hChioDE_E_all  = new TH2F("hChioDE_E_all","hChioDE_E_all",1000,1000,41000,500,1000,26000);
  TH2F *hChioDE_E      = new TH2F("hChioDE_E","hChioDE_E",1000,1000,41000,500,1000,26000);
  TH2F *hChioDE_E_corr = new TH2F("hChioDE_E_corr","hChioDE_E_corr",1000,1000,41000,500,1000,26000);

  int Nentries=chain->GetEntries();

  for(int e=0;e<Nentries;++e)
  {
    if(e%100000==0) cout << e*100./Nentries << "% \r" << flush;

    chain->GetEntry(e);
    hChioDE_E_all->Fill(Chio_Ebis,Chio_DE); 
    for(int i=0;i<Ncuts;i++) if(cutZ[i]->IsInside(Chio_Ebis,Chio_DE)) {
      hChioDE_E->Fill(Chio_Ebis,Chio_DE); 
      Chio_DEcorr = Chio_DEcorr0[i] * Chio_DE / gspline[i]->Eval(Chio_Ebis); 
      hChioDE_E_corr->Fill(Chio_Ebis,Chio_DEcorr); 
      break;
    }
  }

  TCanvas * can = new TCanvas(Form("ChioEbis_vs_ChioDE_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                              Form("ChioEbis_vs_ChiodE_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                              0,0,2000,1000);
  can->Divide(1,3);
  can->cd(1); gPad-> SetLogz(); hChioDE_E_all->Draw("colz");
  can->cd(2); gPad-> SetLogz(); hChioDE_E->Draw("colz");
  can->cd(3); gPad-> SetLogz(); hChioDE_E_corr->Draw("colz");

}

void ApplySpline()
{
  // TCutG from Lucas
  TFile *fcut=new TFile("Chio_Cuts_Lucas.root");
  TCutG *cutZ[Ncuts];

  // TSpline
  TFile *fspline=new TFile("spline_Chio_2024.root","read");
  TSpline3 *gspline[Ncuts];  
  
  for(int i=0;i<Ncuts;i++){ 
    cutZ[i]=(TCutG*)fcut->Get(Form("ZGates%d",i+29));
    gspline[i] = (TSpline3*)fspline->FindObjectAny(Form("fspline_%d",i+1));
  }

  // input data
  TChain *chain=new TChain("PhysicsTree");
  for(int i=0;i<Nrun;i++) 
  {
    chain->Add(Form("../root/analysis/run_%d.root",RunNumber[i]));
    cout << "RunNumber=" << RunNumber[i] << endl;
  }
  double Chio_DE, Chio_Ebis;
  chain->SetBranchStatus("Chio_DE",true);
  chain->SetBranchAddress("Chio_DE",&Chio_DE);
  chain->SetBranchStatus("Chio_Ebis",true);
  chain->SetBranchAddress("Chio_Ebis",&Chio_Ebis);

  // variable after applying TSpline
  double Chio_DEcorr, Zrough, Zabitbetter;
  double Chio_DEcorr0[Ncuts] = {
    6003.61,  6407.03,  7288.66,  7553.76, 
    7813.05,  8041.28,  8362.7 ,  8688   , 
    9066.38,  9447.99,  9833.3 ,  10248.3, 
    10674.6,  11120.6,  11552.1,  12036.4, 
    12441.9,  12927.1,  13417.2,  13850.7, 
    14294.9,  14686.5,  15143  ,  15503.7, 
    15875  ,  16296.6,  16618.1,  16978.3, 
    17371  ,  17696.9,  18093.9,  18522.9, 
    18888.3,  19253  ,  19643.1,  19988.4, 
    20330.3,  20716.8,  21052.9 
  };

  // histos
  TH2F *hChioDE_E_all  = new TH2F("hChioDE_E_all","hChioDE_E_all",1000,1000,41000,500,1000,26000);
  TH2F *hChioDE_E_corr = new TH2F("hChioDE_E_corr","hChioDE_E_corr",1000,1000,41000,500,1000,26000);
  TH1F *hChioDE_corr   = new TH1F("hChioDE_corr","hChioDE_corr",2500,1000,26000);
  TH2F *hChioZ_E_rough = new TH2F("hChioZ_E_rough","hChioZ_E_rough",1000,1000,41000,500,25,65); 
  TH1F *hChioZ_rough = new TH1F("hChioZ_rough","hChioZ_rough",2500,25,65); 
  TH1F *hChioZ_abitbetter = new TH1F("hChioZ_abitbetter","hChioZ_abitbetter",2500,25,65); 

  int Nentries=chain->GetEntries();

  for(int e=0;e<Nentries;++e)
  {
    if(e%100000==0) cout << e*100./Nentries << "% \r" << flush;

    chain->GetEntry(e);
    if (Chio_DE<0) continue;
     
    hChioDE_E_all->Fill(Chio_Ebis,Chio_DE);
 
    float Eval_DEspline, DEspline0;
    int   index=0;
    for(int i=0; i<Ncuts; i++){
      Eval_DEspline = gspline[i]->Eval(Chio_Ebis); 
      if(Chio_DE<Eval_DEspline) break;
      index = i;
    }
    
    Eval_DEspline = gspline[index]->Eval(Chio_Ebis);
    DEspline0 = Chio_DEcorr0[index];
    float dmin, dsup;
    if( index < (Ncuts-1) && Chio_DE > gspline[0]->Eval(Chio_Ebis)){
      dmin = Chio_DE-gspline[index]->Eval(Chio_Ebis);
      dsup = gspline[index+1]->Eval(Chio_Ebis)-Chio_DE;
      if(dmin<0) cout << "negative value of dmin = " << dmin << ", for index = " << index << endl;
      if(dsup<0) cout << "negative value of dsup = " << dsup << ", for index = " << index << endl;
      //if(dsup<dmin) {
      //  Eval_DEspline = gspline[index+1]->Eval(Chio_Ebis) ;
      //  DEspline0 = Chio_DEcorr0[index] ;
      //}
      Eval_DEspline = dsup*gspline[index]->Eval(Chio_Ebis)/(dmin+dsup) + dmin*gspline[index+1]->Eval(Chio_Ebis)/(dmin+dsup) ;
      DEspline0 = dsup*Chio_DEcorr0[index]/(dmin+dsup) + dmin*Chio_DEcorr0[index+1]/(dmin+dsup) ;
    }

    Chio_DEcorr = DEspline0 * Chio_DE / Eval_DEspline ;
    hChioDE_E_corr->Fill(Chio_Ebis,Chio_DEcorr);

    // should be pol2 !!!
    Zrough = -110.165 + 3.34475*sqrt(Chio_DEcorr) - 0.0271123*Chio_DEcorr + 8.60752e-05 * pow(sqrt(Chio_DEcorr),3);
    Zabitbetter = -127.117 + 3.83463*sqrt(Chio_DEcorr) - 0.0317448 *Chio_DEcorr + 0.000100428 * pow(sqrt(Chio_DEcorr),3);
    hChioZ_E_rough->Fill(Chio_Ebis,Zrough);
    
    if(8000<Chio_Ebis && Chio_Ebis<25000){ 
      hChioDE_corr->Fill(Chio_DEcorr);
      hChioZ_rough->Fill(Zrough);
      hChioZ_abitbetter->Fill(Zabitbetter);
    }
  }

  TCanvas * can = new TCanvas(Form("ChioEbis_vs_ChioDE_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                              Form("ChioEbis_vs_ChiodE_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                              0,0,2000,2000);
  can->Divide(1,3);
  can->cd(1); gPad-> SetLogz(); hChioDE_E_all->Draw("colz");
  for(int i = 0 ; i < Ncuts ; i++) cutZ[i]->Draw("same");
  can->cd(2); gPad-> SetLogz(); hChioDE_E_corr->Draw("colz");
  can->cd(3); gPad-> SetLogz(); hChioZ_E_rough->Draw("colz");

  TCanvas * can1D = new TCanvas(Form("DE_and_Z_ifE_8000to25000_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                                Form("DE_and_Z_ifE_8000to25000_run%04d_%04d",RunNumber[0],RunNumber[Nrun-1]),
                                0, 0, 2000,2000);
  can1D->Divide(1,3);
  can1D->cd(1);  hChioDE_corr->Draw(); 
  can1D->cd(2);  hChioZ_rough->Draw();
  can1D->cd(3);  hChioZ_abitbetter->Draw();

  TFile * fsave = new TFile(Form("DEvsE_corr_run%04d_%04d.root",RunNumber[0],RunNumber[Nrun-1]),"recreate");  
  can->Write();
  hChioDE_E_all->Write();
  hChioDE_E_corr->Write();
  hChioZ_E_rough->Write();
  can1D->Write();
  hChioDE_corr->Write();
  hChioZ_rough->Write();
  hChioZ_abitbetter->Write();
  fsave->Close();
}

void Fit_DE_E_corr()
{

  TFile * f = new TFile(Form("DEvsE_corr_run%04d_%04d.root",RunNumber[0],RunNumber[Nrun-1]),"read");
  f->ls();
  TH1F * h1DEcorr = (TH1F*)f->Get("hChioDE_corr");
  h1DEcorr->SetDirectory(0);
  f->Close();

  TCanvas * canfit = new TCanvas("DEcorrforfit","Decorrforfit",0,0,3000,1000);
  canfit->Divide(2,1);
  canfit->cd(1);   h1DEcorr->Draw();

  float InitMean[34] = {
    7.55521e+03,     7.82392e+03,    8.04283e+03,     8.36164e+03,   
    8.68128e+03,     9.06218e+03,    9.44613e+03,     9.82352e+03,   
    1.02433e+04,     1.06666e+04,    1.11215e+04,     1.15371e+04,   
    1.20246e+04,     1.24492e+04,    1.29190e+04,     1.34056e+04,   
    1.38367e+04,     1.42968e+04,    1.46816e+04,     1.51614e+04,   
    1.55115e+04,     1.58873e+04,    1.63049e+04,     1.66187e+04,   
    1.69730e+04,     1.73969e+04,    1.76965e+04,     1.80730e+04,   
    1.85234e+04,     1.88849e+04,    1.92127e+04,     1.96378e+04,   
    1.99820e+04,     2.02914e+04
  };

  float InitSigma[34] = {
    6.97215e+01,    7.76394e+01,    7.84506e+01,    8.23239e+01, 
    9.25943e+01,    1.06154e+02,    1.03566e+02,    1.02429e+02, 
    1.10066e+02,    1.15032e+02,    1.20449e+02,    1.15397e+02, 
    1.30322e+02,    1.29513e+02,    1.45757e+02,    1.53552e+02, 
    1.48061e+02,    1.49866e+02,    1.60311e+02,    1.56691e+02, 
    1.55461e+02,    1.78093e+02,    1.66829e+02,    1.67863e+02, 
    1.75509e+02,    1.75189e+02,    1.70589e+02,    1.76178e+02, 
    1.65192e+02,    1.87470e+02,    1.78153e+02,    1.58322e+02, 
    1.53678e+02,    1.97134e+02
  };

  char name[1000];
  TString totname;
  for(int i=0; i<34;i++){
    sprintf(name,"%i",i*3);
    totname += "gaus(" + TString(name) + ")+";
  }  
  totname.Remove(totname.Last('+'));
  TF1 * gtot = new TF1("MultiGaussianFit",totname,InitMean[0]-2*InitSigma[0],InitMean[33]+2*InitSigma[33]);

  for(int i=0; i<34;i++){
    gtot->SetParameter(i*3+1,InitMean[i]); 
    gtot->SetParameter(i*3+2,InitSigma[i]); 
  }  
  h1DEcorr->Fit(gtot,"R+");

  TGraph * gr = new TGraph();
  gr->SetName("DEfit_vs_Z");
  gr->SetMarkerStyle(20);
  for(int i=0; i<33; i++){
   gr->SetPoint(i,sqrt(gtot->GetParameter(i*3+1)),i+32); 
  }

  canfit->cd(2); gr->Draw("AP");

}
