#include "TCutG.h"
#include "TFile.h"

using namespace std;

TCutG *cut[34];
TSpline3 *gSpline3[34];

int IdentifyZ(double,double);
int FindClosestZ(double, double);

void FillHistoPISTA(int Run)
{

  //  const int Nrun=10;
  const int Nrun=1;
  //  int RunNumber[Nrun]={29,41,42,43,44,45,49,50,51,52};
  int RunNumber[Nrun]={Run};
  //  int RunNumber[Nrun]={45,49,50,51};

  double Reference[34];
  for(int i=0;i<34;i++) Reference[i]=10000-i/34.*(10000-6539);

  TChain *chain=new TChain("PhysicsTree");
  for(int i=0;i<Nrun;i++) 
  {
    chain->Add(Form("../root/analysis/run_%d.root",RunNumber[i]));
    cout << "RunNumber=" << RunNumber[i] << endl;
  }

  ///////// Cuts

  TFile *fcut=new TFile("../Cut_Pista_Th232.root");
  TCutG *cutBe10=(TCutG*)fcut->Get("cut10Be");
  TCutG *cutC12=(TCutG*)fcut->Get("cutC");
  TCutG *cut2a=(TCutG*)fcut->Get("cut2a");
  
  // TFile *fcutChio=new TFile("cut_Chio_Indu.root");
  // for(int i=0;i<34;i++) cut[i]=(TCutG*)fcutChio->Get(Form("Z%d",i+1));

  // TFile *Fspline=new TFile("spline_Chio.root");
  // for(int i=0;i<34;i++) gSpline3[i]=(TSpline3*)Fspline->Get(Form("fspline_%d",i+1));

  //////// Variables and branches

  double DeltaE, Elab;
  ULong64_t fTS_TMW;
  double Ex232Th, Ex234U, Ex236U;
  double Chio_DE, Chio_E;
  Float_t fTAC_MW1_PISTA;
  int m_2alpha;

  chain->SetBranchStatus("DeltaE",true);
  chain->SetBranchAddress("DeltaE",&DeltaE);
  chain->SetBranchStatus("Elab",true);
  chain->SetBranchAddress("Elab",&Elab);
  chain->SetBranchStatus("fTS_TMW",true);
  chain->SetBranchAddress("fTS_TMW",&fTS_TMW);
  chain->SetBranchStatus("fTAC_MW1_PISTA",true);
  chain->SetBranchAddress("fTAC_MW1_PISTA",&fTAC_MW1_PISTA);
  chain->SetBranchStatus("Ex232Th",true);
  chain->SetBranchAddress("Ex232Th",&Ex232Th);
  
  chain->SetBranchStatus("Ex236U",true);
  chain->SetBranchAddress("Ex236U",&Ex236U);
  chain->SetBranchStatus("Ex234U",true);
  chain->SetBranchAddress("Ex234U",&Ex234U);
  chain->SetBranchStatus("Chio_DE",true);
  chain->SetBranchAddress("Chio_DE",&Chio_DE);
  chain->SetBranchStatus("Chio_E",true);
  chain->SetBranchAddress("Chio_E",&Chio_E);
  chain->SetBranchAddress("m_2alpha",&m_2alpha);

  /////// Histograms

  TH2F* DE_E=new TH2F("DE_E","DE_E",1000,0,200,1000,0,60);
  TH1F* hEx232Th=new TH1F("hEx232Th","hEx232Th",500,-5,20);
  TH1F* hEx232Th_chio=new TH1F("hEx232Th_chio","hEx232Th_chio",500,-5,20);
  TH1F* hEx236U=new TH1F("hEx236U","hEx236U",500,-5,20);
  TH1F* hEx236U_chio=new TH1F("hEx236U_chio","hEx236U_chio",500,-5,20);
  TH1F* hEx234U=new TH1F("hEx234U","hEx234U",500,-5,20);
  TH1F* hEx234U_chio=new TH1F("hEx234U_chio","hEx234U_chio",500,-5,20);
  
  TH2F* hChio_DE_E=new TH2F("hChio_DE_E","hChio_DE_E",1000,0,20000,1000,0,30000);
  TH2F* hChio_DE_E_corr=new TH2F("hChio_DE_E_corr","hChio_DE_E_corr",1000,0,20000,1000,0,30000);
  TH2F* hChio1_DE_E=new TH2F("hChio1_DE_E","hChio1_DE_E",1000,0,20000,1000,0,20000);
  TH2F* hChioDE_Ex232Th=new TH2F("hChioDE_Ex232Th","hChioDE_Ex232Th",500,0,20,900,0,30000);
  TH2F* hChioDE_Ex236U=new TH2F("hChioDE_Ex236U","hChioDE_Ex236U",500,0,20,1000,0,30000);
  TH2F* hChioDE_Ex234U=new TH2F("hChioDE_Ex234U","hChioDE_Ex234U",500,0,20,1000,0,30000);


  /////// Filling of histograms

  int Nentries=chain->GetEntries();

  for(int e=0;e<Nentries;++e)
    {
      if(e%1000000==0) cout << e*100./Nentries << "% \r" << flush;

      chain->GetEntry(e);


      //    if(fTS_TMW==0){
      //   if(cutBe10->IsInside(Eres,DeltaEcorr)) {
      //     hEx240Pu_div->Fill(Ex240Pu);
      //   }
      //   else if(cutC12->IsInside(Eres,DeltaEcorr)) {
      //     hEx232Th_div->Fill(Ex232Th);
      //   }
      //    }
    
      //else if(fTS_TMW>0 && fTAC_MW1_PISTA>0)
      {
	DE_E->Fill(Elab,DeltaE);
	hChio_DE_E->Fill(Chio_E,Chio_DE);

	// if(Chio_E>4000 && Chio_E<15000) 
	//   {
	//     int Zid=FindClosestZ(Chio_E,Chio_DE); //IdentifyZ(Chio_E,Chio_DE);
	//     //	  if(Zid==-1) {Zid=FindClosestZ(Chio_E,Chio_DE); }//cout << "Zid=" << Zid << " Chio_E=" << Chio_E << " Chio_DE=" << Chio_DE << endl;}
	//     if(Zid==-1) continue;
	//     double reference=Reference[Zid];
	//     double delta=gSpline3[Zid]->Eval(Chio_E) - gSpline3[Zid]->Eval(reference);
	//     hChio_DE_E_corr->Fill(Chio_E,Chio_DE-delta);
	//   }

	if(cutBe10->IsInside(Elab,DeltaE)) 
	  {
	    hEx234U->Fill(Ex234U);
	    if(Chio_E>5000 && Chio_E<10000) hChioDE_Ex234U->Fill(Ex234U,Chio_DE);
	    // if(Chio_E>1000 && Chio_E<15000){
	    //   int Zid=FindClosestZ(Chio_E,Chio_DE); //IdentifyZ(Chio_E,Chio_DE);
	    //   //	  if(Zid==-1) Zid=FindClosestZ(Chio_E,Chio_DE); 
	    //   if(Zid==-1) continue;
	    //   double reference=Reference[Zid];
	    //   double delta=gSpline3[Zid]->Eval(Chio_E) - gSpline3[Zid]->Eval(reference);
	    //   hChioDEcorr_Ex234U->Fill(Ex234U,Chio_DE-delta);
	    //   //          hChioDEcorr_Ex240Pu->Fill(Ex240Pu,Chio_DE*10000./fspline->Eval(Chio_E));
	    hEx234U_chio->Fill(Ex234U);
	  }
	else if(cutC12->IsInside(Elab,DeltaE)) 
	  {
	    hEx232Th->Fill(Ex232Th);
	    if(Chio_E>1000 && Chio_E<15000){ 
	      //  hChioDE_Ex232Th->Fill(Ex232Th,Chio_DE*10000./fspline->Eval(Chio_E));
	      hEx232Th_chio->Fill(Ex232Th);
	    }
	  }
	else if(cut2a->IsInside(Elab,DeltaE) && m_2alpha==1) 
	  {
	    hEx236U->Fill(Ex236U);
	    if(Chio_E>1000 && Chio_E<15000){
	      
	      // int Zid=FindClosestZ(Chio_E,Chio_DE); //IdentifyZ(Chio_E,Chio_DE);
	      // //	  if(Zid==-1) Zid=FindClosestZ(Chio_E,Chio_DE); 
	      // if(Zid==-1) continue;
	      // double reference=Reference[Zid];
	      // double delta=gSpline3[Zid]->Eval(Chio_E) - gSpline3[Zid]->Eval(10000);

	      // //hChioDEcorr_Ex236U->Fill(Ex236U,Chio_DE*10000./fspline->Eval(Chio_E));
	      hChioDE_Ex236U->Fill(Ex236U,Chio_DE);//-delta);
	      hEx236U_chio->Fill(Ex236U);
	    }
	  }

	//	if(cutChio1->IsInside(Chio_E,Chio_DE))
	hChio1_DE_E->Fill(Chio_E,Chio_DE);

      }
    }

  TH1F* hProfile=(TH1F*)hChio1_DE_E->ProfileX();
  // TSpline3();

  TFile *OutFile;
  if(Nrun>1) OutFile=new TFile(Form("hist/HistoPISTA_run%d-%d.root",RunNumber,RunNumber[Nrun-1]), "recreate");
  else if(Nrun==1) OutFile=new TFile(Form("hist/HistoPISTA_run%d.root",RunNumber[0]), "recreate");

  TH1F* hEx232Th_FB=(TH1F*)hEx232Th_chio->Clone("hEx232Th_FB");
  TH1F* hEx236U_FB=(TH1F*)hEx236U_chio->Clone("hEx236U_FB");
  TH1F* hEx234U_FB=(TH1F*)hEx234U_chio->Clone("hEx234U_FB");
  
  DE_E->Write();
  hChio_DE_E->Write();
  hChio1_DE_E->Write();
  hChio_DE_E_corr->Write();

  hEx234U->Write();
  hEx232Th->Write();
  hEx236U->Write();
  hEx234U_chio->Write();
  hEx232Th_chio->Write();
  hEx236U_chio->Write();
  hEx234U_FB->Write();
  hEx232Th_FB->Write();
  hEx236U_FB->Write();

  hChioDE_Ex234U->Write();
  //  hChioDEcorr_Ex234U->Write();
  hChioDE_Ex232Th->Write();
  hChioDE_Ex236U->Write();

  OutFile->Close();
}


int IdentifyZ(double Chio_E, double Chio_DE)
{

  if(cut[0]->IsInside(Chio_E,Chio_DE)) return 0;
  else if(cut[1]->IsInside(Chio_E,Chio_DE)) return 1;
  else if(cut[2]->IsInside(Chio_E,Chio_DE)) return 2;
  else if(cut[3]->IsInside(Chio_E,Chio_DE)) return 3;
  else if(cut[4]->IsInside(Chio_E,Chio_DE)) return 4;
  else if(cut[5]->IsInside(Chio_E,Chio_DE)) return 5;
  else if(cut[6]->IsInside(Chio_E,Chio_DE)) return 6;
  else if(cut[7]->IsInside(Chio_E,Chio_DE)) return 7;
  else if(cut[8]->IsInside(Chio_E,Chio_DE)) return 8;
  else if(cut[9]->IsInside(Chio_E,Chio_DE)) return 9;
  else if(cut[10]->IsInside(Chio_E,Chio_DE)) return 10;
  else if(cut[11]->IsInside(Chio_E,Chio_DE)) return 11;
  else if(cut[12]->IsInside(Chio_E,Chio_DE)) return 12;
  else if(cut[13]->IsInside(Chio_E,Chio_DE)) return 13;
  else if(cut[14]->IsInside(Chio_E,Chio_DE)) return 14;
  else if(cut[15]->IsInside(Chio_E,Chio_DE)) return 15;
  else if(cut[16]->IsInside(Chio_E,Chio_DE)) return 16;
  else if(cut[17]->IsInside(Chio_E,Chio_DE)) return 17;
  else if(cut[18]->IsInside(Chio_E,Chio_DE)) return 18;
  else if(cut[19]->IsInside(Chio_E,Chio_DE)) return 19;
  else if(cut[20]->IsInside(Chio_E,Chio_DE)) return 20;
  else if(cut[21]->IsInside(Chio_E,Chio_DE)) return 21;
  else if(cut[22]->IsInside(Chio_E,Chio_DE)) return 22;
  else if(cut[23]->IsInside(Chio_E,Chio_DE)) return 23;
  else if(cut[24]->IsInside(Chio_E,Chio_DE)) return 24;
  else if(cut[25]->IsInside(Chio_E,Chio_DE)) return 25;
  else if(cut[26]->IsInside(Chio_E,Chio_DE)) return 26;
  else if(cut[27]->IsInside(Chio_E,Chio_DE)) return 27;
  else if(cut[28]->IsInside(Chio_E,Chio_DE)) return 28;
  else if(cut[29]->IsInside(Chio_E,Chio_DE)) return 29;
  else if(cut[30]->IsInside(Chio_E,Chio_DE)) return 30;
  else if(cut[31]->IsInside(Chio_E,Chio_DE)) return 31;
  else if(cut[32]->IsInside(Chio_E,Chio_DE)) return 32;
  else if(cut[33]->IsInside(Chio_E,Chio_DE)) return 33;
  else return -1;
}

int FindClosestZ(double Chio_E, double Chio_DE)
{
  double DEmin=100000;
  int Zmin=-1;

  for(int i=2;i<34;i++)
  {
    //cout << "i= " << i << endl;
    double test_DE=gSpline3[i]->Eval(Chio_E);
    //cout << test_DE << endl;
    if(fabs(test_DE-Chio_DE)<DEmin && test_DE>1000)
    {
      //cout << "DEmin=" << DEmin << " / test_DE-Chio_DE=" << fabs(test_DE-Chio_DE) << " / test_DE=" << test_DE << " /Chio_DE=" << Chio_DE << endl;
      DEmin=fabs(test_DE-Chio_DE);
      Zmin=i;
    }
  }
  return Zmin;
}
