#include "TCutG.h"
#include "TFile.h"

using namespace std;

const int Ncuts=39;
TCutG *cut[Ncuts];
TSpline3 *gSpline3[Ncuts];

int IdentifyZ(double,double);
int FindClosestZ(double, double);

void fastFillHistoPISTA2(int RunNumber, int FileNumber)
{
  double Reference[Ncuts];
  int Zvalue[Ncuts];
  for(int i=0;i<Ncuts;i++)
    {
      Reference[i]=20000-i*1./Ncuts*(10000-6539);
      Zvalue[i]=i+29;
    }

  TChain *chain=new TChain("PhysicsTree");
  chain->Add(Form("../root/analysis/run_files/run_%d_0%d.root",RunNumber,FileNumber));
  cout << "Treating Run=" << RunNumber << " file=" << FileNumber << endl;

  ///////// PISTA cuts

  TFile *fcut=new TFile("cut_PISTA_PID.root");
  TCutG * cutBe10=(TCutG*)fcut->Get("cut_be10");
  TCutG *  cutC12=(TCutG*)fcut->Get("cut_c12");
  TCutG *  cutC14=(TCutG*)fcut->Get("cut_c14");
  TCutG *   cut2a=(TCutG*)fcut->Get("cut_2a");
  
  TFile *fcutChio=new TFile("Chio_Cuts_Lucas.root");
  for(int i=0;i<39;i++) cut[i]=(TCutG*)fcutChio->Get(Form("ZGates%d",i+29));

  TFile *Fspline=new TFile("spline_Chio_2024.root");
  for(int i=0;i<39;i++) gSpline3[i]=(TSpline3*)Fspline->Get(Form("fspline_%d",i+1));

  //////// Variables and branches

  double DeltaE, Elab;
  ULong64_t fTS_TMW;
  double Ex238U, Ex242Pu, Ex236U, Ex240Pu;
  double Chio_DE, Chio_E, Mass;
  Float_t fTAC_MW1_PISTA;
  int Telescope, m_2alpha;

  chain->SetBranchStatus("DeltaE",true);
  chain->SetBranchAddress("DeltaE",&DeltaE);
  chain->SetBranchStatus("Eres",true);
  chain->SetBranchAddress("Eres",&Elab);
  chain->SetBranchStatus("fTS_TMW",true);
  chain->SetBranchAddress("fTS_TMW",&fTS_TMW);
  chain->SetBranchStatus("fTAC_MW1_PISTA",true);
  chain->SetBranchAddress("fTAC_MW1_PISTA",&fTAC_MW1_PISTA);
  chain->SetBranchStatus("Ex238U",true);
  chain->SetBranchAddress("Ex238U",&Ex238U);
  chain->SetBranchAddress("Telescope",&Telescope);
  chain->SetBranchAddress("m_2alpha",&m_2alpha);

  chain->SetBranchStatus("Ex242Pu",true);
  chain->SetBranchAddress("Ex242Pu",&Ex242Pu);
  chain->SetBranchStatus("Ex240Pu",true);
  chain->SetBranchAddress("Ex240Pu",&Ex240Pu);
  chain->SetBranchStatus("Ex236U",true);
  chain->SetBranchAddress("Ex236U",&Ex236U);
  

  chain->SetBranchStatus("Chio_DE",true);
  chain->SetBranchAddress("Chio_DE",&Chio_DE);
  chain->SetBranchStatus("Chio_Ebis",true);
  chain->SetBranchAddress("Chio_Ebis",&Chio_E);
  chain->SetBranchStatus("Mass",true);
  chain->SetBranchAddress("Mass",&Mass);

  /////// Histograms

  TH2F* DE_E=new TH2F("DE_E","DE_E",1000,0,200,1000,0,60);
  TH2F* DE_E_Mass_Tel[8];
  TH2F* hEx238U_run[8];
  for(int i=0;i<8;i++)
    {
      DE_E_Mass_Tel[i]=new TH2F(Form("DE_E_Mass_%d",i),Form("DE_E_Mass_%d",i),1000,0,200,1000,0,60);
      hEx238U_run[i] = new TH2F(Form("hEx238U_run_Tel%d",i+1),Form("hEx238U_run_Tel%d",i+1),200,100,300,250,-5,20);
    }
  
  TH1F* hEx238U=new         TH1F("hEx238U","hEx238U",250,-5,20);
  TH1F* hEx238U_chio=new   TH1F("hEx238U_chio","hEx238U_chio",250,-5,20);
  TH1F* hEx238U_div=new TH1F("hEx238U_div","hEx238U_div",250,-5,20);
  TH1F* hEx242Pu=new TH1F("hEx242Pu","hEx242Pu",300,-5,35);
  TH1F* hEx242Pu_chio=new TH1F("hEx242Pu_chio","hEx242Pu_chio",300,-5,35);
  TH1F* hEx242Pu_div=new TH1F("hEx242Pu_div","hEx242Pu_div",300,-5,35);
  TH1F* hEx236U=new TH1F("hEx236U","hEx236U",250,-5,20);
  TH1F* hEx236U_chio=new TH1F("hEx236U_chio","hEx236U_chio",250,-5,20);
  TH1F* hEx236U_div=new TH1F("hEx236U_div","hEx236U_div",250,-5,20);
  TH1F* hEx240Pu=new TH1F("hEx240Pu","hEx240Pu",250,-5,20);
  TH1F* hEx240Pu_chio=new TH1F("hEx240Pu_chio","hEx240Pu_chio",250,-5,20);
  TH1F* hEx240Pu_div=new TH1F("hEx240Pu_div","hEx240Pu_div",250,-5,20);
  TH1F* hEx240Pu_Tel1=new TH1F("hEx240Pu_Tel1","hEx240Pu_Tel1",250,-5,20);
  TH1F* hEx240Pu_Tel1_chio=new TH1F("hEx240Pu_Tel1_chio","hEx240Pu_Tel1_chio",250,-5,20);
  TH1F* hEx240Pu_Tel1_div=new TH1F("hEx240Pu_Tel1_div","hEx240Pu_Tel1_div",250,-5,20);

  TH2F* hEx240Pu_run=new TH2F("hEx240Pu_run","hEx240Pu_run",250,-5,20,30,100,130);
  TH2F* hEx236U_run=new TH2F("hEx236U_run","hEx236U_run",250,-5,20,30,100,130);
  TH2F* hEx242Pu_run=new TH2F("hEx242Pu_run","hEx242Pu_run",250,-5,20,30,100,130);
  //  TH2F* hEx238U_run=new TH2F("hEx238U_run","hEx238U_run",250,-5,20,30,100,130);

  TH2F* hEx240Pu_Tel=new TH2F("hEx240Pu_Tel","hEx240Pu_Tel",250,-5,20,9,1,10);
  TH2F* hEx236U_Tel=new TH2F("hEx236U_Tel","hEx236U_Tel",250,-5,20,9,1,10);
  TH2F* hEx242Pu_Tel=new TH2F("hEx242Pu_Tel","hEx242Pu_Tel",350,0,35,9,1,10);
  TH2F* hEx238U_Tel=new TH2F("hEx238U_Tel","hEx238U_Tel",250,-5,20,9,1,10);
  

  TH1F* hMass=new TH1F("hMass","hMass",500,70,160);
  TH2F* hChio_DE_E=new TH2F("hChio_DE_E","hChio_DE_E",1000,0,60000,1000,0,30000);
  TH2F* hChio_DE_E_corr=new TH2F("hChio_DE_E_corr","hChio_DE_E_corr",1000,0,60000,1000,0,30000);
  TH2F* hChioCorr_Z=new TH2F("hChioCorr_Z","hChioCorr_Z",1000,0,30000,80,0,80);
  TH2F* hChio1_DE_E=new TH2F("hChio1_DE_E","hChio1_DE_E",1000,0,20000,1000,0,20000);
  TH2F* hMass_Ex238U=new TH2F("hMass_Ex238U","hMass_Ex238U",20,0,20,90,69.5,159.5);
  TH2F* hMass_Ex242Pu=new TH2F("hMass_Ex242Pu","hMass_Ex242Pu",35,0,35,90,69.5,159.5);
  TH2F* hMass_Ex240Pu=new TH2F("hMass_Ex240Pu","hMass_Ex240Pu",20,0,20,90,69.5,159.5);
  TH2F* hMass_Ex236U=new TH2F("hMass_Ex236U","hMass_Ex236U",20,0,20,90,69.5,159.5);
  TH2F* hZ_Ex238U=new TH2F("hZ_Ex238U","hZ_Ex238U",20,0,20,50,20.5,70.5);
  TH2F* hZ_Ex242Pu=new TH2F("hZ_Ex242Pu","hZ_Ex242Pu",35,0,35,50,20.5,70.5);
  TH2F* hZ_Ex240Pu=new TH2F("hZ_Ex240Pu","hZ_Ex240Pu",20,0,20,50,20.5,70.5);
  TH2F* hZ_Ex236U=new TH2F("hZ_Ex236U","hZ_Ex236U",20,0,20,50,20.5,70.5);
  TH2F* hNorm_Ex238U=new TH2F("hNorm_Ex238U","hNorm_Ex238U",20,0,20,90,69.5,159.5);
  TH2F* hNorm_Ex242Pu=new TH2F("hNorm_Ex242Pu","hNorm_Ex242Pu",35,0,35,90,69.5,159.5);
  TH2F* hNorm_Ex240Pu=new TH2F("hNorm_Ex240Pu","hNorm_Ex240Pu",20,0,20,90,69.5,159.5);
  TH2F* hNorm_Ex236U=new TH2F("hNorm_Ex236U","hNorm_Ex236U",20,0,20,90,69.5,159.5);


  /////// Filling of histograms

  int Nentries=chain->GetEntries();

  for(int e=0;e<Nentries;++e)
    {
      if(e%1000000==0) cout << e*100./Nentries << "% \r" << flush;

      chain->GetEntry(e);

      int Z=-1;
      hMass->Fill(Mass);
      //    if(fTS_TMW==0){
      //   if(cutBe10->IsInside(Eres,DeltaEcorr)) {
      //     hEx240Pu_div->Fill(Ex240Pu);
      //   }
      //   else if(cutC12->IsInside(Eres,DeltaEcorr)) {
      //     hEx238U_div->Fill(Ex238U);
      //   }
      //    }
    
      //else if(fTS_TMW>0 && fTAC_MW1_PISTA>0)
      {

     	hChio_DE_E->Fill(Chio_E,Chio_DE);

	if(Mass>60 && Mass<200 && Telescope>0)
	  {
	    DE_E_Mass_Tel[Telescope-1]->Fill(Elab,DeltaE);
	    DE_E->Fill(Elab,DeltaE);
	  }

	if(Chio_E>6000 && Chio_E<60000) 
	  {
	    int Zid=FindClosestZ(Chio_E,Chio_DE); //IdentifyZ(Chio_E,Chio_DE);
	    //	  if(Zid==-1) {Zid=FindClosestZ(Chio_E,Chio_DE); }//cout << "Zid=" << Zid << " Chio_E=" << Chio_E << " Chio_DE=" << Chio_DE << endl;}
	    if(Zid==-1) continue;
	    double reference=Reference[Zid];
	    double delta=gSpline3[Zid]->Eval(Chio_E) - gSpline3[Zid]->Eval(reference);
	    hChio_DE_E_corr->Fill(Chio_E,Chio_DE-delta);
	    hChioCorr_Z->Fill(Chio_DE-delta,Zvalue[Zid]);
	    Z=Zvalue[Zid];
	  }

	if(cutBe10->IsInside(Elab,DeltaE)) 
	  {
	    hEx240Pu->Fill(Ex240Pu);
	    hEx240Pu_run->Fill(Ex240Pu,RunNumber);
	    hEx240Pu_Tel->Fill(Ex240Pu,Telescope);
	    if(Telescope==5) hEx240Pu_Tel1->Fill(Ex240Pu);
	    if(Mass>70 && Mass<160) hMass_Ex240Pu->Fill(Ex240Pu,Mass);
	    if(Mass>0 && Mass<200){
	    //   int Zid=FindClosestZ(Chio_E,Chio_DE); //IdentifyZ(Chio_E,Chio_DE);
	    //   //	  if(Zid==-1) Zid=FindClosestZ(Chio_E,Chio_DE); 
	    //   if(Zid==-1) continue;
	    //   double reference=Reference[Zid];
	    //   double delta=gSpline3[Zid]->Eval(Chio_E) - gSpline3[Zid]->Eval(reference);
	    //   hMasscorr_Ex240Pu->Fill(Ex240Pu,Chio_DE-delta);
	    //   //          hMasscorr_Ex240Pu->Fill(Ex240Pu,Chio_DE*10000./fspline->Eval(Chio_E));
	      hEx240Pu_chio->Fill(Ex240Pu);
	    if(Telescope==5) hEx240Pu_Tel1_chio->Fill(Ex240Pu);
	    }
	    else
	      {
		hEx240Pu_div->Fill(Ex240Pu);
		if(Telescope==5) hEx240Pu_Tel1_div->Fill(Ex240Pu);
	      }
	    if(Z>0) hZ_Ex240Pu->Fill(Ex240Pu,Z);

	  }
	else if(cutC14->IsInside(Elab,DeltaE)) 
	  {
	    hEx236U->Fill(Ex236U);
	    hEx236U_run->Fill(Ex236U,RunNumber);
	    hEx236U_Tel->Fill(Ex236U,Telescope);
	    if(Mass>0 && Mass<200){ 
	      hMass_Ex236U->Fill(Ex236U,Mass);
	      hEx236U_chio->Fill(Ex236U);
	    }
	    else hEx236U_div->Fill(Ex236U);

	    if(Z>0) hZ_Ex236U->Fill(Ex236U,Z);

	  }
	else if(cutC12->IsInside(Elab,DeltaE)) 
	  {
	    hEx238U->Fill(Ex238U);
	    hEx238U_Tel->Fill(Ex238U,Telescope);
	    if(Mass>0 && Mass<200){ 
	      if(Ex238U>4) hMass_Ex238U->Fill(Ex238U,Mass);
	      hEx238U_chio->Fill(Ex238U);
	    }
	    else
	      {
		hEx238U_div->Fill(Ex238U);
		hEx238U_run[Telescope-1]->Fill(RunNumber,Ex238U);
	      }
	    
	    if(Z>0) hZ_Ex238U->Fill(Ex238U,Z);

	  }
	else if(cut2a->IsInside(Elab,DeltaE)) 
	  {
	    hEx242Pu->Fill(Ex242Pu);
	    hEx242Pu_run->Fill(Ex242Pu,RunNumber);
	    hEx242Pu_Tel->Fill(Ex242Pu,Telescope);

	    if(Mass>0 && Mass<200){
	      // int Zid=FindClosestZ(Chio_E,Chio_DE); //IdentifyZ(Chio_E,Chio_DE);
	      // //	  if(Zid==-1) Zid=FindClosestZ(Chio_E,Chio_DE); 
	      // if(Zid==-1) continue;
	      // double reference=Reference[Zid];
	      // double delta=gSpline3[Zid]->Eval(Chio_E) - gSpline3[Zid]->Eval(10000);

	      // //hMasscorr_Ex242Pu->Fill(Ex242Pu,Chio_DE*10000./fspline->Eval(Chio_E));
	      hMass_Ex242Pu->Fill(Ex242Pu,Mass);//-delta);
	      hEx242Pu_chio->Fill(Ex242Pu);
	    }
	    else hEx242Pu_div->Fill(Ex242Pu);

	    if(Z>0) hZ_Ex242Pu->Fill(Ex242Pu,Z);

	  }

	//	if(cutChio1->IsInside(Chio_E,Chio_DE))
	hChio1_DE_E->Fill(Chio_E,Chio_DE);

      }
    }

  TH1F* hProfile=(TH1F*)hChio1_DE_E->ProfileX();
  // TSpline3();

TFile *OutFile;
OutFile=new TFile(Form("hist/HistoPISTA_run_%d_%d.root",RunNumber,FileNumber), "recreate");

 for(int i=0;i<hEx238U_div->GetNbinsX();i++)
   for(int j=0;j<hNorm_Ex238U->GetNbinsY();j++)
     {
       double Ex=hEx238U_div->GetBinCenter(i);
       double binx=hNorm_Ex238U->GetXaxis()->FindBin(Ex);
       
       double Val=hEx238U_div->GetBinContent(i+1);
       double Err=hEx238U_div->GetBinError(i+1);
       hNorm_Ex238U->SetBinContent(binx,j+1,Val);
       hNorm_Ex238U->SetBinError(binx,j+1,Err);
       
       Val=hEx240Pu_div->GetBinContent(i+1);
       Err=hEx240Pu_div->GetBinError(i+1);
       hNorm_Ex240Pu->SetBinContent(binx,j+1,Val);
       hNorm_Ex240Pu->SetBinError(binx,j+1,Err);
       
       Val=hEx236U_div->GetBinContent(i+1);
       Err=hEx236U_div->GetBinError(i+1);
       hNorm_Ex236U->SetBinContent(binx,j+1,Val);
       hNorm_Ex236U->SetBinError(binx,j+1,Err);
     }

 for(int i=0;i<hEx242Pu_div->GetNbinsX();i++)
   for(int j=0;j<hNorm_Ex242Pu->GetNbinsY();j++)
     {
       double Ex=hEx242Pu_div->GetBinCenter(i);
       double binx=hNorm_Ex242Pu->GetXaxis()->FindBin(Ex);
       
       double Val=hEx242Pu_div->GetBinContent(i+1);
       double Err=hEx242Pu_div->GetBinError(i+1);

       hNorm_Ex242Pu->SetBinContent(binx,j+1,Val);
       hNorm_Ex242Pu->SetBinError(binx,j+1,Err);
     }

 
  TH1F* hEx238U_FB=(TH1F*)hEx238U_chio->Clone("hEx238U_FB");
  TH1F* hEx242Pu_FB=(TH1F*)hEx242Pu_chio->Clone("hEx242Pu_FB");
  TH1F* hEx240Pu_FB=(TH1F*)hEx240Pu_chio->Clone("hEx240Pu_FB");

  hEx238U_FB->Divide(hEx238U);
  hEx240Pu_FB->Divide(hEx240Pu);
  hEx242Pu_FB->Divide(hEx242Pu);
  

  DE_E->Write();

  for(int i=0;i<8;i++)
    {
      DE_E_Mass_Tel[i]->Write();
      hEx238U_run[i]->Write();
    }
  hChio_DE_E->Write();
  hChio1_DE_E->Write();
  hChio_DE_E_corr->Write();
  hChioCorr_Z->Write();
  hMass->Write();
    
  hEx240Pu->Write();
  hEx238U->Write();
  hEx242Pu->Write();
  hEx236U->Write();
  hEx240Pu_Tel1->Write();
  hEx240Pu_chio->Write();
  hEx238U_chio->Write();
  hEx242Pu_chio->Write();
  hEx236U_chio->Write();
  hEx240Pu_Tel1_chio->Write();
  hEx240Pu_div->Write();
  hEx238U_div->Write();
  hEx242Pu_div->Write();
  hEx236U_div->Write();
  hEx240Pu_Tel1_div->Write();
  hEx240Pu_FB->Write();
  hEx238U_FB->Write();
  hEx242Pu_FB->Write();
  hEx240Pu_run->Write();
  //  hEx238U_run->Write();
  hEx242Pu_run->Write();
  hEx236U_run->Write();
  hEx240Pu_Tel->Write();
  hEx238U_Tel->Write();
  hEx242Pu_Tel->Write();
  hEx236U_Tel->Write();

  hMass_Ex240Pu->Write();
  //  hMasscorr_Ex240Pu->Write();
  hMass_Ex238U->Write();
  hMass_Ex242Pu->Write();
  hMass_Ex236U->Write();

  hNorm_Ex240Pu->Write();
  //  hMasscorr_Ex240Pu->Write();
  hNorm_Ex238U->Write();
  hNorm_Ex242Pu->Write();
  hNorm_Ex236U->Write();
  
  OutFile->Close();
}


int IdentifyZ(double Chio_E, double Chio_DE)
{

  if(cut[0]->IsInside(Chio_E,Chio_DE)) return 0;
  else if(cut[1]->IsInside(Chio_E,Chio_DE)) return 1;
  else if(cut[2]->IsInside(Chio_E,Chio_DE)) return 2;
  else if(cut[3]->IsInside(Chio_E,Chio_DE)) return 3;
  else if(cut[4]->IsInside(Chio_E,Chio_DE)) return 4;
  else if(cut[5]->IsInside(Chio_E,Chio_DE)) return 5;
  else if(cut[6]->IsInside(Chio_E,Chio_DE)) return 6;
  else if(cut[7]->IsInside(Chio_E,Chio_DE)) return 7;
  else if(cut[8]->IsInside(Chio_E,Chio_DE)) return 8;
  else if(cut[9]->IsInside(Chio_E,Chio_DE)) return 9;
  else if(cut[10]->IsInside(Chio_E,Chio_DE)) return 10;
  else if(cut[11]->IsInside(Chio_E,Chio_DE)) return 11;
  else if(cut[12]->IsInside(Chio_E,Chio_DE)) return 12;
  else if(cut[13]->IsInside(Chio_E,Chio_DE)) return 13;
  else if(cut[14]->IsInside(Chio_E,Chio_DE)) return 14;
  else if(cut[15]->IsInside(Chio_E,Chio_DE)) return 15;
  else if(cut[16]->IsInside(Chio_E,Chio_DE)) return 16;
  else if(cut[17]->IsInside(Chio_E,Chio_DE)) return 17;
  else if(cut[18]->IsInside(Chio_E,Chio_DE)) return 18;
  else if(cut[19]->IsInside(Chio_E,Chio_DE)) return 19;
  else if(cut[20]->IsInside(Chio_E,Chio_DE)) return 20;
  else if(cut[21]->IsInside(Chio_E,Chio_DE)) return 21;
  else if(cut[22]->IsInside(Chio_E,Chio_DE)) return 22;
  else if(cut[23]->IsInside(Chio_E,Chio_DE)) return 23;
  else if(cut[24]->IsInside(Chio_E,Chio_DE)) return 24;
  else if(cut[25]->IsInside(Chio_E,Chio_DE)) return 25;
  else if(cut[26]->IsInside(Chio_E,Chio_DE)) return 26;
  else if(cut[27]->IsInside(Chio_E,Chio_DE)) return 27;
  else if(cut[28]->IsInside(Chio_E,Chio_DE)) return 28;
  else if(cut[29]->IsInside(Chio_E,Chio_DE)) return 29;
  else if(cut[30]->IsInside(Chio_E,Chio_DE)) return 30;
  else if(cut[31]->IsInside(Chio_E,Chio_DE)) return 31;
  else if(cut[32]->IsInside(Chio_E,Chio_DE)) return 32;
  else if(cut[33]->IsInside(Chio_E,Chio_DE)) return 33;
  else if(cut[34]->IsInside(Chio_E,Chio_DE)) return 34;
  else if(cut[35]->IsInside(Chio_E,Chio_DE)) return 35;
  else if(cut[36]->IsInside(Chio_E,Chio_DE)) return 36;
  else if(cut[37]->IsInside(Chio_E,Chio_DE)) return 37;
  else if(cut[38]->IsInside(Chio_E,Chio_DE)) return 38;
  else return -1;
}

int FindClosestZ(double Chio_E, double Chio_DE)
{
  double DEmin=100000;
  int Zmin=-1;

  for(int i=0;i<39;i++)
  {
    //cout << "i= " << i << endl;
    double test_DE=gSpline3[i]->Eval(Chio_E);
    //cout << test_DE << endl;
    if(fabs(test_DE-Chio_DE)<DEmin && test_DE>1000)
    {
      //cout << "DEmin=" << DEmin << " / test_DE-Chio_DE=" << fabs(test_DE-Chio_DE) << " / test_DE=" << test_DE << " /Chio_DE=" << Chio_DE << endl;
      DEmin=fabs(test_DE-Chio_DE);
      Zmin=i;
    }
  }
  return Zmin;
}
