#include "TCutG.h"
#include "TFile.h"

using namespace std;

void track_hist(int runStart, int runStop){
    
  TChain *chain=new TChain("PhysicsTree");
  for(int run=runStart;run<=runStop;run++){
    chain->Add(Form("/data/e850X/workdir/Pierre_2024/np_e850/root/analysis/run%d.root",run));
  }
  

  //histo
  TH2F* IdCouple = new TH2F("IdCouple","IdCouple",1000,-10,10,1000,-2,2);
  TH2F* De_E = new TH2F("De_E","De_E",500,0,100,500,0,15);

  TFile* fcut2a = new TFile("cut_PISTA_PID.root");
  TCutG* cut2a = (TCutG*)fcut2a->Get("cut_2a");

  double DeltaE,Eres,E1,E2,DE1,DE3;
  int       m_2alpha;
  chain->SetBranchStatus  ("DeltaE",true)   ;
  chain->SetBranchAddress ("DeltaE",&DeltaE);
  chain->SetBranchStatus  ("Eres",true)     ;
  chain->SetBranchAddress ("Eres",&Eres)    ;
  chain->SetBranchStatus  ("E1",true)        ;
  chain->SetBranchAddress ("E1",&E1)          ;
  chain->SetBranchStatus  ("E2",true)        ;
  chain->SetBranchAddress ("E2",&E2)          ;
  chain->SetBranchStatus  ("DE1",true)       ;
  chain->SetBranchAddress ("DE1",&DE1)        ;
  chain->SetBranchStatus  ("DE3",true)       ;
  chain->SetBranchAddress ("DE3",&DE3)        ;
  chain->SetBranchStatus  ("m_2alpha",true)       ;
  chain->SetBranchAddress ("m_2alpha",&m_2alpha)        ;
   

  //Loop on event
  int Nentries=chain->GetEntries();

  for(int e=0;e<Nentries;++e)
    {
      if(e%1000000==0) cout << e*100./Nentries << "% \r" << flush;

      chain->GetEntry(e);
      if ( m_2alpha == 1) {
        if (cut2a->IsInside(Eres,DeltaE)){
         De_E->Fill(Eres,DeltaE);
         IdCouple->Fill(E1-E2,DE1-DE3); }}
    }

  TCanvas *c1=new TCanvas("c1","c1",1000,500);
  c1->Divide(2,1);
  c1->cd(1);
  De_E->GetXaxis()->SetTitle("E");
  De_E->GetYaxis()->SetTitle("De");
  De_E->Draw("colz");
  c1->cd(2);
  IdCouple->GetXaxis()->SetTitle("E1-E2");
  IdCouple->GetYaxis()->SetTitle("DE1-DE3");
  IdCouple->Draw("colz");
//  c1->cd(3);
//  hAoQvsTime->Draw("colz");
  

  c1->SaveAs(Form("hist_theodore/IdCouple%d_%d.root",runStart,runStop));
  //hQvsTime->SaveAs(Form("hist/QvsTime/fQvsTime_run%d-%d.png",runStart,runStop));


  TFile *OutFile;
  OutFile=new TFile(Form("hist_theodore/HistoPISTA_run_%d_%d.root",runStart,runStop), "recreate");
  De_E->Write();
  IdCouple->Write();
  OutFile->Close();

} 



