#include "TICPhysics.h"
#include "TTimeData.h"
#ifdef TICPhysics
#include <TICPhysics.h>
#endif
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <fstream>
#include <vector>
#include <TCutG.h>

using namespace std;

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin , double
        XMax, double YMin, double YMax);

vector<TH2F*>HistoFillerIC(int segment, vector<vector<TSpline3*>> Spline_All_ICprev);

vector<vector<TH2F*>>HistoFillerICcorr(int segment, vector<vector<TSpline3*>> Spline_All_ICprev);

vector<double> TxtToVector(const char *Path);

void HistoDrawer(vector<TH2F*> h ,const char* CharName);

//////////////////////////////////////////////////////////////////////////////////////////
void SplineChioRobust() {

    int NSegment = 5 ;
    double XMin[4] , XMax[4], YMin[4] , YMax[4];
    XMin[0] = -520 ;   XMax[0] = 380 ; YMin[0] = 2000 ; YMax[0] = 9500 ; 
    XMin[1] = -1000  ;   XMax[1] = 1000  ; YMin[1] = 1000 ; YMax[1] = 10000 ; 
    XMin[2] = -520 ;   XMax[2] = 380 ; YMin[2] = 0 ; YMax[2] = 1.3 ; 
    XMin[3] = -1000  ;   XMax[3] = 1000  ; YMin[3] = 0 ; YMax[3] = 1.3 ; 

    //Reinitialising the outfile
    TFile *OutFile = new TFile("Output/HistoP2P.root","recreate");
    OutFile->Close();
    //Initialising with IC1
    int NCallSpline = 0;
    int initseg = 1;

    vector<vector<TSpline3*>> SplineAllIC(NSegment + 1);
    vector<TSpline3*> SplineICurrent(2);

    vector<vector<TH2F*>> hIC(NSegment);
    hIC.at(1)= (HistoFillerIC(1,SplineAllIC));

    SplineICurrent.at(0) = MakeSpline(hIC[initseg][0] , NCallSpline, XMin[0] , XMax[0], YMin[0], YMax[0]);
    SplineICurrent.at(0)->SetName("fspline_IC1_X");
    NCallSpline+=1;

    SplineICurrent.at(1) = MakeSpline(hIC[initseg][1] , NCallSpline, XMin[1] , XMax[1], YMin[1], YMax[1]);
    SplineICurrent.at(1)->SetName("fspline_IC1_Y");
    NCallSpline+=1;

    SplineAllIC.at(1)= (SplineICurrent);  
    SplineAllIC.erase(SplineAllIC.begin() + 1); 
    ///activate this to not make the
    //spline of IC1 


    for (int seg = 0 ; seg < NSegment ; seg++){
        if(seg ==1) {
        }

        else{

            //Load Histo 
            hIC.at(seg) = (HistoFillerIC(seg,SplineAllIC));

            //Reset spline holder
            vector<TSpline3*> SplineICurrent; 

            // X Spline
            SplineICurrent.push_back(MakeSpline(hIC[seg][0] , NCallSpline, XMin[2] , XMax[2], YMin[2], YMax[2]));
            SplineICurrent.at(0)->SetName(Form("fspline_IC%d_X",seg));
            NCallSpline+=1;

            // Y Spline             
            SplineICurrent.push_back(MakeSpline(hIC[seg][1] , NCallSpline, XMin[3] , XMax[3], YMin[3], YMax[3]));
            SplineICurrent.at(1)->SetName(Form("fspline_IC%d_Y",seg));
            NCallSpline+=1;

            //push vector into memory
            SplineAllIC.at(seg)= SplineICurrent;
        }
    }
    //write spline
    TFile* fspline = new TFile("Output/spline_P2P_2024.root", "recreate");
    for (int seg = 0 ; seg < NSegment ; seg++){
        if (!SplineAllIC.at(seg).empty()){
            SplineAllIC.at(seg)[0]->Write();
            SplineAllIC.at(seg)[1]->Write();
        }
    }
    fspline->Close();


    vector<vector<TH2F*>> hIC_Corrall(NSegment);
    vector<TH1F*> hIC_Corr_profile(NSegment);
    vector<TH2F*> hIC_Corr(NSegment);
    hIC_Corrall = HistoFillerICcorr(NSegment,SplineAllIC);
    hIC_Corr = hIC_Corrall.at(1);

    TCanvas *c1 = new TCanvas("c1","c1");
    c1->Divide(NSegment);
    for (int i=0 ; i<NSegment ; i++){
        hIC_Corr_profile.at(i) = (TH1F*)hIC_Corr.at(i)->ProfileX();  
        c1->cd(i+1);
        hIC_Corr_profile.at(i)->Draw();
    }


} // End spline chio XY

///////////////////////////////////////////////////////////////////////////////////////////
/**
 * This function fill the histograms of ICseg/ICcorrSeg-1 expect for IC1 where
 * it give IC1 and ICO where it give IC0/IC1corr
 */
vector<TH2F*> HistoFillerIC(int segment, vector<vector<TSpline3*>> Spline_All_ICprev) {

    // Input and setters
    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");

    TICPhysics* IC = new TICPhysics(); 
    TTimeData *Time = new TTimeData();

    double FF_IC_X, FF_IC_Y;

    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);

    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC);

    chain->SetBranchStatus("Time", true);
    chain->SetBranchAddress("Time", &Time);

    vector<double> Toff13 , Toff14, Toff23, Toff24;
    const char* Path13 = "../../mwpc/Toff/output/Toff13.txt";
    const char* Path14 = "../../mwpc/Toff/output/Toff14.txt";
    const char* Path23 = "../../mwpc/Toff/output/Toff23.txt";
    const char* Path24 = "../../mwpc/Toff/output/Toff24.txt";

    Toff13 = TxtToVector(Path13);
    Toff14 = TxtToVector(Path14);
    Toff23 = TxtToVector(Path23);
    Toff24 = TxtToVector(Path24);

    // Get the number of previous spline to load
    int NSpline = abs(segment-1);

    //Filling all  previous vector spline
    vector<TSpline3*> SplineIC_X(NSpline+1);
    vector<TSpline3*> SplineIC_Y(NSpline+1);

    //Fill all the previous spline , if the segment is 0 it'll fill spline 1 only 
    for (int pseg = 1 ; pseg<=NSpline; pseg++){
        if(!Spline_All_ICprev.at(pseg).empty()){
            SplineIC_X.at(pseg) = Spline_All_ICprev.at(pseg).at(0);
            SplineIC_Y.at(pseg) = Spline_All_ICprev.at(pseg).at(1);
        }
    }

    // Histograms
    TH2F *hIC_X , *hIC_Y;

    if (segment == 1 ) { 
        hIC_Y = new TH2F(Form("hChio_IC%d_Y",segment),
                Form("hChio_IC%d_Y",segment), 1000, -1000, 1000, 1000, 1000, 10000);
        hIC_X = new TH2F(Form("hChio_IC%d_X",segment),
                Form("hChio_IC%d_X",segment), 1000, -800, 800, 1000, 1000, 10000);

    }

    else {
        hIC_Y = new TH2F(Form("hChio_IC%d_IC%d_Y",segment,segment-1),
                Form("hChio_IC%d_IC%d_Y",segment,segment-1), 1000, -1000, 1000, 500, 0, 2);
        hIC_X = new TH2F(Form("hChio_IC%d_IC%d_X",segment,segment-1),
                Form("hChio_IC%d_IC%d_X",segment,segment-1), 1000, -800, 800, 500, 0, 2);
    }


    //===========================================================================================================
    //																	Beginning loop on entries
    //===========================================================================================================

    int Nentries = chain->GetEntries();
    //int Nentries = 1000000;
    auto start = std::chrono::high_resolution_clock::now();
    for (int e = 0; e < Nentries; e++) {

        chain->GetEntry(e);


        vector<double> IC_Spline_CorrX(NSpline + 1),IC_Spline_CorrY(NSpline+1);

        if (Time->GetMWPC13Mult() ==1  && IC->fIC_TS.size()>= (segment+1)){ //only mult 1 event

            UShort_t FPMW_Section = Time->GetSection_MWPC3(0);


                FF_IC_Y = 10* (IC->fIC_TS.at(segment) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))) ;
                //Get the corrected value for all previous spline
                for (int pseg = 1 ; pseg<=NSpline ; pseg++){
                    //If the spline doesnt exist define default value
                    if(Spline_All_ICprev.at(pseg).empty()){
                        IC_Spline_CorrX.at(pseg)= IC->fIC_raw[pseg];
                        IC_Spline_CorrY.at(pseg)= IC->fIC_raw[pseg]; 
                    }

                    //If it exist correct the current line
                    else if (!Spline_All_ICprev.at(pseg).empty()){
                        IC_Spline_CorrX.at(pseg) = IC->fIC_raw[pseg] * SplineIC_X.at(pseg)->Eval(0) / SplineIC_X.at(pseg)->Eval(FF_IC_X); 
                        IC_Spline_CorrY.at(pseg) = IC->fIC_raw[pseg] * SplineIC_Y.at(pseg)->Eval(0) / SplineIC_Y.at(pseg)->Eval(FF_IC_Y); 
                    } //End if non empty

                } // End loop IC non 0


                //===========================================================================================================
                //                               Fill histo

                //===========================================================================================================/

                if (segment == 1){
                    hIC_Y->Fill(FF_IC_Y,IC->fIC_raw[segment]);
                    hIC_X->Fill(FF_IC_X,IC->fIC_raw[segment]);
                }

                else if (segment == 0){
                    hIC_Y->Fill(FF_IC_Y,IC->fIC_raw[segment]/IC_Spline_CorrY.at(1));
                    hIC_X->Fill(FF_IC_X,IC->fIC_raw[segment]/IC_Spline_CorrX.at(1));
                }


                else {
                    hIC_Y->Fill(FF_IC_Y,IC->fIC_raw[segment]/IC_Spline_CorrY.at(segment-1));
                    hIC_X->Fill(FF_IC_X,IC->fIC_raw[segment]/IC_Spline_CorrX.at(segment-1));    
                }
        } //end only m2

        // Estimate time left every `updateInterval` iterations
        if (e % 100000  == 0 && e > 0) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "Treating segment : " << segment 
                << "  | Estimated time left: " << int(timeLeft) << " seconds" << "\r" << flush;
        }

    } // end loop on entries

    //===========================================================================================================
    //                                       Output 
    //===========================================================================================================
    vector<TH2F*> hIC;
    hIC.push_back(hIC_X);
    hIC.push_back(hIC_Y);

    TFile *OutFile = new TFile("Output/HistoP2P.root","UPDATE");
    hIC_X->Write();
    hIC_Y->Write();
    OutFile->Close();

    return hIC;
} // End HistoFiller

vector<vector<TH2F*>>HistoFillerICcorr(int segment, vector<vector<TSpline3*>> Spline_All_ICprev){

    // Input and setters
    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");

    TTimeData *Time = new TTimeData();
    TICPhysics* IC = new TICPhysics() ; 
    double FF_IC_X, FF_IC_Y;

    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC);
    chain->SetBranchStatus("Time", true);
    chain->SetBranchAddress("Time", &Time);

    vector<double> Toff13 , Toff14, Toff23, Toff24;
    const char* Path13 = "../../mwpc/Toff/output/Toff13.txt";
    const char* Path14 = "../../mwpc/Toff/output/Toff14.txt";
    const char* Path23 = "../../mwpc/Toff/output/Toff23.txt";
    const char* Path24 = "../../mwpc/Toff/output/Toff24.txt";

    Toff13 = TxtToVector(Path13);
    Toff14 = TxtToVector(Path14);
    Toff23 = TxtToVector(Path23);
    Toff24 = TxtToVector(Path24);

    // Get the number of previous spline to load
    segment = segment -1;
    int NSpline = abs(segment);

    //Filling all  previous vector spline
    vector<TSpline3*> SplineIC_X(NSpline+1);
    vector<TSpline3*> SplineIC_Y(NSpline+1);

    //Fill all the previous spline , if the segment is 0 it'll fill spline 1 only 
    for (int pseg = 0 ; pseg<=NSpline; pseg++){
        if(!Spline_All_ICprev.at(pseg).empty()){
            SplineIC_X.at(pseg) = Spline_All_ICprev.at(pseg).at(0);
            SplineIC_Y.at(pseg) = Spline_All_ICprev.at(pseg).at(1);
        }
    }

    // Histograms
    vector<TH2F*> hIC_X(segment +1) , hIC_Y(segment+1);

    for (int pseg = 0 ; pseg<=NSpline; pseg++){
        if (pseg == 1 ) { 
            hIC_Y.at(pseg) = new TH2F(Form("hChiocorr_IC%d_Y",pseg),
                    Form("hChiocorr_IC%d_Y",pseg), 1000, -1000, 1000, 1000, 0, 20000);
            hIC_X.at(pseg) = new TH2F(Form("hChiocorr_IC%d_X",pseg),
                    Form("hChiocorr_IC%d_X",pseg), 1000, -800, 800, 1000, 0, 20000);

        }

        else {
            hIC_Y.at(pseg) = new TH2F(Form("hChiocorr_IC%d_IC%d_Y",pseg,pseg-1),
                    Form("hChiocorr_IC%d_IC%d_Y",pseg,pseg-1), 1000, -1000, 1000, 500, 0, 2);
            hIC_X.at(pseg) = new TH2F(Form("hChiocorr_IC%d_IC%d_X",pseg,pseg-1),
                    Form("hChiocorr_IC%d_IC%d_X",pseg,pseg-1), 1000, -800, 800, 500, 0, 2);
        }
    }

    //===========================================================================================================
    //																	Beginning loop on entries
    //===========================================================================================================

    int Nentries = chain->GetEntries();
    //int Nentries = 1000000;
    auto start = std::chrono::high_resolution_clock::now();
    for (int e = 0; e < Nentries; e++) {

        chain->GetEntry(e);

        vector<double> IC_Spline_CorrX(NSpline + 1),IC_Spline_CorrY(NSpline+1);


        if (Time->GetMWPC13Mult() ==1  && IC->fIC_TS.size()>= (segment+1)){ //only mult 1 event

            UShort_t FPMW_Section = Time->GetSection_MWPC3(0);         

                FF_IC_Y = 10* (IC->fIC_TS.at(segment) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))) ;
                //Get the corrected value for all previous spline
                for (int pseg = 0 ; pseg<=NSpline ; pseg++){
                    //If the spline doesnt exist define default value
                    if(Spline_All_ICprev.at(pseg).empty()){
                        IC_Spline_CorrX.at(pseg)= IC->fIC_raw[pseg];
                        IC_Spline_CorrY.at(pseg)= IC->fIC_raw[pseg];
                        //cout << pseg << endl;
                    }

                    //If it exist correct the current line
                    else if (!Spline_All_ICprev.at(pseg).empty()){
                        IC_Spline_CorrX.at(pseg) = IC->fIC_raw[pseg] * SplineIC_X.at(pseg)->Eval(0) / SplineIC_X.at(pseg)->Eval(FF_IC_X); 
                        IC_Spline_CorrY.at(pseg) = IC->fIC_raw[pseg] * SplineIC_Y.at(pseg)->Eval(0) / SplineIC_Y.at(pseg)->Eval(FF_IC_Y); 
                    } //End if non empty
                } // end loop

                for (int pseg = 0 ; pseg<=NSpline ; pseg++){
                    if (pseg == 1){
                        hIC_Y.at(pseg)->Fill(FF_IC_Y,IC_Spline_CorrY.at(pseg));
                        hIC_X.at(pseg)->Fill(FF_IC_X,IC_Spline_CorrY.at(pseg));
                    }

                    else if (pseg == 0){
                        hIC_Y.at(pseg)->Fill(FF_IC_Y,IC_Spline_CorrY.at(pseg)/IC_Spline_CorrY.at(1));
                        hIC_X.at(pseg)->Fill(FF_IC_X,IC_Spline_CorrY.at(pseg)/IC_Spline_CorrX.at(1));
                    }


                    else {
                        hIC_Y.at(pseg)->Fill(FF_IC_Y,IC_Spline_CorrY.at(pseg)/IC_Spline_CorrY.at(pseg-1));
                        hIC_X.at(pseg)->Fill(FF_IC_X,IC_Spline_CorrY.at(pseg)/IC_Spline_CorrX.at(pseg-1));    
                    }

                }
                //===========================================================================================================
                //                               Fill histo
                //===========================================================================================================/

                // Estimate time left every `updateInterval` iterations
                if (e % 100000  == 0 && e > 0) {
                    auto now = std::chrono::high_resolution_clock::now();
                    std::chrono::duration<double> elapsed = now - start;
                    double avgTimePerIteration = elapsed.count() / e;
                    double timeLeft = avgTimePerIteration * (Nentries - e);

                    std::cout << "Treating segment : " << segment 
                        << "  | Estimated time left: " << timeLeft << " seconds" << "\r" << flush;
                }
            } // end loop on entries
    } //end only m2
      //===========================================================================================================
      //                                       Output 
      //===========================================================================================================
    vector<vector<TH2F*>> hIC;
    hIC.push_back(hIC_X);
    hIC.push_back(hIC_Y);

    return hIC;
} // End HistoFiller


/**
  }
 *  Create the spline for one TH2F
 *
 *  Input :
 *     Pointer to a TH2F
 *
 *
 *  Output:
 *
 *     TSpline3 : the output spline
 *
 */

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin = 0 , double
        XMax = 1000 , double YMin=0 , double YMax = 100) {

    TSpline3* gspline;

    TH1F* hProfile;
    TH1F* pfx; // extended hProfile

    TCanvas* canfit = new TCanvas(Form("CanSpline_%02d",NCallee),
            Form("canfit_%02d",NCallee), 0, 0, 1000, 500);


    //===========================================================================================================
    //                                            Init profile
    //===========================================================================================================

    // Create the TProfile
    hProfile = (TH1F*)hInput->ProfileX();
    hProfile->SetLineWidth(2);
    hProfile->SetDirectory(0);
    canfit->cd();

    hProfile->GetYaxis()->SetRangeUser(YMin, YMax);
    hProfile->Draw();
    //===========================================================================================================
    //                                          First and last bin to get to 6k
    //                                          event get promoted
    //===========================================================================================================
    int FirstBin, LastBin;
    double Treshold = 1;
    for (int bin =1 ; bin<hProfile->GetNbinsX(); bin++) {
        FirstBin = bin;
        if (hProfile->GetBinLowEdge(bin)> XMin) break;
    }
    for (int bin = hProfile->GetNbinsX(); bin>1 ; bin--) {
        LastBin = bin;
        if (hProfile->GetBinLowEdge(bin)< XMax) break;
    }
    //===========================================================================================================
    //                                          Init Extended profile function
    //===========================================================================================================
    // Create the extended TProfile
    pfx = new TH1F(
            Form("pfx_%02d", Int_t(NCallee/2)), Form("pfx_%02d", Int_t(NCallee/2)), hProfile->GetNbinsX(), hProfile->GetBinLowEdge(1),
            hProfile->GetBinLowEdge(hProfile->GetNbinsX()) + hProfile->GetBinWidth(hProfile->GetNbinsX()));
    pfx->SetLineColor(8);
    pfx->SetDirectory(0);
    //===========================================================================================================
    //                                           Fill extended profile
    //===========================================================================================================

    // fill the extended TProfile
    float newval, lastval, lasterr;
    for (int bin = 1; bin <= FirstBin; bin++) {
        newval = 0;
        pfx->SetBinContent(bin, newval);
    }


    for (int bin = FirstBin; bin <= LastBin; bin++) {
        newval = hProfile->GetBinContent(bin);
        if (newval != 0) {
            pfx->SetBinContent(bin, newval);
            pfx->SetBinError(bin, hProfile->GetBinError(bin));
            lastval = newval;
            lasterr = hProfile->GetBinError(bin);
        }
        else {
            pfx->SetBinContent(bin, lastval);
            pfx->SetBinError(bin, lasterr);
        }
    }
    pfx->Draw("same");

    gspline = new TSpline3(pfx);
    gspline->SetName(Form("fspline_%d", NCallee + 1));

    return gspline;

} // end makespline

void HistoDrawer(vector<TH2F*> h,const char* CharName){

    int NHisto = static_cast<int>(h.size());

    TCanvas* Can= new TCanvas(CharName,CharName,0,0,2000,1000);

    Can->Divide(NHisto);

    for ( int i=0 ; i < NHisto ; i+=1 ){
        //All x are even and all y are odd
        int CanvaNumber = (i) ;
        Can->cd(CanvaNumber);
        h[i]->Draw("colz");
    }
}


vector<double> TxtToVector(const char *Path){
    string line;
    vector<double> values;
    ifstream file(Path); 

    if (file.is_open()) {
        while (std::getline(file, line)) {
            try {
                values.push_back(std::stod(line));
            } catch (const std::invalid_argument& e) {
                std::cerr << "Invalid number in line: " << line << '\n';
            }
        }
        file.close();
    } else {
        std::cerr << "Error opening file.\n";
    }

    return values;
}
