#include <TICPhysics.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <TCutG.h>
#include <fstream>
#include <vector>

using namespace std;

int GetNumberKey(TFile *infile ,const char* ClassName);

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin , double
    XMax, double YMin, double YMax);
//////////////////////////////////////////////////////////////////////////////////////////
void DECorr(bool cut = false, bool spline = false) {
    //===========================================================================================================
    //                           Loading var
    //===========================================================================================================
    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");

    TICPhysics* IC = new TICPhysics() ;
    double FF_IC_X, FF_IC_Y, FF_V13;

    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
    chain->SetBranchStatus("FF_IC_Y", true);
    chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);
    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC); 
    chain->SetBranchStatus("FF_V13", true);
    chain->SetBranchAddress("FF_V13", &FF_V13); 

    TCutG *CutZ;
    TCutG *CutZbis;
    TFile *fCut;
    if (cut) {
        fCut =  TFile::Open("Output/CutDeCorr.root");
        CutZbis = (TCutG*)fCut->Get("CutZbis") ;
        CutZ = (TCutG*)fCut->Get("CutZ") ;
    }

    TFile *fSpline, *fSplineIC; 
    TSpline3 *splineDEcorr, *splineDEbis;
    fSpline= new TFile("Output/splineDE.root","open");
    splineDEcorr = (TSpline3*) fSpline->Get("SplineDe_0");
    splineDEbis = (TSpline3*) fSpline->Get("SplineDebis");
    
    vector<TSpline3*> vsplineDE;
    // Get number of spline
    int SplineCountDe = 0 ;
    TIter next1(fSpline->GetListOfKeys());
    TKey* key1;

    while ((key1=(TKey*)next1())){
        if (std::string(key1->GetClassName()) == "TSpline3"){
            SplineCountDe ++;
        }
    }


    //Fill spline vector
    for (int i =0 ; i<SplineCountDe ; i++){
        vsplineDE.push_back((TSpline3*) fSpline->Get(Form("SplineDe_%d",i)));
    }

    //empty backup one
    for (int i =0 ; i<SplineCountDe ; i++){
        if (vsplineDE[i]==nullptr) vsplineDE.erase(vsplineDE.begin()+i);
    }


    fSplineIC = new TFile("Output/spline_P2P_2024.root","open"); 
    // Get number of spline
    int SplineCount = 0 ;
    TIter next(fSplineIC->GetListOfKeys());
    TKey* key;

    while ((key=(TKey*)next())){
        if (std::string(key->GetClassName()) == "TSpline3"){
            SplineCount ++;
        }
    }


    vector<TSpline3*> spline_X(11), spline_Y(11);
    for (int i = 0; i < SplineCount  ; i++) {
        int seg = int((i-2)/2 +1 );
        if (i < 2){
            spline_X.at(0) = (TSpline3 *)fSplineIC->Get("fspline_IC0_X");
            spline_Y.at(0) = (TSpline3 *)fSplineIC->Get("fspline_IC0_Y");
        }

        else if ( i>=2 && i<4){ 
            spline_X.at(1) = (TSpline3 *)fSplineIC->Get("fspline_IC1_X");
            spline_Y.at(1) = (TSpline3 *)fSplineIC->Get("fspline_IC1_Y");
        }
        else if (seg >=2 && (i%2 == 0) ) {
            spline_X.at(seg) = (TSpline3 *)fSplineIC->Get(Form("fspline_IC%d_X",seg));
        }
        else if (seg >=2 && !(i%2 == 0) ) {	
            spline_Y.at(seg) = (TSpline3 *)fSplineIC->Get(Form("fspline_IC%d_Y",seg));
        }
    } //End loop on histogram

    //Defining output histo
    TH2F* hDE_E = new TH2F("DE_E","DE_E",1000,0,22000,1000,0,28000);
    TH2F* hDE_E_splined = new TH2F("DE_E_splined","DE_E_splined",1000,0,22000,1000,0,28000);
    TH2F* hDE_Ebis = new TH2F("DE_Ebis","DE_Ebis",1000,0,22000,1000,0,24000);


    TH2F* hDE_Y = new TH2F("DE_Y","DE_Y",1000,-100,100,1000,0,22000);
    TH2F* hDE_Y_splined = new TH2F("DE_Ysplined","DE_Ysplined",1000,-100,100,1000,0,22000);
    TH2F* hDE_Y_splined_nocut = new TH2F("DE_Ysplined_nocut","DE_Ysplined_nocut",1000,-100,100,1000,0,22000);
    TH2F* hIC_Y = new TH2F("IC_Y","IC_Y",1000,-100,100,1000,0,22000);
    TH2F* hIC_Y_splined = new TH2F("IC_Ysplined","IC_Ysplined",1000,-100,100,1000,0,22000);


    TH2F* hDE_Y_corr = new TH2F("DE_Ycorr","DE_Ycorr",1000,-100,100,1000,0,22000);
    TH2F* hDE_Y_corr_nocut = new TH2F("DE_Ycorr_nocut","DE_Ycorr_nocut",1000,-100,100,1000,0,22000);

    TH2F* hDE_Y_bis = new TH2F("DE_Ybis","DE_Ybis",1000,-100,100,1000,0,22000);
    TH2F* hDEbis_Y_corr = new TH2F("DE_Ybiscorr","DE_Ybiscorr",1000,-100,100,1000,0,22000);

    TH2F* hDE_E_corr = new TH2F("DE_Ecorr","DE_Ecorr",1000,0,22000,1000,0,28000);
    TH2F* hDE_Ebis_corr = new TH2F("DE_Ebiscorr","DE_Ebiscorr",1000,0,22000,1000,0,24000);
    TH2F* hDE_E_Recur = new TH2F("DE_E_Recur","DE_E_Recur",1000,0,22000,1000,0,28000);

    TH2F* hDE_V = new TH2F("DE_V","DE_V",1000,0,5,1000,0,28000);

    // Def Lim spline
    double Ymin = -100 , Ymax =100;
    //===========================================================================================================
    //                                        Event Loop
    //===========================================================================================================
    int Nentries = chain->GetEntries();
    //int Nentries = 10000000;
    auto start = std::chrono::high_resolution_clock::now();
    int NSegment = 5 ;

    for (int e = 0; e < Nentries; e++) {
        if (e % 100000  == 0 && e > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }

        chain->GetEntry(e);

        vector<double> ICcorr_Y(NSegment), ICcorr_X(NSegment); // the [0] element of spline is the
        vector<double> Temp_X(NSegment) , Temp_Y(NSegment);    // correction on the first
                                                               // segment of ic
        for (int seg = 1; seg < NSegment+1 ; seg++) {

            if (seg == NSegment) seg = 0; //from 1to NSeg finishing with 0

            if (spline_Y.at(seg)==0){
                ICcorr_Y.at(seg) = IC->fIC_raw[seg]; 
            }

            else {
                if (seg == 0) {
                    Temp_Y.at(seg) = ICcorr_Y.at(1) / IC->fIC_raw[seg] * spline_Y.at(seg)->Eval(0)/ spline_Y.at(seg)->Eval(FF_IC_Y);
                    ICcorr_Y.at(seg) = ICcorr_Y.at(1) / Temp_Y.at(seg);
                }

                else if (seg == 1){ 
                    ICcorr_Y.at(seg) = IC->fIC_raw[1]*spline_Y.at(1)->Eval(0)/spline_Y.at(1)->Eval(FF_IC_Y);
                }

                else if (seg > 1) {
                    Temp_Y.at(seg) =  IC->fIC_raw[seg]/ICcorr_Y.at(seg-1) * spline_Y.at(seg)->Eval(0)/ spline_Y.at(seg)->Eval(FF_IC_Y);
                    ICcorr_Y.at(seg) = ICcorr_Y.at(seg-1) * Temp_Y.at(seg);
                }

                if (!(ICcorr_Y.at(seg)==ICcorr_Y.at(seg))) ICcorr_Y.at(seg) = 0;
            } //end if non empty
            if (seg == 0) break;
        }//endloop seg


        double DE = 0 , DE_splined=0 ,DE_corr=0, DE_Ybis=0, DE_Y_Recur=0;
        double E =0, Ecorr=0; 
        for (int seg = 0 ;  seg < sizeof(IC->fIC_raw)/sizeof(IC->fIC_raw[0]) ; seg++ ){
            if (seg >4){
                E += IC->fIC_raw[seg] ;
                //Ecorr += ICcorr_Y.at(seg) ;
            }
        }



        // ********************* DE setter *********************
        double IC1corr = (IC->fIC_raw[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
        double DE_Bis = 0.5*(IC1corr+IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

        DE = 0.5*(IC->fIC_raw[0]+ IC->fIC_raw[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];
        DE_splined = 0.5*(ICcorr_Y[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];
        //DE_splined = 0.5*( ICcorr_Y[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];
        

        //************* Cut ******************
        bool CutV = abs( FF_V13-3.94)<0.01;
        bool CutVbis =  abs( FF_V13-3.94)<0.01;
        
        //bool CutV = true;
        //bool CutVbis =  true;
       
        //***********Fill histo*************

        if (FF_IC_Y >Ymin && FF_IC_Y <Ymax ){
            
            //*************Init DE_E***************** 
            hDE_E->Fill(E,DE);
            hDE_E_splined->Fill(E,DE_splined);
            hDE_Ebis->Fill(E,DE_Bis);

            hDE_V->Fill(FF_V13,DE);    

            //******* Coor DE ************
            if (spline== true  ){     
                DE_corr = DE_splined * splineDEcorr->Eval(0) / splineDEcorr->Eval(FF_IC_Y);
                hDE_E_corr->Fill(E,DE_corr);

                DE_Ybis = DE_Bis * splineDEbis->Eval(0) / splineDEbis->Eval(FF_IC_Y);
                hDE_Ebis_corr->Fill(E,DE_Ybis); 

                DE_Y_Recur = DE_splined;

                for (int i=0; i<vsplineDE.size() ; i++){
                    DE_Y_Recur = DE_Y_Recur * vsplineDE.at(i)->Eval(0) / vsplineDE.at(i)->Eval(FF_IC_Y);
                }
                hDE_E_Recur->Fill(E,DE_Y_Recur);    
            }   
                
            else {
                DE_corr = DE_splined;
                hDE_E_corr->Fill(E,DE_corr);

                DE_Ybis = DE_Bis ;
                hDE_Ebis_corr->Fill(E,DE_Ybis);  
            }

            bool CutCharge;
            if (cut == true) {
                CutCharge   =  CutZ->IsInside(FF_V13,DE);
            }
            //bool CutChargeBis =  CutZ->IsInside(E,DE_Y_Recur);
            //bool CutVbis =  CutZbis->IsInside(E,DE_Bis);
            // DE_Y in a cut
            
            hDE_Y_corr_nocut->Fill(FF_IC_Y,DE);
            hDE_Y_splined_nocut->Fill(FF_IC_Y,DE_splined);
            if (cut == true ) {
                if (CutV && CutCharge){ 
                    hDE_Y->Fill(FF_IC_Y,DE);
                    hDE_Y_splined->Fill(FF_IC_Y,DE_splined); 
                    hDE_Y_corr->Fill(FF_IC_Y,DE_corr);

                    hIC_Y->Fill(FF_IC_Y,IC->fIC_raw[1]);
                    hIC_Y_splined->Fill(FF_IC_Y,ICcorr_Y.at(1));
                }
            }//end cut
            else {
                hDE_Y->Fill(FF_IC_Y,DE);
                hDE_Y_splined->Fill(FF_IC_Y,DE_splined); 
                hDE_Y_corr->Fill(FF_IC_Y,DE_corr);

                hIC_Y->Fill(FF_IC_Y,IC->fIC_raw[1]);
                hIC_Y_splined->Fill(FF_IC_Y,ICcorr_Y.at(1));}

            if (cut == true && CutVbis) {
                hDE_Y_bis->Fill(FF_IC_Y,DE_Bis);
                hDEbis_Y_corr->Fill(FF_IC_Y,DE_Ybis);
            }
            else{;}
            //end cut
        }//end cond Y
    } //endl loop event

    // making spline

    TSpline3 *splineDE, *osplineDEbis;
    if (spline == true){
        splineDE = MakeSpline(hDE_Y_splined,0,Ymin,Ymax,0,23000); 
        osplineDEbis = MakeSpline(hDE_Y_bis,1,Ymin,Ymax,0,23000); 
        TFile *fSpline = new TFile("Output/splineDE.root","recreate");
        splineDE->SetName("SplineDe_0");
        splineDE->Write();
        osplineDEbis->SetName("SplineDebis");
        osplineDEbis->Write(); 
        fSpline->Close();
    }
    //===========================================================================================================
    //                                          Drawing histo
    //===========================================================================================================

    TCanvas* c = new TCanvas("c","c",1500,1000);
    c->Divide(2,2);
    c->cd(1);
    gPad->SetLogz();
    hDE_Y->ProfileX()->Draw();
    c->cd(2);
    gPad->SetLogz();
    hDE_Y_splined->ProfileX()->Draw();
    c->cd(3);
    gPad->SetLogz();
    hIC_Y->ProfileX()->Draw();
    c->cd(4);
    gPad->SetLogz();
    hIC_Y_splined->ProfileX()->Draw();

    TCanvas *c4 = new TCanvas("c4","c4",1500,1000);
    c4->Divide(2);
    c4->cd(1);
    hDE_Y_splined_nocut->ProfileX()->Draw();
    c4->cd(2);
    hDE_Y_corr_nocut->ProfileX()->Draw();



    TCanvas *c1 = new TCanvas("c1","c1",1500,1000);
    hDE_V->Draw();

    TCanvas* c2 = new TCanvas("c2","c2",1500,1000);
    c2->Divide(2 + 3*spline);
    c2->cd(1);
    gPad->SetLogz();
    hDE_E->Draw();
    c2->cd(2);
    gPad->SetLogz();
    hDE_Ebis->Draw();
    if (spline){
        c2->cd(3);
        gPad->SetLogz();
        hDE_E_splined->Draw();
        c2->cd(4);
        gPad->SetLogz();
        hDE_E_corr->Draw();   
        c2->cd(5);
        gPad->SetLogz();
        hDE_Ebis_corr->Draw();
    }
} // End spline chio XY


int GetNumberKey(TFile *infile ,const char* ClassName){
    // Get number of spline
    int KeyCount = 0 ;
    TIter next(infile->GetListOfKeys());
    TKey* keyFit;

    while ((keyFit=(TKey*)next())){
        if (std::string(keyFit->GetClassName()) == ClassName){
            KeyCount ++;
        }
    }

    return KeyCount;

}

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin = 0 , double
        XMax = 1000 , double YMin=0 , double YMax = 100) {

    TSpline3* gspline;

    TH1F* hProfile;
    TH1F* pfx; // extended hProfile

    TCanvas* canfit = new TCanvas(Form("CanSpline_%02d",NCallee),
            Form("canfit_%02d",NCallee), 0, 0, 1000, 500);


    //===========================================================================================================
    //                                            Init profile
    //===========================================================================================================

    // Create the TProfile
    hProfile = (TH1F*)hInput->ProfileX();
    hProfile->SetLineWidth(2);
    hProfile->SetDirectory(0);
    canfit->cd();

    hProfile->GetYaxis()->SetRangeUser(YMin, YMax);
    hProfile->Draw();
    //===========================================================================================================
    //                                          First and last bin to get to 6k
    //                                          event get promoted
    //===========================================================================================================
    int FirstBin, LastBin;
    double Treshold = 0;
    for (int bin =1 ; bin<hProfile->GetNbinsX(); bin++) {
        FirstBin = bin;
        if (hProfile->GetBinLowEdge(bin)> XMin) break;
    }
    for (int bin = hProfile->GetNbinsX(); bin>1 ; bin--) {
        LastBin = bin;
        if (hProfile->GetBinLowEdge(bin)< XMax) break;
    }
    //===========================================================================================================
    //                                          Init Extended profile function
    //===========================================================================================================
    // Create the extended TProfile
    pfx = new TH1F(
            Form("pfx_%02d", Int_t(NCallee/2)), Form("pfx_%02d", Int_t(NCallee/2)), hProfile->GetNbinsX(), hProfile->GetBinLowEdge(1),
            hProfile->GetBinLowEdge(hProfile->GetNbinsX()) + hProfile->GetBinWidth(hProfile->GetNbinsX()));
    pfx->SetLineColor(8);
    pfx->SetDirectory(0);
    //===========================================================================================================
    //                                           Fill extended profile
    //===========================================================================================================

    // fill the extended TProfile
    float newval, lastval, lasterr;
    for (int bin = 1; bin <= FirstBin; bin++) {
        newval = 0;
        pfx->SetBinContent(bin, newval);
    }


    for (int bin = FirstBin; bin <= LastBin; bin++) {
        newval = hProfile->GetBinContent(bin);
        if (newval != 0) {
            pfx->SetBinContent(bin, newval);
            pfx->SetBinError(bin, hProfile->GetBinError(bin));
            lastval = newval;
            lasterr = hProfile->GetBinError(bin);
        }
        else {
            pfx->SetBinContent(bin, lastval);
            pfx->SetBinError(bin, lasterr);
        }
    }
    pfx->Draw("same");

    gspline = new TSpline3(pfx);
    gspline->SetName(Form("fspline_%d", NCallee + 1));

    return gspline;

} // end makespline

