//#include <TICPhysics.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <TCutG.h>
#include <fstream>
#include <vector>

using namespace std;

int GetNumberKey(TFile *infile ,const char* ClassName);

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin , double
    XMax, double YMin, double YMax);
//////////////////////////////////////////////////////////////////////////////////////////
void DECorrParra(bool cut = false, bool spline = false) {
  //===========================================================================================================
  //                           Loading var
  //===========================================================================================================
  TChain* chain = new TChain("PhysicsTree");
  chain->Add("../../../root/analysis/VamosCalib241.root");

  TICPhysics* IC = new TICPhysics() ;
  double FF_IC_X, FF_IC_Y;

  chain->SetBranchStatus("FF_IC_X", true);
  chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
  chain->SetBranchStatus("FF_IC_Y", true);
  chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);
  chain->SetBranchStatus("IC", true);
  chain->SetBranchAddress("IC", &IC); 

 TCutG *CutZ;
 TCutG *CutZbis;
 TFile *fCut; 
  if (cut) {
    fCut =  TFile::Open("Output/CutDeCorr.root");
    CutZbis = (TCutG*)fCut->Get("CutZbis") ;
    CutZ = (TCutG*)fCut->Get("CutZbis") ;
  }

  TFile *fSpline; 
  TSpline3 *splineDEcorr, *splineDEbis;
  if(spline){
    fSpline= new TFile("Output/splineDE.root","open");
    splineDEcorr = (TSpline3*) fSpline->Get("SplineDe");
    splineDEbis = (TSpline3*) fSpline->Get("SplineDebis");
  }

  //Defining output histo
  TH2F* hDE_E = new TH2F("DE_E","DE_E",1000,100,1,1000,100,1);
  TH2F* hDE_Ebis = new TH2F("DE_Ebis","DE_Ebis",1000,100,1,1000,100,1);
  TH2F* hDE_Y = new TH2F("DE_Y","DE_Y",1000,-100,100,1000,0,22000);
  TH2F* hDE_Y_bis = new TH2F("DE_Ybis","DE_Ybis",1000,-100,100,1000,0,22000);
  TH2F* hDE_E_corr = new TH2F("DE_Ecorr","DE_Ecorr",1000,100,1,1000,0,22000);
  TH2F* hDE_Ebis_corr = new TH2F("DE_Ebiscorr","DE_Ebiscorr",1000,100,1,1000,0,22000);


  //===========================================================================================================
  //                                        Event Loop
  //===========================================================================================================
  int Nentries = chain->GetEntries();
  //int Nentries = 1000000;
  auto start = std::chrono::high_resolution_clock::now();
  for (int e = 0; e < Nentries; e++) {

    if (e % 100000  == 0 && e > 0 ) {
      auto now = std::chrono::high_resolution_clock::now();
      std::chrono::duration<double> elapsed = now - start;
      double avgTimePerIteration = elapsed.count() / e;
      double timeLeft = avgTimePerIteration * (Nentries - e);

      std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
    }

    chain->GetEntry(e);


    double DE = 0 ,E =0 ,DE_Ycorr=0, DE_Ybis=0;

    for (int seg = 0 ;  seg < sizeof(IC->fIC_raw)/sizeof(IC->fIC_raw[0]) ; seg++ ){
      if (seg >4){
        E += IC->fIC_raw[seg] ;
        //Ecorr += ICcorr_Y.at(seg) ;
      }
    }

    double IC1corr = (IC->fIC_raw[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
    double DE_Bis = 0.5*(IC1corr+IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

    DE = 0.5*(IC->fIC_raw[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

    if (FF_IC_Y >-100 && FF_IC_Y <100){
      hDE_E->Fill(E,DE);
      hDE_Ebis->Fill(E,DE_Bis);

      if (spline== true){     
        DE_Ycorr = DE * 13750 / splineDEcorr->Eval(FF_IC_Y);
        hDE_E_corr->Fill(E,DE_Ycorr);

        DE_Ybis = DE_Bis * 13750 / splineDEbis->Eval(FF_IC_Y);
        hDE_Ebis_corr->Fill(E,DE_Ybis);  
      }

      if (cut == true && !(CutZ->IsInside(E,DE))) {;}
      else {
        hDE_Y->Fill(FF_IC_Y,DE);
      }//end cut

      if (cut == true && !(CutZbis->IsInside(E,DE_Bis))) {;}
      else {
        hDE_Y_bis->Fill(FF_IC_Y,DE_Bis);
      }//end cut
    }//end cond Y
  } //endl loop event

  // making spline

  TSpline3 *splineDE, *osplineDEbis;
  if (cut == true){
    splineDE = MakeSpline(hDE_Y,0,-100,100,14300,15000); 
    osplineDEbis = MakeSpline(hDE_Y_bis,1,-100,100,14300,15000); 
    TFile *fSpline = new TFile("Output/splineDE.root","recreate");
    splineDE->SetName("SplineDe");
    splineDE->Write();
    osplineDEbis->SetName("SplineDebis");
    osplineDEbis->Write(); 
    fSpline->Close();
    fCut->cd();
  }
  //===========================================================================================================
  //                                          Drawing histo
  //===========================================================================================================

  TCanvas* c2 = new TCanvas("c2","c2",1500,1000);
  c2->Divide(2 + 2*spline);
  c2->cd(1);
  gPad->SetLogz();
  hDE_E->Draw();
  c2->cd(2);
  gPad->SetLogz();
  hDE_Ebis->Draw();
  if (spline){
    c2->cd(3);
    gPad->SetLogz();
    hDE_E_corr->Draw();
    c2->cd(4);
    gPad->SetLogz();
    hDE_Ebis_corr->Draw();
  }
} // End spline chio XY


int GetNumberKey(TFile *infile ,const char* ClassName){
  // Get number of spline
  int KeyCount = 0 ;
  TIter next(infile->GetListOfKeys());
  TKey* keyFit;

  while ((keyFit=(TKey*)next())){
    if (std::string(keyFit->GetClassName()) == ClassName){
      KeyCount ++;
    }
  }

  return KeyCount;

}

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin = 0 , double
    XMax = 1000 , double YMin=0 , double YMax = 100) {

  TSpline3* gspline;

  TH1F* hProfile;
  TH1F* pfx; // extended hProfile

  TCanvas* canfit = new TCanvas(Form("CanSpline_%02d",NCallee),
      Form("canfit_%02d",NCallee), 0, 0, 1000, 500);


  //===========================================================================================================
  //                                            Init profile
  //===========================================================================================================

  // Create the TProfile
  hProfile = (TH1F*)hInput->ProfileX();
  hProfile->SetLineWidth(2);
  hProfile->SetDirectory(0);
  canfit->cd();

  hProfile->GetYaxis()->SetRangeUser(YMin, YMax);
  hProfile->Draw();
  //===========================================================================================================
  //                                          First and last bin to get to 6k
  //                                          event get promoted
  //===========================================================================================================
  int FirstBin, LastBin;
  double Treshold = 1;
  for (int bin =1 ; bin<hProfile->GetNbinsX(); bin++) {
    FirstBin = bin;
    if (hProfile->GetBinLowEdge(bin)> XMin) break;
  }
  for (int bin = hProfile->GetNbinsX(); bin>1 ; bin--) {
    LastBin = bin;
    if (hProfile->GetBinLowEdge(bin)< XMax) break;
  }
  //===========================================================================================================
  //                                          Init Extended profile function
  //===========================================================================================================
  // Create the extended TProfile
  pfx = new TH1F(
      Form("pfx_%02d", Int_t(NCallee/2)), Form("pfx_%02d", Int_t(NCallee/2)), hProfile->GetNbinsX(), hProfile->GetBinLowEdge(1),
      hProfile->GetBinLowEdge(hProfile->GetNbinsX()) + hProfile->GetBinWidth(hProfile->GetNbinsX()));
  pfx->SetLineColor(8);
  pfx->SetDirectory(0);
  //===========================================================================================================
  //                                           Fill extended profile
  //===========================================================================================================

  // fill the extended TProfile
  float newval, lastval, lasterr;
  for (int bin = 1; bin <= FirstBin; bin++) {
    newval = 0;
    pfx->SetBinContent(bin, newval);
  }


  for (int bin = FirstBin; bin <= LastBin; bin++) {
    newval = hProfile->GetBinContent(bin);
    if (newval != 0) {
      pfx->SetBinContent(bin, newval);
      pfx->SetBinError(bin, hProfile->GetBinError(bin));
      lastval = newval;
      lasterr = hProfile->GetBinError(bin);
    }
    else {
      pfx->SetBinContent(bin, lastval);
      pfx->SetBinError(bin, lasterr);
    }
  }
  pfx->Draw("same");

  gspline = new TSpline3(pfx);
  gspline->SetName(Form("fspline_%d", NCallee + 1));

  return gspline;

} // end makespline

