#include "TICPhysics.h"
#include "TTimeData.h"
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TLatex.h>
#include <iostream>
using namespace std;

vector<int> SortSection(UShort_t STargFocal[2]);
vector<double> ToffCalc(vector<double> ToffP2P);

void ToffGenerator(bool Create = true ){

    //===========================================================================================================
    //                                              Setters
    //===========================================================================================================

    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");


    Int_t M13 , M14, M23, M24 ;// we will fetch multiplicity during
                               // the getentry

    UShort_t S13[20] , S14[20] , S23[20] , S24[20];

    TTimeData *Time = new TTimeData();
    TICPhysics *IC = new TICPhysics();

    chain->SetBranchStatus("MTOF_FP0_T0VN", true);
    chain->SetBranchAddress("MTOF_FP0_T0VN", &S13);

    chain->SetBranchStatus("MTOF_FP0_T1VN", true);
    chain->SetBranchAddress("MTOF_FP0_T1VN", &S14);

    chain->SetBranchStatus("MTOF_FP1_T0VN", true);
    chain->SetBranchAddress("MTOF_FP1_T0VN", &S23);

    chain->SetBranchStatus("MTOF_FP1_T1VN", true);
    chain->SetBranchAddress("MTOF_FP1_T1VN", &S24);

    chain->SetBranchStatus("Time", true);
    chain->SetBranchAddress("Time", &Time);

    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC);
    //===========================================================================================================
    //                              Histograms
    //===========================================================================================================

    TH2F *hToff13;
    TH2F *hToff14;
    TH2F *hToff23;
    TH2F *hToff24;

    TH2F *hToffOnline13;
    TH2F *hToffOnline14;
    TH2F *hToffOnline23;
    TH2F *hToffOnline24;


    double xmin[4], xmax[4];
    double ymin[4], ymax[4];
    int binx = 20 , biny = 400;
    
    xmin[0] = 0 ; xmax[0] = 20 ; ymin[0] = -10 ; ymax[0] = 10 ;
    xmin[1] = 0 ; xmax[1] = 20 ; ymin[1] = -10 ; ymax[1] = 10 ;
    xmin[2] = 0 ; xmax[2] = 20 ; ymin[2] = -10 ; ymax[2] = 10 ;
    xmin[3] = 0 ; xmax[3] = 20 ; ymin[3] = -10 ; ymax[3] = 10 ;

    // Toff from aligning AoQ
    double Toff[20] = {0, 588.0, 588.5, 587.95, 588.04, 587.72, 587.92, 587.9, 587.9, 588.66, 588.80, 588.67, 588.64, 588.75, 588.47, 588.65, 588.65, 588.67, 589.05, 590.3};

    if (Create == true){

        hToff13 = new TH2F("Toff13","Toff13",binx ,xmin[0] , xmax[0], biny, ymin[0], ymax[0]);
        hToff14 = new TH2F("Toff14","Toff14",binx ,xmin[1] , xmax[1], biny, ymin[1], ymax[1]);
        hToff23 = new TH2F("Toff23","Toff23",binx ,xmin[2] , xmax[2], biny, ymin[2], ymax[2]);
        hToff24 = new TH2F("Toff24","Toff24",binx ,xmin[3] , xmax[3], biny, ymin[3], ymax[3]);

        hToffOnline13 = new TH2F("ToffOnline13","ToffOnline13",binx ,xmin[0] , xmax[0], biny, ymin[0], ymax[0]);
        hToffOnline14 = new TH2F("ToffOnline14","ToffOnline14",binx ,xmin[1] , xmax[1], biny, ymin[1], ymax[1]);
        hToffOnline23 = new TH2F("ToffOnline23","ToffOnline23",binx ,xmin[2] , xmax[2], biny, ymin[2], ymax[2]);
        hToffOnline24 = new TH2F("ToffOnline24","ToffOnline24",binx ,xmin[3] , xmax[3], biny, ymin[3], ymax[3]);


        //===========================================================================================================
        //                            Loop on entries
        //===========================================================================================================
        int Nentries = chain->GetEntries();
        //int Nentries = 1000000;
        auto start = std::chrono::high_resolution_clock::now();

        for (int e = 0; e < Nentries; e++) {

            chain->GetEntry(e);
            if (e % 100000  == 0 && e > 0 ) {
                auto now = std::chrono::high_resolution_clock::now();
                std::chrono::duration<double> elapsed = now - start;
                double avgTimePerIteration = elapsed.count() / e;
                double timeLeft = avgTimePerIteration * (Nentries - e);

                std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
            }

            // Get Multiplicity
            M13 = Time->GetMWPC13Mult();
            M14 = Time->GetMWPC14Mult();
            M23 = Time->GetMWPC23Mult();
            M24 = Time->GetMWPC24Mult();
            


            for (int i=0 ; i<M13 ; i++){
                S13[i] = Time->GetSection_MWPC3(i);
                S23[i] = Time->GetSection_MWPC3(i);
            }
 
            for (int i=0 ; i<M14 ; i++){
                S14[i] = Time->GetSection_MWPC4(i);
                S24[i] = Time->GetSection_MWPC4(i);
            }

            if (M13 == 2 && (abs(S13[0]-S13[1]) == 1)){
                vector<int> Pos = SortSection(S13);
                hToff13->Fill(S13[0],  Time->GetTime_MWPC13(Pos[0])  - Time->GetTime_MWPC13(Pos[1])  );

                hToffOnline13->Fill(S13[0], (Time->GetTime_MWPC13(Pos[0]) - Toff[S13[0]]) -( Time->GetTime_MWPC13(Pos[1]) -Toff[S13[1]]  )); 
            }
            if (M23 == 2 && (abs(S23[0]-S23[1]) == 1)){
                vector<int> Pos = SortSection(S23);
                hToff23->Fill(S23[0],Time->GetTime_MWPC23(Pos[0]) - Time->GetTime_MWPC23(Pos[1])); 
                hToffOnline23->Fill(S23[0],(Time->GetTime_MWPC23(Pos[0]) - Toff[S23[0]]) -( Time->GetTime_MWPC23(Pos[1])  -Toff[S23[1]]  )   ); 
            }

            if (M14 == 2 && (abs(S14[0]-S14[1]) == 1)){
                vector<int> Pos = SortSection(S14);
                hToff14->Fill(S14[0],Time->GetTime_MWPC14(Pos[0]) - Time->GetTime_MWPC14(Pos[1])); 
                hToffOnline14->Fill(S14[0],(Time->GetTime_MWPC14(Pos[0]) - Toff[S14[0]]) -( Time->GetTime_MWPC14(Pos[1])  -Toff[S14[1]]  )   ); 
            }

            if (M24 == 2 && (abs(S24[0]-S24[1]) == 1)){
                vector<int> Pos = SortSection(S24);
                hToff24->Fill(S24[0],Time->GetTime_MWPC24(Pos[0]) - Time->GetTime_MWPC24(Pos[1])); 
                hToffOnline24->Fill(S24[0],(Time->GetTime_MWPC24(Pos[0]) - Toff[S24[0]]) -( Time->GetTime_MWPC24(Pos[1])  -Toff[S24[1]]  )   ); 
            }
        } // End loop event

        TFile *out = new TFile("output/ToffHisto.root","recreate");
        hToff13->Write();
        hToff14->Write();
        hToff23->Write();
        hToff24->Write();
        
        hToffOnline13->Write();
        hToffOnline14->Write();
        hToffOnline23->Write();
        hToffOnline24->Write();
    }// end if create

    else {

        TFile *in = new TFile("output/ToffHisto.root","open");
        hToff13 = (TH2F*)in->Get("Toff13");
        hToff14 = (TH2F*)in->Get("Toff14");
        hToff23 = (TH2F*)in->Get("Toff23");
        hToff24 = (TH2F*)in->Get("Toff24");

        hToffOnline13 = (TH2F*)in->Get("ToffOnline13");
        hToffOnline14 = (TH2F*)in->Get("ToffOnline14");
        hToffOnline23 = (TH2F*)in->Get("ToffOnline23");
        hToffOnline24 = (TH2F*)in->Get("ToffOnline24");

    }

    int MinX = 2;
    hToff13->SetMinimum(MinX);
    hToff14->SetMinimum(MinX);
    hToff23->SetMinimum(MinX);
    hToff24->SetMinimum(MinX);
    //===========================================================================================================
    //                                       Proj fit
    //===========================================================================================================
    TCanvas* c13 = new TCanvas("c13", "Toff13", 1800, 1600);
    c13->Divide(5, 4); // Adjust rows/columns to fit the number of bins (e.g., 20 bins here)

    TCanvas* c14 = new TCanvas("c14", "Toff14", 1800, 1600);
    c14->Divide(5, 4); // Adjust rows/columns to fit the number of bins (e.g., 20 bins here)

    TCanvas* c23 = new TCanvas("c23", "Toff23", 1800, 1600);
    c23->Divide(5, 4); // Adjust rows/columns to fit the number of bins (e.g., 20 bins here)

    TCanvas* c24 = new TCanvas("c24", "Toff24", 1800, 1600);
    c24->Divide(5, 4); // Adjust rows/columns to fit the number of bins (e.g., 20 bins here)

    vector<double> P2PToff13(19), P2PToff14(19) , P2PToff23(19), P2PToff24(19);

    // Loop through bins and draw each fit
    for (int binX = 1; binX <= hToff13->GetNbinsX()-1; ++binX) {

        c13->cd(binX); 

        // Create a projection along the Y-axis for the current X-bin
        TH1F* proj = (TH1F*)hToff13->ProjectionY(Form("proj_bin13%d", binX), binX, binX);
        proj->SetMinimum(MinX);

        // Fit the projection with a Gaussian
        TF1* fit = new TF1(Form("fit_bin%d", binX), "gaus",-5,5);

        // Select subpad and draw
        proj->Fit(fit,"R"); // Suppress output with "Q"
        proj->Draw();
        fit->SetLineColor(kRed); // Optional: Differentiate fit line
        fit->Draw("SAME");

        // Retrieve fit parameters
        double mean = fit->GetParameter(1);
        double sigma = fit->GetParameter(2);
        P2PToff13.at(binX-1) = mean;

        // Add fit results to the subpad using TLatex
        TLatex text;
        text.SetNDC();
        text.SetTextSize(0.05);
        text.SetTextAlign(13); // Align at top-left corner
        text.DrawLatex(0.15, 0.85, Form("#mu = %.2f", mean));
        text.DrawLatex(0.15, 0.75, Form("#sigma = %.2f", sigma));


        // Save the mean to the text file

    }
    // Display the canvas
    c13->Update();

    // Loop through bins and draw each fit
    for (int binX = 1; binX <= hToff14->GetNbinsX()-1; ++binX) {

        c14->cd(binX); 

        // Create a projection along the Y-axis for the current X-bin
        TH1F* proj = (TH1F*)hToff14->ProjectionY(Form("proj_bin14%d", binX), binX, binX);
        proj->SetMinimum(MinX);

        // Fit the projection with a Gaussian
        TF1* fit = new TF1(Form("fit_bin%d", binX), "gaus");

        // Select subpad and draw
        proj->Fit(fit); // Suppress output with "Q"
        proj->Draw();
        fit->SetLineColor(kRed); // Optional: Differentiate fit line
        fit->Draw("SAME");

        // Retrieve fit parameters
        double mean = fit->GetParameter(1);
        double sigma = fit->GetParameter(2);
        P2PToff14.at(binX-1) = mean;



        // Add fit results to the subpad using TLatex
        TLatex text;
        text.SetNDC();
        text.SetTextSize(0.05);
        text.SetTextAlign(13); // Align at top-left corner
        text.DrawLatex(0.15, 0.85, Form("#mu = %.2f", mean));
        text.DrawLatex(0.15, 0.75, Form("#sigma = %.2f", sigma));


        // Save the mean to the text file

    }
    // Display the canvas
    c14->Update();


    // Loop through bins and draw each fit
    for (int binX = 1; binX <= hToff23->GetNbinsX()-1; ++binX) {

        c23->cd(binX); 

        // Create a projection along the Y-axis for the current X-bin
        TH1F* proj = (TH1F*)hToff23->ProjectionY(Form("proj_bin23%d", binX), binX, binX);
        proj->SetMinimum(MinX);

        // Fit the projection with a Gaussian
        TF1* fit = new TF1(Form("fit_bin%d", binX), "gaus");

        // Select subpad and draw
        proj->Fit(fit); // Suppress output with "Q"
        proj->Draw();
        fit->SetLineColor(kRed); // Optional: Differentiate fit line
        fit->Draw("SAME");

        // Retrieve fit parameters
        double mean = fit->GetParameter(1);
        double sigma = fit->GetParameter(2);
        P2PToff23.at(binX-1) = mean;

        // Add fit results to the subpad using TLatex
        TLatex text;
        text.SetNDC();
        text.SetTextSize(0.05);
        text.SetTextAlign(13); // Align at top-left corner
        text.DrawLatex(0.15, 0.85, Form("#mu = %.2f", mean));
        text.DrawLatex(0.15, 0.75, Form("#sigma = %.2f", sigma));


        // Save the mean to the text file

    }
    // Display the canvas
    c23->Update();

    // Loop through bins and draw each fit
    for (int binX = 1; binX <= hToff24->GetNbinsX()-1; ++binX) {

        c24->cd(binX); 

        // Create a projection along the Y-axis for the current X-bin
        TH1F* proj = (TH1F*)hToff24->ProjectionY(Form("proj_bin24%d", binX), binX, binX);
        proj->SetMinimum(MinX);

        // Fit the projection with a Gaussian
        TF1* fit = new TF1(Form("fit_bin%d", binX), "gaus");

        // Select subpad and draw
        proj->Fit(fit,"Q"); // Suppress output with "Q"
        proj->Draw();
        fit->SetLineColor(kRed); // Optional: Differentiate fit line
        fit->Draw("SAME");

        // Retrieve fit parameters
        double mean = fit->GetParameter(1);
        double sigma = fit->GetParameter(2);
        P2PToff24.at(binX-1) = mean;

        // Add fit results to the subpad using TLatex
        TLatex text;
        text.SetNDC();
        text.SetTextSize(0.05);
        text.SetTextAlign(13); // Align at top-left corner
        text.DrawLatex(0.15, 0.85, Form("#mu = %.2f", mean));
        text.DrawLatex(0.15, 0.75, Form("#sigma = %.2f", sigma));


        // Save the mean to the text file

    }
    // Display the canvas
    c24->Update();  

    //===========================================================================================================
    //                                          Saving correction
    //===========================================================================================================


    //First we get the absolute correction to use on each section
    vector<double> Toff13, Toff14, Toff23, Toff24;
    Toff13 = ToffCalc(P2PToff13);
    Toff23 = ToffCalc(P2PToff23);
    Toff14 = ToffCalc(P2PToff14);
    Toff24 = ToffCalc(P2PToff24);

    //Save it in a txt file
    ofstream o13("output/Toff13.txt");
    ofstream o23("output/Toff23.txt");
    ofstream o14("output/Toff14.txt");
    ofstream o24("output/Toff24.txt");

    for (auto &elem : Toff13) o13 << elem << endl;
    for (auto &elem : Toff14) o14 << elem << endl;
    for (auto &elem : Toff23) o23 << elem << endl;
    for (auto &elem : Toff24) o24 << elem << endl;


    //===========================================================================================================
    //                                       Testing correction
    //===========================================================================================================


    TH2F*  hCorrToff13 = new TH2F("CorrToff13","CorrToff13",binx ,xmin[0] , xmax[0], biny, ymin[0], ymax[0]);
    TH2F*  hCorrToff14 = new TH2F("CorrToff14","CorrToff14",binx ,xmin[1] , xmax[1], biny, ymin[1], ymax[1]);
    TH2F*  hCorrToff23 = new TH2F("CorrToff23","CorrToff23",binx ,xmin[2] , xmax[2], biny, ymin[2], ymax[2]);
    TH2F*  hCorrToff24 = new TH2F("CorrToff24","CorrToff24",binx ,xmin[3] , xmax[3], biny, ymin[3], ymax[3]);


    int Nentries = chain->GetEntries();
    //int Nentries = 1000000;
    auto start = std::chrono::high_resolution_clock::now();

    for (int e = 0; e < Nentries; e++) {
        chain->GetEntry(e);
        
        if (e % 100000  == 0 && e > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }

        // Get Multiplicity
        M13 = Time->GetMWPC13Mult();
        M14 = Time->GetMWPC14Mult();
        M23 = Time->GetMWPC23Mult();
        M24 = Time->GetMWPC24Mult();

        for (int i=0 ; i<M13 ; i++){
            S13[i] = Time->GetSection_MWPC3(i);
            S23[i] = Time->GetSection_MWPC3(i);
        }

        for (int i=0 ; i<M14 ; i++){
            S14[i] = Time->GetSection_MWPC4(i);
            S24[i] = Time->GetSection_MWPC4(i);
        }


        if (M13 == 2 && (abs(S13[0]-S13[1]) == 1)){
            vector<int> Pos = SortSection(S13);
            hCorrToff13->Fill(S13[0],(Time->GetTime_MWPC13(Pos[0]) - double(Time->GetTS_MWPC13(Pos[0]))+ Toff13.at(S13[0])) - (Time->GetTime_MWPC13(Pos[1]) - double(Time->GetTS_MWPC13(Pos[1]))+ Toff13.at(S13[1])));
        }


        if (M23 == 2 && (abs(S23[0]-S23[1]) == 1)){
            vector<int> Pos = SortSection(S23);
            hCorrToff23->Fill(S23[0],(Time->GetTime_MWPC23(Pos[0]) + Toff23.at(S23[0])) - (Time->GetTime_MWPC23(Pos[1]) + Toff23.at(S23[1])));
        }


        if (M14 == 2 && (abs(S14[0]-S14[1]) == 1)){
            vector<int> Pos = SortSection(S14);
            hCorrToff14->Fill(S14[0],(Time->GetTime_MWPC14(Pos[0]) + Toff14.at(S14[0])) - (Time->GetTime_MWPC14(Pos[1]) + Toff14.at(S14[1])));
        }


        if (M24 == 2 && (abs(S24[0]-S24[1]) == 1)){
            vector<int> Pos = SortSection(S24);
            hCorrToff24->Fill(S24[0],(Time->GetTime_MWPC24(Pos[0]) + Toff24.at(S24[0])) - (Time->GetTime_MWPC24(Pos[1]) + Toff24.at(S24[1])));
        }


    } // End loop event

    //===========================================================================================================
    //                                       Drawing canvas
    //===========================================================================================================

    TCanvas *c1 = new TCanvas("c1","c1");
    c1->Divide(3);

    c1->cd(1);
    hToff13->Draw("colz");
    c1->cd(2);
    hCorrToff13->Draw("colz");
    c1->cd(3);
    hToffOnline13->Draw("colz");


    TCanvas *c2 = new TCanvas("c2","c2");
    c2->Divide(3);

    c2->cd(1);
    hToff23->Draw("colz");
    c2->cd(2);
    hCorrToff23->Draw("colz");
    c2->cd(3);
    hToffOnline23->Draw("colz");



    TCanvas *c3 = new TCanvas("c3","c3");
    c3->Divide(3);

    c3->cd(1);
    hToff14->Draw("colz");
    c3->cd(2);
    hCorrToff14->Draw("colz");
    c3->cd(3);
    hToffOnline14->Draw("colz");


    TCanvas *c4 = new TCanvas("c4","c4");
    c4->Divide(3);

    c4->cd(1);
    hToff24->Draw("colz");
    c4->cd(2);
    hCorrToff24->Draw("colz");
    c4->cd(3);
    hToffOnline24->Draw("colz");

}

vector<int> SortSection(UShort_t STargFocal[2]){
    UShort_t FSection ,LSection ;
    vector<int> Pos(2) ;
    if (STargFocal[0]<STargFocal[1]){
        Pos[0] = 0    ;
        Pos[1]  = 1    ;
        FSection = STargFocal[0];
        LSection = STargFocal[1];
    }
    else {
        Pos[0] = 1    ;
        Pos[1]  = 0   ;
        FSection = STargFocal[1];
        LSection = STargFocal[0];    
    }

    STargFocal[0] = FSection;
    STargFocal[1] = LSection;
    return Pos; 
}

vector<double> ToffCalc(vector<double> ToffP2P){

    vector<double> res;
    double Sum = 0;
    res.push_back(Sum); // first section 
    for (int i=0; i<ToffP2P.size() ; i++){
        Sum += ToffP2P.at(i);
        res.push_back(Sum);
    }
    return res;
}
