TChain* chain;
int nentries;
TTree* otree;
double DeltaE;
double DeltaE_back;
double Eres;
double Mass;
double Xcalc;
double Ycalc;
double Zcalc;
double XTarget;
double YTarget;
int Telescope;
int m_2alpha;
int run_number;
TPISTAPhysics * PISTA;
TCutG* cut_12C[8];
TCutG* cut_9Be[8];


/////////////////////////////////////////////////////////
void LoadCuts(){
  TFile* ifile = new TFile("cut/cut_12C.root");
  for(int i=0; i<8; i++){
    TString cutname = Form("cut_det%d",i+1);
    cut_12C[i] = (TCutG*)ifile->FindObjectAny(cutname);
  }

  /* TFile* ifile2 = new TFile("cut/cut_9Be.root"); */
  /* for(int i=0; i<8; i++){ */
  /*   TString cutname = Form("cut_det%d",i+1); */
  /*   cut_9Be[i] = (TCutG*)ifile2->FindObjectAny(cutname); */
  /* } */

}

/////////////////////////////////////////////////////////
void LoadRootFile(int run){
  chain = new TChain("PhysicsTree");
  TString Filename = Form("../../../root/analysis/run_calib_%d.root",run);
  chain->Add(Filename);

  PISTA = new TPISTAPhysics();
  NPL::InputParser parser("../../../pista_e850_2024.detector");
  PISTA->ReadConfiguration(parser);
  chain->SetBranchAddress("PISTA",&PISTA);

  chain->SetBranchStatus("Mass","true");
  chain->SetBranchAddress("Mass",&Mass);

  chain->SetBranchStatus("DeltaE","true");
  chain->SetBranchAddress("DeltaE",&DeltaE);

  chain->SetBranchStatus("DeltaE_back","true");
  chain->SetBranchAddress("DeltaE_back",&DeltaE_back);

  chain->SetBranchStatus("Eres","true");
  chain->SetBranchAddress("Eres",&Eres);

  chain->SetBranchStatus("Xcalc","true");
  chain->SetBranchAddress("Xcalc",&Xcalc);
  chain->SetBranchStatus("Ycalc","true");
  chain->SetBranchAddress("Ycalc",&Ycalc);
  chain->SetBranchStatus("Zcalc","true");
  chain->SetBranchAddress("Zcalc",&Zcalc);
 
  chain->SetBranchStatus("XTarget","true");
  chain->SetBranchAddress("XTarget",&XTarget);
 
  chain->SetBranchStatus("YTarget","true");
  chain->SetBranchAddress("YTarget",&YTarget);
 
  chain->SetBranchStatus("Telescope","true");
  chain->SetBranchAddress("Telescope",&Telescope);
 
  chain->SetBranchStatus("m_2alpha","true");
  chain->SetBranchAddress("m_2alpha",&m_2alpha);

 
}

/////////////////////////////////////////////////////////
void InitOutputTree(){
  otree = new TTree("tree","Select Tree");

  otree->Branch("DeltaE",&DeltaE,"DeltaE/D");
  otree->Branch("DeltaE_back",&DeltaE_back,"DeltaE_back/D");
  otree->Branch("Eres",&Eres,"Eres/D");
  otree->Branch("XTarget",&XTarget,"XTarget/D");
  otree->Branch("YTarget",&YTarget,"YTarget/D");
  otree->Branch("Xcalc",&Xcalc,"Xcalc/D");
  otree->Branch("Ycalc",&Ycalc,"Ycalc/D");
  otree->Branch("Zcalc",&Zcalc,"Zcalc/D");
  otree->Branch("Telescope",&Telescope,"Telescope/I");
  otree->Branch("run_number",&run_number,"run_number/I");
  otree->Branch("PISTA",&PISTA);
}

/////////////////////////////////////////////////////////
void FillSelectTree(int run=235){
  LoadRootFile(run);
  LoadCuts();
  run_number = run;

  nentries = 1e5; //chain->GetEntries();
  cout << "Number of entries: " << nentries << endl;

  TString ofilename = Form("SelectRun/SelectTree_run%d.root",run);
  TFile* ofile = new TFile(ofilename,"recreate");
  InitOutputTree();

  int good_entry = 0;
  for(int i=0; i<nentries; i++){
    chain->GetEntry(i);

    if(i%100000==0){
      cout << "\033[34m\r Processing tree..." << (double)i/nentries*100 << "\% done" << flush;
    }

    //cout << PISTA->EventMultiplicity << endl;

    if(Telescope>0 && Xcalc != -1000 && PISTA->EventMultiplicity==1){
      if(cut_12C[Telescope-1]->IsInside(Eres,DeltaE)){
        otree->Fill(); 
        good_entry++;
        //if(good_entry==100000) break; 
     }
    }
  }

  otree->Write();
  delete ofile;
}
