TChain *chain;
double* parameter;
//const int nb_parameter = 29;
const int nb_parameter = 24;
int iteration = 0;
int m_telescope;

int NumericalMinimization(const char* minName = "Minuit", const char* algoName = "");
double ConstantFactor(const double* parameter);

NPL::Reaction* elastic;
NPL::Reaction* elasticBe;

TH2F* hEx;
TH2F* hExmean_Tel;
TH2F* hExmean_ThetaLab;
TH2F* hExBe;
TH2F* hExmeanBe_Tel;
TH2F* hExmeanBe_ThetaLab;
TH1D* hExmean;
///////////////////////////////////////////////////
void Minimization2(int telescope=3){

  elastic   = new NPL::Reaction("238U(12C,12C)238U@1416");
  elasticBe = new NPL::Reaction("238U(9Be,9Be)238U@1416");
  hEx = new TH2F("hEx","hEx",6000,0,6000,500,-10,10);
  hExmean_Tel = new TH2F("hExmean_Tel","hExmean_Tel",8,1,9,500,-10,10);
  hExmean_ThetaLab = new TH2F("hExmean_ThetaLab","hExmean_ThetaLab",300,15,75,500,-10,10);
  hExBe = new TH2F("hExBe","hExBe",6000,0,6000,500,-10,10);
  hExmeanBe_Tel = new TH2F("hExmeanBe_Tel","hExmeanBe_Tel",8,1,9,500,-10,10);
  hExmeanBe_ThetaLab = new TH2F("hExmeanBe_ThetaLab","hExmeanBe_ThetaLab",300,15,75,500,-10,10);

  m_telescope = telescope;
  chain = new TChain("tree"); 
  chain->Add("SelectTree.root");
  //chain->Add("SelectTree_run215.root");

  double buffer[nb_parameter];
  for(int i=0; i<7; i++){
    buffer[3*i] = 0;
    buffer[3*i+1] = 0.0029;
    buffer[3*i+2] = 0.;
  }
  buffer[24-3] = 0;
  buffer[25-3] = 0;
  buffer[26-3] = 0;
  //buffer[27-3] = 0;
  //buffer[28-3] = 0;

  parameter = new double[nb_parameter];
  parameter = buffer;

  ConstantFactor(parameter);
  NumericalMinimization("Minuit","Migrad");


  TCanvas* c1 = new TCanvas("c1","c1",1600,800);
  c1->Divide(2,1);
  c1->cd(1);
  hExmean_Tel->Draw("colz");
  c1->cd(2);
  hExmean_ThetaLab->Draw("colz");

  TCanvas* c3 = new TCanvas("c3","c3",1600,800);
  c3->Divide(2,1);
  c3->cd(1);
  hExmeanBe_Tel->Draw("colz");
  c3->cd(2);
  hExmeanBe_ThetaLab->Draw("colz");

  TCanvas* c2 = new TCanvas("c2","c2",1600,800);
  c2->Divide(2,1);
  c2->cd(1);
  hEx->Draw("colz");
  c2->cd(2);
  hExBe->Draw("colz");
}



////////////////////////////////////////////////////
int NumericalMinimization(const char* minName, const char* algoName){

  ROOT::Math::Minimizer* min = ROOT::Math::Factory::CreateMinimizer(minName, algoName);

  min->SetMaxFunctionCalls(1000000);
  min->SetMaxIterations(100000);
  min->SetTolerance(0.001);
  min->SetPrecision(0.0001);
  min->SetPrintLevel(1);

  ROOT::Math::Functor f(&ConstantFactor,nb_parameter);

  min->SetFunction(f);

  for(int i=0; i<7; i++){
    
    int indice = 3*i;
    if(i==6) i=7;
    string par0_name = "p0_E" + to_string(i+1);
    string par1_name = "p1_E" + to_string(i+1);
    string par2_name = "p2_E" + to_string(i+1);

    min->SetLimitedVariable(indice,par0_name,0,0.01,-0.2,0.2);
    min->SetLimitedVariable(indice+1,par1_name,0.0029,0.0001,0.9*0.0029,1.1*0.0029);
    min->SetLimitedVariable(indice+2,par2_name,0,0.0001,-1.e-8,1.e-8);
  }
  min->SetLimitedVariable(24-3,"Xt",0,0.01,-2,2);
  min->SetLimitedVariable(25-3,"Yt",0,0.01,-2,2);
  min->SetLimitedVariable(26-3,"Zt",0,0.01,-1,1);
  //min->SetLimitedVariable(27-3,"ThetaX",0,0.01,-0.5,0.5);
  //min->SetLimitedVariable(28-3,"ThetaY",0,0.01,-0.5,0.5);

  min->Minimize();

  const double* xs = min->X();

  ofstream ofile;
  string filename = "PISTA_BACK_E_min.cal";
  ofile.open(filename.c_str());

  for(int i=0; i<7; i++){
    int indice = 3*i;
    if(i==6) i=7;
    string token = "PISTA_T" + to_string(i+1) + "_BACK_E";

    ofile << token << " " << xs[indice] << " " << xs[indice+1] << " " << xs[indice+2] << endl;
  }
  ofile.close();

  ofstream ofile2;
  string filename2 = "AnalysisConfig_min.dat";
  ofile2.open(filename2.c_str());
  ofile2 <<  "AnalysisConfig" << endl;
  ofile2 <<  " XTARGET_OFFSET " << xs[24-3] << endl;
  ofile2 <<  " YTARGET_OFFSET " << xs[25-3] << endl;
  ofile2 <<  " ZTARGET_OFFSET " << xs[26-3] << endl;
  //ofile2 <<  " BEAM_THETAX " << xs[27-3] << endl;
  //ofile2 <<  " BEAM_THETAY " << xs[28-3] << endl;
  ofile2.close();

  cout << "MinValue = " << min->MinValue() << endl;
  ConstantFactor(xs);

  return 0;

}

////////////////////////////////////////////////////
double ConstantFactor(const double* parameter){

  iteration++;

  double DeltaE;
  double Eres;
  double Xcalc;
  double Ycalc;
  double Zcalc;
  double XTarget;
  double YTarget;
  int Telescope;
  int run_number;

  chain->SetBranchStatus("DeltaE","true");
  chain->SetBranchAddress("DeltaE",&DeltaE);

  chain->SetBranchStatus("Eres","true");
  chain->SetBranchAddress("Eres",&Eres);

  chain->SetBranchStatus("XTarget","true");
  chain->SetBranchAddress("XTarget",&XTarget);

  chain->SetBranchStatus("YTarget","true");
  chain->SetBranchAddress("YTarget",&YTarget);

  chain->SetBranchStatus("Xcalc","true");
  chain->SetBranchAddress("Xcalc",&Xcalc);

  chain->SetBranchStatus("Ycalc","true");
  chain->SetBranchAddress("Ycalc",&Ycalc);

  chain->SetBranchStatus("Zcalc","true");
  chain->SetBranchAddress("Zcalc",&Zcalc);

  chain->SetBranchStatus("Telescope","true");
  chain->SetBranchAddress("Telescope",&Telescope);

  chain->SetBranchStatus("run_number","true");
  chain->SetBranchAddress("run_number",&run_number);



  int nentries = chain->GetEntries();

  hExmean_Tel->Reset();
  hExmean_ThetaLab->Reset();

  double Elab = 0;
  double ThetaLab = 0;
  double Ecal = 0;
  double distance=100;
  for(int i=0; i<nentries; i++){
    chain->GetEntry(i);
    if(Telescope==8) {
      Telescope=7;
    }
    int indice = 3*(Telescope-1);
    Ecal = parameter[indice] + parameter[indice+1]*Eres + parameter[indice+2]*Eres*Eres;
    Elab = DeltaE + Ecal;
    //cout << "Telescope=" << Telescope << " indice=" << indice << endl;
    if(Xcalc!=-1000 && Ycalc!=-1000 && Zcalc!=-1000){
      TVector3 PositionOnTarget = TVector3(XTarget+parameter[24-3],YTarget+parameter[25-3],parameter[26-3]);
      TVector3 HitPosition = TVector3(Xcalc,Ycalc,Zcalc);

      TVector3 BeamDirection = TVector3(0,0,1);
      //BeamDirection.RotateX(parameter[27-3]*3.1415/180);
      //BeamDirection.RotateY(parameter[28-3]*3.1415/180);
      TVector3 HitDirection = HitPosition - PositionOnTarget;
      ThetaLab = HitDirection.Angle(BeamDirection);

      double Ex=-1000;
      if(run_number==215)
	{
	  Ex = elasticBe->ReconstructRelativistic(Elab,ThetaLab);
	  if(abs(Ex)<10){
	    hExBe->Fill(iteration,Ex);
	    hExmeanBe_Tel->Fill(indice/3+1,Ex);
	    hExmeanBe_ThetaLab->Fill(ThetaLab*180/3.1416,Ex);
	  }
	}
      else
	{
	  Ex = elastic->ReconstructRelativistic(Elab,ThetaLab);

	  if(abs(Ex)<10){
	    hEx->Fill(iteration,Ex);
	    hExmean_Tel->Fill(indice/3+1,Ex);
	    hExmean_ThetaLab->Fill(ThetaLab*180/3.1416,Ex);
	  }
	}

    }
  }

  double Emean[7];
  double sigma[7];
  double AvEmean=0;
  distance = 0;  

  for(int i=0;i<7;i++)
  {
    // C12 case
    hExmean = hExmean_Tel->ProjectionY(Form("hExmean_%d",i+1),i+1,i+1);
    Emean[i] = hExmean->GetMean();
    double Emin = Emean[i] - 6;
    double Emax = Emean[i] + 6;
    hExmean->GetXaxis()->SetRangeUser(Emin,Emax);
    Emean[i] = hExmean->GetMean();
    sigma[i] = hExmean->GetRMS();
    distance += pow(Emean[i],2) + pow(sigma[i]-0.3,2);
    AvEmean += Emean[i];

    // Be9 case
    hExmean = hExmeanBe_Tel->ProjectionY(Form("hExmean_%d",i+1),i+1,i+1);
    Emean[i] = hExmean->GetMean();
    Emin = Emean[i] - 6;
    Emax = Emean[i] + 1;
    hExmean->GetXaxis()->SetRangeUser(Emin,Emax);
    Emean[i] = hExmean->GetMean();
    sigma[i] = hExmean->GetRMS();
    distance += pow(Emean[i],2) + pow(sigma[i]-0.3,2);
    
  }
  AvEmean/=8.;
  if(iteration%1==0){
    cout << "Number of iterations: " << iteration  << endl;
    cout << "Emean= " << AvEmean << endl;
    for(int j=0;j<24;j++){
      cout << parameter[j] << endl;
    }

    cout << "distance= " << distance << endl;
  }


  //delete hExmean;
  return distance;
}
