#include <iostream>

void Secator(const char* inputFileName, const char* outputDir, int N) {
    // Open the original ROOT file
    TFile* inputFile = TFile::Open(inputFileName, "READ");
    if (!inputFile || inputFile->IsZombie()) {
        printf("Error: Cannot open file %s\n", inputFileName);
        return;
    }

    // Extract the base name of the input file (without path and extension)
    TString baseName = gSystem->BaseName(inputFileName);
    baseName.ReplaceAll(".root", "");  // Remove the .root extension

    // Ensure the output directory ends with a '/'
    TString outputDirStr = outputDir;
    if (!outputDirStr.EndsWith("/")) {
        outputDirStr.Append("/");
    }

    // Get the tree from the file
    TTree* inputTree = (TTree*)inputFile->Get("RawTree"); // replace with actual tree name
    Long64_t totalEntries = inputTree->GetEntries();

    // Calculate entries per subfile
    Long64_t entriesPerFile = totalEntries / N;

    // Loop to create N subfiles
    for (int i = 0; i < N; ++i) {
        // Create output file name by appending a number to the base name
        TString outputFileName = TString::Format("%s%s_%d.root", outputDirStr.Data(), baseName.Data(), i);
        TFile* outputFile = TFile::Open(outputFileName, "RECREATE");

        // Create a clone of the tree in the new file
        TTree* outputTree = inputTree->CloneTree(0);

        // Write the appropriate entries to the subfile
        Long64_t start = i * entriesPerFile;
        Long64_t end = (i == N - 1) ? totalEntries : start + entriesPerFile;

        for (Long64_t entry = start; entry < end; ++entry) {
            inputTree->GetEntry(entry);
            outputTree->Fill();

            // Progress bar update
            float progress = (float)(entry - start + 1) / (end - start) * 100;
            printf("\rProcessing subfile %d: [%.2f%%]", i + 1, progress);
            fflush(stdout); // Ensure immediate print
        }

        printf("\n");  // Newline after progress bar completion for this subfile

        // Write and close the subfile
        outputTree->Write();
        outputFile->Close();
        delete outputFile;
    }

    inputFile->Close();
    delete inputFile;
}

