#ifndef Analysis_h 
#define Analysis_h
/*****************************************************************************
 * Copyright (C) 2009-2016    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: XAUTHORX  contact address: XMAILX                        *
 *                                                                           *
 * Creation Date  : XMONTHX XYEARX                                           *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This class describe  TOGAXSI_SI analysis project                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *****************************************************************************/


#include "NPVAnalysis.h"
#include "NPEnergyLoss.h"
#include "NPQFS.h"
#include "RootOutput.h"
#include "RootInput.h"
#include "TTOGAXSI_SIPhysics.h"
#include "TInitialConditions.h"
#include "TInteractionCoordinates.h"
#include "TReactionConditions.h"
#include <TRandom3.h>
#include <TVector3.h>
#include <TLorentzVector.h>
#include <TMath.h>


class Analysis: public NPL::VAnalysis{
  public:
    Analysis();
    ~Analysis();

  public: 
    void Init();
    void TreatEvent();
    void End();

    void InitOutputBranch();
    void InitInputBranch();
    void ReInitValue();
    static NPL::VAnalysis* Construct();
    TVector3 InterpolateInPlaneZ(TVector3, TVector3, double);
    double ApplyCATANAResoGamma(double);
    double ApplyCATANAResoProton(double);


  private:
    double Ex;
    double E1;
    double E2;
    double E1s;
    double E2s;
    double Theta12;
    double ThetaCM;
    double VertexX;
    double VertexY;
    double VertexZ;
    double VertexX_sim;
    double VertexY_sim;
    double VertexZ_sim;
    double deltaX;
    double deltaY;
    double deltaZ;
    double Distance;
    double deltaPhi;
    double sumTheta;
    double alpha;
    double Theta1=-1000;
    double Phi1=-1000;
    double Theta2=-1000;
    double Phi2=-1000;
    double Theta1s=-1000;
    double Phi1s=-1000;
    double Theta2s=-1000;
    double Phi2s=-1000;
    double TA;
    double TAcalc;

    double SiInnerZ1_X;
    double SiInnerZ1_Y;
    double SiInnerZ1_Z;
    double SiOuterZ1_X;
    double SiOuterZ1_Y;
    double SiOuterZ1_Z;
    double SiInnerZ1_X_sim;
    double SiInnerZ1_Y_sim;
    double SiInnerZ1_Z_sim;
    double SiOuterZ1_X_sim;
    double SiOuterZ1_Y_sim;
    double SiOuterZ1_Z_sim;

    double SiInnerX1_X;
    double SiInnerX1_Y;
    double SiInnerX1_Z;
    double SiInnerX1_X_sim;
    double SiInnerX1_Y_sim;
    double SiInnerX1_Z_sim;

    double SiInnerZ2_X;
    double SiInnerZ2_Y;
    double SiInnerZ2_Z;
    double SiOuterZ2_X;
    double SiOuterZ2_Y;
    double SiOuterZ2_Z;
    double SiInnerZ2_X_sim;
    double SiInnerZ2_Y_sim;
    double SiInnerZ2_Z_sim;
    double SiOuterZ2_X_sim;
    double SiOuterZ2_Y_sim;
    double SiOuterZ2_Z_sim;

    double SiInnerX2_X;
    double SiInnerX2_Y;
    double SiInnerX2_Z;
    double SiInnerX2_X_sim;
    double SiInnerX2_Y_sim;
    double SiInnerX2_Z_sim;


    double theta_vertex_1_sim;
    double theta_vertex_2_sim;

    double InnerZ1_theta_sim;
    double InnerX1_theta_sim;
    double OuterZ1_theta_sim;


    double z_1_sim;
    double z_2_sim;
    double z_3_sim;
    double z_4_sim;

    TRandom3 *rand;

    TLorentzVector LV_A;
    TLorentzVector LV_T;
    TLorentzVector LV_B;
    TLorentzVector LV_p1;
    TLorentzVector LV_p2;
   
    NPL::Beam* myBeam;
    NPL::QFS* m_QFS;

    EnergyLoss BeamTarget;
    EnergyLoss protonTarget;
    EnergyLoss protonAl;
    EnergyLoss protonSi;
    double ReconstructProtonEnergy(const TVector3& x0, const TVector3& dir, const double& Ecatana);
    void CalcLine(const TVector3& n1, const TVector3& p1, const TVector3& n2, const TVector3& p2, TVector3& point, TVector3& direction);

    TVector3 BeamImpact;
    double TargetThickness;
    double InitialBeamEnergy;

    TRandom3 Rand;
    TInitialConditions* IC;
    TInteractionCoordinates* DC;
    TReactionConditions* RC;

    TTOGAXSI_SIPhysics* TOGAXSI_SI;

};
#endif
