TGraphErrors* GetGoodenYield(int A, int year, double scale);
TGraphErrors* GetTonchevYield(int A);
TGraphErrors* GetGEFYield(int A, double scale);

//////////////////////////////////////////////////
void PlotYieldEvolution(int A_asked=99, double scale=1.25){
  gROOT->SetStyle("pierre_style");
  int Energy[11] = {6,7,8,9,10,11,12,13,14,15,16};
  TGraphErrors* gevol = new TGraphErrors();
  TGraphErrors* g_relative = new TGraphErrors();

  for(int i = 0; i<10; i++){
    ifstream ifile;
    string input_filename = "Mass_Yield_" + to_string(Energy[i]) + "MeV.dat";
    ifile.open(input_filename.c_str());

    int A;
    double yield;
    double yield_err;
    double Sn = 6.534;
    while(ifile>>A>>yield>>yield_err){
      if(A==A_asked){
        gevol->SetPoint(i,Energy[i]-Sn,scale*yield);
        gevol->SetPointError(i,0,yield_err);

        g_relative->SetPoint(i,Energy[i]-5.5,yield_err/yield);
      }
    }
  }

  TH2F* h2 = new TH2F("h2","h2",100,-1,11.5,100,1,11.5);
  TCanvas* c1 = new TCanvas("c1","c1",800,800);
  c1->cd();
  gevol->SetMarkerStyle(8);
  gevol->SetMarkerSize(1.5);
  h2->GetXaxis()->SetTitle("E_{n} (MeV)");
  h2->GetYaxis()->SetTitle("Yield (%)");
  h2->GetXaxis()->CenterTitle();
  h2->GetYaxis()->CenterTitle();
  h2->Draw();
  if(A_asked==140 || A_asked==97 || A_asked==99 || A_asked==147 || A_asked==132 || A_asked==143 || A_asked==103 || A_asked==135){
    TGraphErrors* ggooden2016 = GetGoodenYield(A_asked,2016,1);
    ggooden2016->SetMarkerStyle(22);
    ggooden2016->SetMarkerColor(2);
    ggooden2016->SetLineColor(2);
    ggooden2016->SetMarkerSize(1.5);
    ggooden2016->SetName("gooden2016");
    //ggooden2016->Draw("psame");

    TGraphErrors* ggooden2024 = GetGoodenYield(A_asked,2024,1);
    ggooden2024->SetMarkerStyle(23);
    ggooden2024->SetMarkerColor(4);
    ggooden2024->SetLineColor(4);
    ggooden2024->SetMarkerSize(1.5);
    ggooden2024->SetName("gooden2024");
    //ggooden2024->Draw("psame");

    TGraphErrors* gtonchev = GetTonchevYield(A_asked);
    gtonchev->SetMarkerStyle(23);
    gtonchev->SetMarkerColor(4);
    gtonchev->SetLineColor(4);
    gtonchev->SetMarkerSize(1.5);
    gtonchev->SetName("Tonchev");
    gtonchev->Draw("psame");
  }

  if(A_asked==99 || A_asked==97 || A_asked==147 || A_asked==135){
    TGraphErrors* ggef = GetGEFYield(A_asked,1.);
    ggef->SetName("gef");
    ggef->Draw("psame");
  }

  gevol->SetName("ganil");
  gevol->Draw("psame");

  TLegend* leg = new TLegend(0.6,0.7,0.9,0.95);
  leg->SetBorderSize(0);
  //leg->AddEntry("gooden2016","Gooden 2016","lp");
  //leg->AddEntry("gooden2024","Gooden 2024","lp");
  leg->AddEntry("Tonchev","Tonchev et al. 2024","lp");
  leg->AddEntry("gef","GEF 2024","lp");
  leg->AddEntry("ganil","PISTA 2023","lp");
  leg->Draw("same");
  
  //TCanvas* c2 = new TCanvas("c2","c2",800,800);
  //c2->cd();
  //g_relative->SetMarkerStyle(8);
  //g_relative->Draw("ap");


}

//////////////////////////////////////////////////
TGraphErrors* GetGEFYield(int A, double scale=1){
  string filename = "GEF/mass_" + to_string(A) + ".dat";
  ifstream ifile;
  ifile.open(filename.c_str());

  double En;
  double Y;
  TGraphErrors* gerr = new TGraphErrors();
  int i=0;
  while(ifile>>En>>Y){
    gerr->SetPoint(i,En,scale*Y);
    gerr->SetPointError(i,0,0.01*Y);
    i++;
  }

  gerr->SetMarkerStyle(33);
  gerr->SetMarkerSize(2);
  gerr->SetMarkerColor(2);
  gerr->SetLineColor(2);
  return gerr;

}



//////////////////////////////////////////////////
TGraphErrors* GetGoodenYield(int A, int year, double scale=1){
  string filename = "gooden/" + to_string(year) + "/mass_" + to_string(A) + ".dat";
  ifstream ifile;
  ifile.open(filename.c_str());

  double En;
  double Y;
  double Yerr;
  TGraphErrors* gerr = new TGraphErrors();
  for(int i=0; i<8; i++){
    ifile>>En>>Y>>Yerr;
    gerr->SetPoint(i,En,scale*Y);
    gerr->SetPointError(i,0,Yerr);
  }

  return gerr;

}

//////////////////////////////////////////////////
TGraphErrors* GetTonchevYield(int A){
  string filename = "tonchev/mass_" + to_string(A) + ".dat";
  ifstream ifile;
  ifile.open(filename.c_str());

  double En;
  double Y;
  double Yerr;
  TGraphErrors* gerr = new TGraphErrors();
  int i=0;
  while(ifile>>En>>Y>>Yerr){
    gerr->SetPoint(i,En,Y);
    gerr->SetPointError(i,0,Yerr);
    i++;
  }

  return gerr;

}
