/*****************************************************************************
 * Copyright (C) 2009-2014    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Adrien MATTA  contact address: a.matta@surrey.ac.uk      *
 * Edited for e793s                                                          *
 *                                                                           *
 * Creation Date  :                                                          *
 * Last update    : May 2021                                                 *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 * Class describing the property of an Analysis object                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#include<iostream>
#include<string>

#include<fstream>

using namespace std;

#include "Analysis.h"
#include "NPFunction.h"
#include "NPAnalysisFactory.h"
#include "NPDetectorManager.h"
#include "NPOptionManager.h"
#include "DefineColours.h"

// Ex Observed:Expected correction factors
double ExCorrM, ExCorrC;

////////////////////////////////////////////////////////////////////////////////
Analysis::Analysis(){
}

////////////////////////////////////////////////////////////////////////////////
Analysis::~Analysis(){
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::Init() {
cout << "start_Init" << endl;
 ///////////////////////////////////////////////////////////////////////////////  

  if(NPOptionManager::getInstance()->HasDefinition("Sim")){
    cout << " == == == == SIMULATION == == == ==" << endl;
      isSim=true;
      isPhaseSpace=false;
  } else if (NPOptionManager::getInstance()->HasDefinition("Exp")) {
    cout << " == == == == EXPERIMENT == == == ==" << endl;
    isSim=false;
    isPhaseSpace=false;
  } else if (NPOptionManager::getInstance()->HasDefinition("PSp")) {
    cout << " == == == == PHASE SPACE == == == ==" << endl;
      isSim=false;
      //isSim=true;
      isPhaseSpace=true;
  } else {
    cout << " == == == == DEFAULT (EXP) == == == ==" << endl;
    isSim=false;
    isPhaseSpace=false;
  }
  
  if(NPOptionManager::getInstance()->HasDefinition("Au")){
    cout << " -- -- -- -- Au Target -- -- -- --" << endl;
    runningAuTarget=true;
    ExCorrM = 0.991; //1.0; //0.990;
    ExCorrC = 0.074; //0.0; //0.079;
  } else if(NPOptionManager::getInstance()->HasDefinition("CD2")){
    cout << " -- -- -- -- CD2 Target -- -- -- --" << endl;
    runningAuTarget=false;
    ExCorrM = 0.988; //1.0; //0.977;
    ExCorrC = 0.124; //0.0; //0.142;
  } else {
    cout << " ERROR: NO TARGET SPECIFIED BY --DEFINE !" << endl;
    cout << " ERROR: NO TARGET SPECIFIED BY --DEFINE !" << endl;
    cout << " ERROR: NO TARGET SPECIFIED BY --DEFINE !" << endl;
  }
  cout << "   Ex corr.: y = " << ExCorrM << " x + " << ExCorrC << endl;
  cout << " -- -- -- -- -- -- -- -- -- -- --" << endl;

//  double simCentroid=-10.;
//  double simSigma = 0.104;
//  if(isSim){
//         if(NPOptionManager::getInstance()->HasDefinition("S0000")){ simCentroid = 0.000; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S1675")){ simCentroid = 1.675; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S3572")){ simCentroid = 3.572; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S4071")){ simCentroid = 4.701; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S5004")){ simCentroid = 5.004; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S5228")){ simCentroid = 5.228; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S5629")){ simCentroid = 5.629; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S7622")){ simCentroid = 7.622; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S7754")){ simCentroid = 7.754; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S8554")){ simCentroid = 8.554; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S8962")){ simCentroid = 8.962; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S9770")){ simCentroid = 9.770; }
//    else if(NPOptionManager::getInstance()->HasDefinition("S3575")){
//      simCentroid = 3.575; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      cout << "TESTING TESTING TESTING" << endl; 
//      }
//    else { 
//      cout << " ADD STATE ENERGY DEFINITION FOR CLEANUP OF SOLID ANGLE SIMULATION!!! " << endl;
//      cout << " ADD STATE ENERGY DEFINITION FOR CLEANUP OF SOLID ANGLE SIMULATION!!! " << endl;
//      cout << " ADD STATE ENERGY DEFINITION FOR CLEANUP OF SOLID ANGLE SIMULATION!!! " << endl;
//      cout << " ADD STATE ENERGY DEFINITION FOR CLEANUP OF SOLID ANGLE SIMULATION!!! " << endl;
//      cout << " ADD STATE ENERGY DEFINITION FOR CLEANUP OF SOLID ANGLE SIMULATION!!! " << endl;
//      cout << " ADD STATE ENERGY DEFINITION FOR CLEANUP OF SOLID ANGLE SIMULATION!!! " << endl;
//      cout << " ADD STATE ENERGY DEFINITION FOR CLEANUP OF SOLID ANGLE SIMULATION!!! " << endl;
//    }
//  }
//  cout << simCentroid << " +- " << simSigma << endl;

  agata_zShift=(49.0-0.63)*mm;
  //agata_zShift=51*mm;
  //BrhoRef=0.65;

  if(isSim && !isPhaseSpace){
cout << "here_InIsSimLoop" << endl;
    Initial = new TInitialConditions();
    ReactionConditions = new TReactionConditions(); 
    RootInput::getInstance()->GetChain()->SetBranchAddress("InitialConditions",&Initial);

    // Initilize MGX histograms, because must be inside function
    string base1 = "ThetaCM_detected_MG";
    string base2 = "ThetaLab_detected_MG";

    for(int i=0; i<7; i++){
      int j=i+1;
      string name1    = base1    + to_string(j);	  
      string name2    = base2    + to_string(j);	  

      ThetaCM_detected_MGX[i] 
         = new TH1F( name1.c_str()   , name1.c_str()   , NumThetaAngleBins, 0, 180 ); 
      ThetaLab_detected_MGX[i] 
         = new TH1F( name2.c_str()   , name2.c_str()   , NumThetaAngleBins, 0, 180 );
    }
  }

  // initialize input and output branches
  InitOutputBranch();
  InitInputBranch();
  
  MG = (TMugastPhysics*) m_DetectorManager -> GetDetector("Mugast");
  ML = (TModularLeafPhysics*) m_DetectorManager -> GetDetector("ModularLeaf"); 

  // get reaction information
  reaction.ReadConfigurationFile(NPOptionManager::getInstance()->GetReactionFile());
  OriginalBeamEnergy = reaction.GetBeamEnergy();
  reaction.Print(); //TESTING PARSER

  // get beam position from .reaction file
  Beam = (NPL::Beam*) reaction.GetParticle1(); 
  XBeam = Beam->GetMeanX();
  YBeam = Beam->GetMeanY();

  cout << " ---------------- Beam Position ---------------- " << endl;
  cout << " \tX = " << XBeam << " mm\tY = " << YBeam  << " mm" << endl;
  cout << " ----------------------------------------------- " << endl;

  // target thickness
  TargetThickness = m_DetectorManager->GetTargetThickness();
  string TargetMaterial = m_DetectorManager->GetTargetMaterial();

  // FOLLOWING LINES DO NOT WORK!
  // THE ARE FOR CRYOGENIC TARGETS! NOT SOLID TARGETS!
  //AuThickness = m_DetectorManager->GetBackThickness();
  //string BackMaterial = m_DetectorManager->GetBackMaterial();
  // HENCE, HAVE TO HARD-CODE Au THICKNESS HERE! NOT IDEAL!
  AuThickness = 0.0126; //12.6 um = 0.0126 mm
  
  cout << " ---------------- Target Values ---------------- " << endl;
  cout << " CD2: \tZ = " << m_DetectorManager->GetTargetZ() << " mm\tT = " << TargetThickness  << " mm" << endl;
  cout << " Au:  \tZ = " << m_DetectorManager->GetTargetZ() << " mm\tT = " << AuThickness  << " mm" << endl;
  cout << " ----------------------------------------------- " << endl;
  
  // energy losses
  string light = NPL::ChangeNameToG4Standard(reaction.GetParticle3()->GetName());
  string beam  = NPL::ChangeNameToG4Standard(reaction.GetParticle1()->GetName());
  string heavy = NPL::ChangeNameToG4Standard(reaction.GetParticle4()->GetName());

  if(isPhaseSpace){
    //then doesn't pull definitions properly
    // TEMP FIX::
    light = "proton";
    beam  = "O19";
    heavy = "O20";
  }

  LightTarget = NPL::EnergyLoss(light+"_"+TargetMaterial+".G4table","G4Table",100 );
  LightAl     = NPL::EnergyLoss(light+"_Al.G4table" ,"G4Table",100);
  LightSi     = NPL::EnergyLoss(light+"_Si.G4table" ,"G4Table",100);
  LightAu     = NPL::EnergyLoss(light+"_Au.G4table" ,"G4Table",100);
  BeamTarget  = NPL::EnergyLoss(beam+"_"+TargetMaterial+".G4table","G4Table",100);
  HeavyTarget = NPL::EnergyLoss(heavy+"_"+TargetMaterial+".G4table","G4Table",100);
  HeavyAu     = NPL::EnergyLoss(heavy+"_Au.G4table" ,"G4Table",100);

  /////////////////////////////////

  //Slowing of 19O in first half of CD2
  FinalBeamEnergy = BeamTarget.Slow(OriginalBeamEnergy, 0.5*TargetThickness, 0);
  reaction.SetBeamEnergy(FinalBeamEnergy); 

  cout << "\033[91m Running for reaction "
       << reaction.GetParticle1()->GetName() << "("
       << reaction.GetParticle2()->GetName() << ","
       << reaction.GetParticle3()->GetName() << ")"
       << reaction.GetParticle4()->GetName() << endl;

  cout << "\033[36m Beam energy at middle of CD2: " << FinalBeamEnergy << "\033[37m"<< endl;


  //Slowing of 20O in second half of CD2
  double afterCDBeamEnergy = HeavyTarget.Slow(FinalBeamEnergy, 0.5*TargetThickness, 0);
  cout << "\033[36m Beam energy after CD2: " << afterCDBeamEnergy << "\033[37m"<< endl;
  //Slowing of 20O in full Au thickness
  double afterAuBeamEnergy = HeavyAu.Slow(afterCDBeamEnergy, AuThickness, 0);
  cout << "\033[36m Beam energy after Au: " << afterAuBeamEnergy << "\033[37m"<< endl;

  /////////////////////////////////

  cout << "\033[36m Beta at middle of CD2: " 
       << reaction.GetEnergyImpulsionLab_4().Beta() << "\033[37m"<< endl;

  NPL::Reaction reactionTest = reaction;

  reactionTest.SetBeamEnergy(afterCDBeamEnergy);
  cout << "\033[36m Beta after CD2: " 
       << reactionTest.GetEnergyImpulsionLab_4().Beta() << "\033[37m"<< endl;
  reactionTest.SetBeamEnergy(afterAuBeamEnergy);
  cout << "\033[36m Beta after Au: " 
       << reactionTest.GetEnergyImpulsionLab_4().Beta() << "\033[37m"<< endl;

  /////////////////////////////////

  // initialize various parameters
  Rand = TRandom3();
  ParticleMult = 0;
  GammaMult = 0;
  DetectorNumber = 0;
  ThetaNormalTarget = 0;
  ThetaM2Surface = 0;
  ThetaMGSurface = 0;
  Si_E_M2 = 0;
  CsI_E_M2 = 0;
  Energy = 0;
  ThetaGDSurface = 0;
  elab_tmp = 0;
  thetalab_tmp = 0;
  philab_tmp = 0;
  ThetaGDSurface = 0;
  recoilBeta = 0;
  X.clear();
  Y.clear();
  Z.clear();
  dE = 0;
  BeamDirection = TVector3(0,0,1);
  //nbTrack=0;
  //nbHits=0;
  count=0;
  AHeavy=reaction.GetParticle4()->GetA();
  ALight=reaction.GetParticle3()->GetA(); 
  MHeavy=reaction.GetParticle4()->Mass();
  MLight=reaction.GetParticle3()->Mass();
  //bool writetoscreen=true;

  for(int i=0;i<GATCONF_SIZE;i++){ // loop over the bits
    GATCONF_Counter[i] = 0 ; 
  }

//ThetaCM_detected->Sumw2();
//ThetaLab_detected->Sumw2();
cout << "end_Init" << endl;
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::TreatEvent(){
//cout << "start_TreatEvent" << endl;

  // Reinitiate calculated variable
  ReInitValue();
  CalculateVamos();

  if(isSim && !isPhaseSpace){
    ThetaCM_emmitted->Fill(ReactionConditions->GetThetaCM());
    ThetaLab_emmitted->Fill(ReactionConditions->GetTheta(0));
  }
  
  GATCONF_MASTER=ML->GetCalibratedValue("GATCONF_MASTER");
  for(int i=0;i<GATCONF_SIZE;i++){ // loop over the bits
    if(GATCONF_MASTER & (unsigned int)pow(2,i)){ // test if ith bit is on
      GATCONF_Counter[i]++ ; // increment the array
      //      cout <<  std::bitset<16>(GATCONF_MASTER) << " " << i+1 << endl;
    }
  }

  if(isSim && !isPhaseSpace){
    OriginalELab = ReactionConditions->GetKineticEnergy(0);
    OriginalThetaLab = ReactionConditions->GetTheta(0);
    BeamEnergy = ReactionConditions->GetBeamEnergy();
  }

  TVector3 BeamDirection(0.,0.,1.);
  BeamImpact = TVector3(XBeam,YBeam,m_DetectorManager->GetTargetZ()); 

  ////////////////////////////////////////////////////////////////////////////
  //////////////////////////////// LOOP on MUGAST ////////////////////////////
  ////////////////////////////////////////////////////////////////////////////
  unsigned int sizeMG = MG->DSSD_E.size();
  //for(unsigned int countMugast = 0; countMugast<sizeMG; countMugast++){
  for(unsigned int countMugast = 0; countMugast<MG->DSSD_E.size(); countMugast++){

//cout << "start_MugastEvent" << endl;


// MOVING LATER TO ADD Ex LIMITATIONS
//  if(isSim && !isPhaseSpace){
//    int MGX = MG->TelescopeNumber[0]; 
//    // IGNORE ANNULAR EVENTS?
//    if(MGX==11){ break; }
//    MGX = MGX-1;

//    ThetaCM_detected_MG->Fill(ReactionConditions->GetThetaCM());
//    ThetaCM_detected_MGX[MGX]->Fill(ReactionConditions->GetThetaCM());

//    ThetaLab_detected_MG->Fill(ReactionConditions->GetTheta(0));
//    ThetaLab_detected_MGX[MGX]->Fill(ReactionConditions->GetTheta(0));
//  }

    // Part 1 : Impact Angle
    ThetaMGSurface = 0;
    ThetaNormalTarget = 0;
    thetalab_tmp = 0;
    philab_tmp = 0;
    E4_tmp = 0.0;

    TVector3 HitDirection = MG->GetPositionOfInteraction(countMugast) - BeamImpact;
    thetalab_tmp = HitDirection.Angle(BeamDirection);
    philab_tmp = HitDirection.Phi();

    X.push_back( MG -> GetPositionOfInteraction(countMugast).X());
    Y.push_back( MG -> GetPositionOfInteraction(countMugast).Y());
    Z.push_back( MG -> GetPositionOfInteraction(countMugast).Z());

//cout << " X =  " << MG -> GetPositionOfInteraction(countMugast).X() << "\t Y =  " << MG -> GetPositionOfInteraction(countMugast).Y() << endl;
//cout << MG -> GetPositionOfInteraction(countMugast).X() << "\t" << MG -> GetPositionOfInteraction(countMugast).Y() << endl;
//cout << "STRIP X & Y \t" << MG -> DSSD_X.back() << "\t" << MG -> DSSD_Y.back() << "\t" << MG->DSSD_X.back() + MG->DSSD_Y.back() << endl;

    ThetaMGSurface = HitDirection.Angle( MG -> GetTelescopeNormal(countMugast) );
    ThetaNormalTarget = HitDirection.Angle( TVector3(0.0,0.0,1.0) ) ;
    //ThetaNormalTarget = HitDirection.Angle( TVector3(0.0,0.0,-1.0) ) ;

    MGDet.push_back(MG->TelescopeNumber[0]);
    MGXStrip.push_back(MG->DSSD_X.back());
    MGYStrip.push_back(MG->DSSD_Y.back());

    // Part 2 : Impact Energy
    Energy = elab_tmp = 0;

    /* Below is from IZanon - presumably fixing some issue with the annular? */
    // removed, because I don't care about annular for unbounds
/**///if(MGDet.back()==11){
/**///  if(MGYStrip.back()==1 || MGYStrip.back()==2 || MGYStrip.back()==15 && MGYStrip.back()==16){
/**///    if(MGXStrip.back()>32 && MGXStrip.back()<49) Energy = MG->GetEnergyDeposit(countMugast);
/**///  }
/**///  else if(MGYStrip.back()>2 && MGYStrip.back()<7){
/**///    if(MGXStrip.back()>48 && MGXStrip.back()<65) Energy = MG->GetEnergyDeposit(countMugast);
/**///  }
/**///  else if(MGYStrip.back()>6 && MGYStrip.back()<11){
/**///    if(MGXStrip.back()>16 && MGXStrip.back()<50) Energy = MG->GetEnergyDeposit(countMugast);
/**///  }
/**///  else if(MGYStrip.back()>10 && MGYStrip.back()<15){
/**///    if(MGXStrip.back()>0 && MGXStrip.back()<17) Energy = MG->GetEnergyDeposit(countMugast);
/**///  }
/**///  else Energy = 0*MeV;
/**///}
/**///else Energy = MG->GetEnergyDeposit(countMugast);
    Energy = MG->GetEnergyDeposit(countMugast);

    double rawe = Energy; //for parallel line unbound state calc later, new addiiton
    RawEnergy.push_back(Energy);

//    if(!isSim){
      elab_tmp = LightAl.EvaluateInitialEnergy(
		    Energy,              //particle energy after Al
		    0.4*micrometer,      //thickness of Al
		    //ThetaMGSurface);     //angle of impingement
		    0.);     //angle of impingement
      ELoss_Al.push_back(Energy-elab_tmp);
      double elab_tmp2 = elab_tmp;
      elab_tmp = LightTarget.EvaluateInitialEnergy(
		    elab_tmp,            //particle energy after leaving target
		    TargetThickness*0.5, //distance passed through target
		    ThetaNormalTarget);  //angle of exit from target
      ELoss_Target.push_back(elab_tmp2-elab_tmp);
      ELoss.push_back(Energy-elab_tmp);
//    } else { //TESTING DIFFERENT ENERGY LOSSES IN SIMULATION
//      elab_tmp = Energy; //so I can add and remove sections
//      //elab_tmp = LightSi.EvaluateInitialEnergy(
//      //	    elab_tmp,            //particle energy after Si
//      //	    0.5*500.*micrometer, //thickness of Si
//      //	    ThetaMGSurface);     //angle of impingement
//      //elab_tmp = LightAl.EvaluateInitialEnergy(
//      //              elab_tmp,            //particle energy after Al
//      //	            0.4*micrometer,      //thickness of Al
//      //              ThetaMGSurface);     //angle of impingement
//      elab_tmp = LightTarget.EvaluateInitialEnergy(
//    		    elab_tmp,            //particle energy after leaving target
//    		    TargetThickness*0.5, //distance passed through target
//    		    ThetaNormalTarget);  //angle of exit from target
//    }

    ELab.push_back(elab_tmp);
    hasAuTarget.push_back(runningAuTarget);

///**/if(MG->TelescopeNumber.back()==1) thetalab_tmp = (thetalab_tmp/deg + 0.5)*deg;
///**/if(MG->TelescopeNumber.back()==2) thetalab_tmp = (thetalab_tmp/deg + 0.5)*deg;
///**/if(MG->TelescopeNumber.back()==3) thetalab_tmp = (thetalab_tmp/deg + 0.5)*deg;
///**/if(MG->TelescopeNumber.back()==4) thetalab_tmp = (thetalab_tmp/deg - 0.3)*deg;
///**/if(MG->TelescopeNumber.back()==5) thetalab_tmp = (thetalab_tmp/deg - 0.5)*deg;
///**/if(MG->TelescopeNumber.back()==6) thetalab_tmp = (thetalab_tmp/deg - 0.5)*deg;
///**/if(MG->TelescopeNumber.back()==7) thetalab_tmp = (thetalab_tmp/deg - 0.7)*deg;

//    // Part 3 : Excitation Energy Calculation
//    double slope11[16] = {1.02215,0.99258,1.,1.,1.,1.,1.,1.,1.,1., 1.02094, 1.023497, 1.01173, 1.02576, 1.04290, 1.01356};
//    double inter11[16] = {0.17232, 0.72019,0.,0.,0.,0.,0.,0.,0.,0., 0.25035, 0.00398, 0.13719, 0.02731, 0.22406, 0.18893};
//    Ex.push_back(reaction.ReconstructRelativistic(elab_tmp, thetalab_tmp, philab_tmp));
//    if(MGDet.back()==11){
//      ExCalib.push_back(Ex.back()*slope11[MGYStrip.back()-1]+inter11[MGYStrip.back()-1]);
//    }
//    else if(MGDet.back()==3) ExCalib.push_back(Ex.back()-0.08*MeV);
//    else if(MGDet.back()==5) ExCalib.push_back(Ex.back()+0.03*MeV);
//    else if(MGDet.back()==6) ExCalib.push_back(Ex.back()-0.02*MeV);
//    else ExCalib.push_back(Ex.back());

    //Ex.push_back(reaction.ReconstructRelativistic(elab_tmp, thetalab_tmp, philab_tmp));

    double ex_tmp;
    if(isSim){
      //No adjustment to Ex
      ex_tmp = reaction.ReconstructRelativistic(elab_tmp, thetalab_tmp);
    } else {
      //Adjustment to Ex, using straight line calibration
      ex_tmp = (reaction.ReconstructRelativistic(elab_tmp, thetalab_tmp, philab_tmp) / ExCorrM) - ExCorrC;
    }
    Ex.push_back(ex_tmp);
    Ecm.push_back(elab_tmp*(AHeavy+ALight)/(4*AHeavy*cos(thetalab_tmp)*cos(thetalab_tmp)));
    
    // Part 4 : Theta CM Calculation
    ThetaLab.push_back(thetalab_tmp/deg);
    PhiLab.push_back(philab_tmp/deg);
    ThetaCM.push_back(reaction.EnergyLabToThetaCM(elab_tmp, thetalab_tmp)/deg);

    if(sizeMG==1){
      MG_T = MG->DSSD_T[0];
      MG_E = MG->DSSD_E[0];
      MG_X = MG->DSSD_X[0];
      MG_Y = MG->DSSD_Y[0];
      MG_D = MG->TelescopeNumber[0];
    }

    if(isSim && !isPhaseSpace){
      int MGX = MG->TelescopeNumber[0]; 
      // IGNORE ANNULAR EVENTS?
      if(MGX==11){ break; }
      MGX = MGX-1;

      // REJECT POOR EVENTS (PUNCHTHROUGH ETC.)
      //if(abs(ex_tmp - simCentroid)<3*simSigma){  
        ThetaCM_detected_MG->Fill(ReactionConditions->GetThetaCM());
        ThetaCM_detected_MGX[MGX]->Fill(ReactionConditions->GetThetaCM());
  
        ThetaLab_detected_MG->Fill(ReactionConditions->GetTheta(0));
        ThetaLab_detected_MGX[MGX]->Fill(ReactionConditions->GetTheta(0));
      //}
    }

  double HeavyThetaLab_tmp = reaction.GetEnergyImpulsionLab_4().Theta();
  double HeavyPhiLab_tmp = reaction.GetEnergyImpulsionLab_4().Phi();
  HeavyThetaLab.push_back(HeavyThetaLab_tmp/deg);
  HeavyPhiLab.push_back(HeavyPhiLab_tmp/deg);

  auto tmpHeavy = new TVector3(1,1,1); 
  tmpHeavy->SetTheta(HeavyThetaLab_tmp*(180./M_PI));
  tmpHeavy->SetPhi(HeavyPhiLab_tmp*(180./M_PI));
  tmpHeavy->SetMag(WindowDistance/cos(HeavyThetaLab_tmp));//cos=A/H, using average of MUST2 corners as 'Z'

/**/short OffToF[12] = {0,-1800,-1400,-1450,-2100,-3580,-1300,-2820,0,0,0,0};
/**/int NDet;
/**/NDet = MG->TelescopeNumber.back();
/**/for(int ii = 0; ii<12 ; ii++)	ToF = T_MUGAST_FPMW + OffToF[NDet];

//cout << "end_MugastEvent" << endl;
  }//end loop Mugast


  ////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////// LOOP on AGATA ////////////////////////////
  ////////////////////////////////////////////////////////////////////////////
  //********* Taking this section basically completely from IZanon *********//
  //********* Taking this section basically completely from IZanon *********//
  //********* Taking this section basically completely from IZanon *********//

  for(int j = 0; j < nbAdd; j++){ // all multiplicity

    // Measured E
    double Egamma=AddE[j]/1000.; // From keV to MeV 
    TVector3 GammaHit(AddX[j],AddY[j],AddZ[j]+agata_zShift); 
    //GammaHit.RotateZ(265.401*TMath::DegToRad());
    GammaHit.RotateZ(263.348*TMath::DegToRad());	
    TVector3 GammaDirection = GammaHit-BeamImpact;
    GammaDirection = GammaDirection.Unit();

    // Construct LV in lab frame
    TLorentzVector GammaLVM;
    GammaLVM.SetPx(Egamma*GammaDirection.X());
    GammaLVM.SetPy(Egamma*GammaDirection.Y());
    GammaLVM.SetPz(Egamma*GammaDirection.Z());
    GammaLVM.SetE(Egamma);
    
    // Test EDC calculated taking into account the recoil direction (Irene and Alain)
    recoilBeta = reaction.GetNucleus4()->GetEnergyImpulsion().Beta();

    /*		E4_tmp = reaction.GetEnergyImpulsionLab_4().E()-reaction.GetNucleus4()->Mass();
		TLorentzVector LVector4_tmp=reaction.GetEnergyImpulsionLab_4();
		std::cout << LVector4_tmp.Beta() << "\t\t" << reaction.GetNucleus4()->GetEnergyImpulsion().Beta() << std::endl;
		TVector3 imp4=LVector4_tmp.Vect();
		imp4.SetMag(sqrt((E4_tmp+reaction.GetNucleus4()->Mass())*(E4_tmp+reaction.GetNucleus4()->Mass())-reaction.GetNucleus4()->Mass()*reaction.GetNucleus4()->Mass()));
		TLorentzVector LVector4(imp4,E4_tmp+reaction.GetNucleus4()->Mass());
		recoilBeta = LVector4.Beta();
		recoilDir = LVector4.Vect();*/

    beta_MUGAST.push_back(recoilBeta);
    recoilDir  = reaction.GetNucleus4()->GetEnergyImpulsion().Vect();

    //mRecoilDist->Fill(recoilDir.Phi()*TMath::RadToDeg(), recoilDir.Theta()*TMath::RadToDeg());

    ThetaGR.push_back(GammaDirection.Angle(recoilDir));
    gammaEDC.push_back(Egamma*((1-recoilBeta*TMath::Cos(ThetaGR.back()))/TMath::Sqrt(1-recoilBeta*recoilBeta)));
    ThetaG.push_back(GammaDirection.Theta()/deg);
    PhiG.push_back(GammaDirection.Phi()/deg);

    // EDC Optimization
    //if(T_MUGAST_FPMW > 13e3 && T_MUGAST_FPMW < 26e3 && T_FPMW_HF > 12.1e3 && T_FPMW_HF < 13.2e3){
/*    if(T_MUGAST_FPMW > 7e3 && T_MUGAST_FPMW < 20e3 && T_FPMW_HF > 13.7e3 && T_FPMW_HF < 14.8e3){
      TVector3 gammaDir = GammaDirection; 
      double tmpGR = 0;
      for(UInt_t j = 0 ; j < 360 ; j++){
      	gammaDir.RotateZ(1*TMath::DegToRad());
      	tmpGR = gammaDir.Angle(recoilDir);
      	mEDC_PhiAgata->Fill(j,
			    Egamma*1000*((1-recoilBeta*TMath::Cos(tmpGR))/TMath::Sqrt(1-recoilBeta*recoilBeta)));
      }
      // GammaDirection is a unitary vector so offset does not work;
      gammaDir = GammaHit;
      TVector3 zOffset ;
      TVector3 tmpGamma;
      for(Int_t j = -10 ; j <=10 ; j++){
      	zOffset.SetXYZ(0,0,j);
      	tmpGamma = gammaDir+zOffset;
      	//std::cout << "Z = " << tmpGamma.Z() << " = " << gammaDir.Z() << "+ " << zOffset.Z() << std::endl; 
      	tmpGR = tmpGamma.Angle(recoilDir);
      	mEDC_OffsetAgata->Fill(j,
			       Egamma*1000*((1-recoilBeta*TMath::Cos(tmpGR))/TMath::Sqrt(1-recoilBeta*recoilBeta)));
      }

    }
*/    
   
    //Test (does not work): Energy loss in Au for 20O
    E4_tmp = reaction.GetEnergyImpulsionLab_4().E()-reaction.GetNucleus4()->Mass();
    Energy4 = E4_tmp;
    //if(T_MUGAST_FPMW > 13e3 && T_MUGAST_FPMW < 26e3 && T_FPMW_HF > 12.1e3 && T_FPMW_HF < 13.2e3){std::cout << E4_tmp;}
    //cout << E4_tmp << "\t";
    E4_tmp = HeavyTarget.Slow(E4_tmp, TargetThickness*0.5,Theta4*deg);
    //if(T_MUGAST_FPMW > 13e3 && T_MUGAST_FPMW < 26e3 && T_FPMW_HF > 12.1e3 && T_FPMW_HF < 13.2e3){std::cout << "\t" << E4_tmp;}
    //cout << E4_tmp << "\t";
    //cout << Theta4 << "\t";
    E4_tmp = HeavyAu.Slow(E4_tmp, AuThickness, Theta4*deg);

    //cout <<Theta4 <<  "\t\t" << AuThickness << "\t" << AuThickness/TMath::Cos(Theta4*deg) << endl;
    //if(T_MUGAST_FPMW > 13e3 && T_MUGAST_FPMW < 26e3 && T_FPMW_HF > 12.1e3 && T_FPMW_HF < 13.2e3 && ExCalib.back()>1.5&& ExCalib.back()<5.5)
    //cout << reaction.GetEnergyImpulsionLab_4().E()-reaction.GetNucleus4()->Mass() << "\t" << E4_tmp << "\n";		    

    TLorentzVector LVector4_tmp_1=reaction.GetEnergyImpulsionLab_4();
    TVector3 imp4_1=LVector4_tmp_1.Vect();
    imp4_1.SetMag(sqrt((E4_tmp+reaction.GetNucleus4()->Mass())*(E4_tmp+reaction.GetNucleus4()->Mass())-reaction.GetNucleus4()->Mass()*reaction.GetNucleus4()->Mass()));
    TLorentzVector LVector4_1(imp4_1,E4_tmp+reaction.GetNucleus4()->Mass()); 
    //double bb;
    //bb = sqrt(1-(reaction.GetNucleus4()->Mass()*reaction.GetNucleus4()->Mass()/(E4_tmp+reaction.GetNucleus4()->Mass())/(E4_tmp+reaction.GetNucleus4()->Mass()))); 
    //std::cout << "Beta1 " << LVector4_1.Beta() << "\tBeta2 " << bb << std::endl;
    //if(T_MUGAST_FPMW > 13e3 && T_MUGAST_FPMW < 26e3 && T_FPMW_HF > 12.1e3 && T_FPMW_HF < 13.2e3){std::cout << "\t" << E4_tmp << std::endl;}
    //cout<<"E4_tmp="<<E4_tmp<<"  E4 reaction="<<reaction.GetEnergyImpulsionLab_4().E()-reaction.GetNucleus4()->Mass()<<"   E4Lorentz="<< LVector4.E()-reaction.GetNucleus4()->Mass();
    //cout<<LVector4.Beta()<<endl;

    double beta_fix = 0.126;
    EDC_MUGAST.push_back(Egamma*((1-beta_fix*TMath::Cos(ThetaG.back()*deg))/TMath::Sqrt(1-beta_fix*beta_fix)));

    // Boost back in CM
    /*    TVector3 betaM2=reaction.GetEnergyImpulsionLab_4().BoostVector();
	  TVector3 betaM=LVector4.BoostVector();
	  betaM.SetPhi(HitDirection.Phi()+3.14159);
	  betaM.SetMag(0.125);
	  //cout<<"betaM="<<betaM.Mag()<<" beta original="<<betaM2.Mag()<<endl;
	  beta_MUGAST.push_back(betaM.Mag());
	  betaM=TVector3(0,0,0.125);
	  GammaLVM.Boost(-betaM);
	  ThetaGamma.push_back(betaM.Angle(GammaLVM.BoostVector())/deg);
	  // Get EDC
	  EDC_MUGAST.push_back(GammaLVM.Energy());
	  // Construct LV in lab frame
	  double ThetaGammaV = GammaDirection.Angle(BeamDirection)/deg;
	  TLorentzVector GammaLVV;
	  GammaLVV.SetPx(Egamma*GammaDirection.X());
	  GammaLVV.SetPy(Egamma*GammaDirection.Y());
	  GammaLVV.SetPz(Egamma*GammaDirection.Z());
	  GammaLVV.SetE(Egamma);
	  // Boost back in CM
	  TVector3 betaV(0,0,-mBeta); */
    beta_VAMOS.push_back(LVector4_1.Beta());
    //std::cout << beta_VAMOS.back() << "\t\t" << recoilBeta-beta_VAMOS.back() << std::endl;
    //GammaLVV.Boost(betaV);
    //EDC_VAMOS.push_back(GammaLVV.Energy());

    //IRENE's TEST WITH ENERGY LOSS IN GOLD

    vamosEDC.push_back(Egamma*((1-beta_VAMOS.back()*TMath::Cos(ThetaGR.back()))/TMath::Sqrt(1-beta_VAMOS.back()*beta_VAMOS.back())));

		//Graph for checking degrader thickness
		/*
	  if(T_MUGAST_FPMW > 13e3 && T_MUGAST_FPMW < 26e3 && T_FPMW_HF > 12.1e3 && T_FPMW_HF < 13.2e3 && Ex.back()>1.0 && Ex.back()<5.5){
			double tmpGR = 0;
			TVector3 gammaDir = GammaDirection;

			E4_tmp = reaction.GetEnergyImpulsionLab_4().E()-reaction.GetNucleus4()->Mass();
			E4_tmp = HeavyTarget.Slow(E4_tmp, TargetThickness*0.5,Theta4*deg);

			for(Int_t j = 0 ; j < 30 ; j++){
				double off = 0.;
				//off = (j/10.)*micrometer;
				off = j/10.*micrometer;
				double E4_fix = HeavyAu.Slow(E4_tmp, AuThickness+off, Theta4*deg);
				TLorentzVector LVector4_tmp=reaction.GetEnergyImpulsionLab_4();
				TVector3 imp4=LVector4_tmp.Vect();
				imp4.SetMag(sqrt((E4_fix+reaction.GetNucleus4()->Mass())*(E4_fix+reaction.GetNucleus4()->Mass())-reaction.GetNucleus4()->Mass()*reaction.GetNucleus4()->Mass()));
				TLorentzVector LVector4(imp4,E4_fix+reaction.GetNucleus4()->Mass()); 
				double beta4 = LVector4.Beta();
	
				tmpGR = gammaDir.Angle(recoilDir);
				backingThick->Fill((AuThickness+off)/micrometer+0.025,Egamma*1000*((1-beta4*TMath::Cos(tmpGR))/TMath::Sqrt(1-beta4*beta4)));
				//printf("%13.5f %13.5f\n",(AuThickness+off)/micrometer,Egamma*1000*((1-beta4*TMath::Cos(tmpGR))/TMath::Sqrt(1-beta4*beta4)));
			}
		}*/		//fine grafico
  }

  //////////////////// TRACKING //////////////////////

  for(int j = 0; j < nbTrack; j++){ // all multiplicity

    // Measured E 
    double Etrack=trackE[j]/1000.;
    TVector3 TrackHit(trackX1[j],trackY1[j],trackZ1[j]+agata_zShift-0.578); 
    TrackHit.RotateZ(272.235*TMath::DegToRad());
    TVector3 GammaDirection = TrackHit-BeamImpact;
    GammaDirection = GammaDirection.Unit();
    if(j==0) radius = TrackHit.Mag();
    
    // Construct LV in lab frame
    TLorentzVector TrackLVM;
    TrackLVM.SetPx(Etrack*GammaDirection.X());
    TrackLVM.SetPy(Etrack*GammaDirection.Y());
    TrackLVM.SetPz(Etrack*GammaDirection.Z());
    TrackLVM.SetE(Etrack);
    
    // Test EDC calculated taking into account the recoil direction (Irene and Alain)
    //////recoilBeta = reaction.GetNucleus4()->GetEnergyImpulsion().Beta();
    //////recoilDir  = reaction.GetNucleus4()->GetEnergyImpulsion().Vect();
    recoilBeta = reaction.GetEnergyImpulsionLab_4().Beta();
    recoilDir  = reaction.GetEnergyImpulsionLab_4().Vect();
    ThetaGR.push_back(GammaDirection.Angle(recoilDir));

    trackEDC.push_back(Etrack*((1-recoilBeta*TMath::Cos(ThetaGR.back()))/TMath::Sqrt(1-recoilBeta*recoilBeta)));

/*   E4_tmp = reaction.GetEnergyImpulsionLab_4().E()-reaction.GetNucleus4()->Mass();
    //Energy4 = E4_tmp;
    E4_tmp = HeavyTarget.Slow(E4_tmp, TargetThickness*0.5,Theta4*deg);
    E4_tmp = HeavyAu.Slow(E4_tmp, AuThickness, Theta4*deg);
    TLorentzVector LVector4_tmp_1=reaction.GetEnergyImpulsionLab_4();
    TVector3 imp4_1=LVector4_tmp_1.Vect();
    imp4_1.SetMag(sqrt((E4_tmp+reaction.GetNucleus4()->Mass())*(E4_tmp+reaction.GetNucleus4()->Mass())-reaction.GetNucleus4()->Mass()*reaction.GetNucleus4()->Mass()));
    TLorentzVector LVector4_1(imp4_1,E4_tmp+reaction.GetNucleus4()->Mass()); */
    goldEDC.push_back(Etrack*((1-beta_VAMOS.back()*TMath::Cos(ThetaGR.back()))/TMath::Sqrt(1-beta_VAMOS.back()*beta_VAMOS.back())));


    /*   	double tmpGR = 0;
		TVector3 gammaDir = GammaDirection; 
		for(UInt_t j = 0 ; j < 360 ; j++){
		gammaDir.RotateZ(1*TMath::DegToRad());
		tmpGR = gammaDir.Angle(recoilDir);
		mEDC_PhiAgata->Fill(j,Etrack*1000*((1-recoilBeta*TMath::Cos(tmpGR))/TMath::Sqrt(1-recoilBeta*recoilBeta)));
		}
		// GammaDirection is a unitary vector so offset does not work;
		gammaDir = TrackHit;
		TVector3 zOffset ;
		TVector3 tmpGamma;
		for(Int_t j = -10 ; j <=10 ; j++){
		zOffset.SetXYZ(0,0,j);
		tmpGamma = gammaDir+zOffset;
		//std::cout << "Z = " << tmpGamma.Z() << " = " << gammaDir.Z() << "+ " << zOffset.Z() << std::endl; 
		tmpGR = tmpGamma.Angle(recoilDir);
		mEDC_OffsetAgata->Fill(j,
		Etrack*1000*((1-recoilBeta*TMath::Cos(tmpGR))/TMath::Sqrt(1-recoilBeta*recoilBeta)));
		}*/

  }

  //	radius = TMath::Sqrt(trackX1[0]*trackX1[0]+trackY1[0]*trackY1[0]+(trackZ1[0]+*trackZ1[0]);



  // Agata add back is not always multiplicity 1 ?? NO, not necessarily!
  if(nbAdd==1){
    TLorentzVector GammaLV;
    // Measured E
    double Egamma=AddE[0]/1000.; // From keV to MeV 
    // Gamma detection position
    // TrackZ1 to be corrected there is a shift of +51mm
    TVector3 GammaHit(AddX[0],AddY[0],AddZ[0]+agata_zShift); 
    // TVector3 GammaHit(trackX1[0],trackY1[0],trackZ1[0]);
    TVector3 GammaDirection = GammaHit-BeamImpact;
    GammaDirection = GammaDirection.Unit();
    // Beta from Two body kinematic
    //TVector3 beta = reaction.GetEnergyImpulsionLab_4().BoostVector();
    // Beta from the Beam mid target 
    //reaction.GetKinematicLine4();
    TVector3 beta(0,0,-reaction.GetNucleus4()->GetBeta());
    // THIS IS THE ONE USED BY IZANON? -> // TVector3 beta(0,0,-mBeta);
    
    RawGammaEnergy.push_back(Egamma);
    BetaUsedByAddBackEDC.push_back(beta.Z());

    double ThetaGamma = GammaDirection.Angle(BeamDirection)/deg;

    // Construct LV in lab frame
    GammaLV.SetPx(Egamma*GammaDirection.X());
    GammaLV.SetPy(Egamma*GammaDirection.Y());
    GammaLV.SetPz(Egamma*GammaDirection.Z());
    GammaLV.SetE(Egamma);
    // Boost back in CM
    GammaLV.Boost(beta);
    // Get EDC
    AddBack_EDC.push_back(GammaLV.Energy());
  }


//  // Agata by track
//  /*
//  for(int j=0; j<nbTrack; j++){ // all multiplicity
//    TLorentzVector GammaLV;
//    // Measured E
//    double Egamma=trackE[j]/1000.; // From keV to MeV 
//    // Gamma detection position
//    // TrackZ1 to be corrected there is a shift of +51mm
//    TVector3 GammaHit(trackX1[j],trackY1[j],trackZ1[j]+agata_zShift); 
//    // TVector3 GammaHit(trackX1[0],trackY1[0],trackZ1[0]); 
//    // Gamma Direction 
//    TVector3 GammaDirection = GammaHit-BeamImpact;
//    GammaDirection = GammaDirection.Unit();
//    // Beta from Two body kinematic
//    //TVector3 beta = reaction.GetEnergyImpulsionLab_4().BoostVector();
//    // Beta from the Beam mid target 
//    reaction.GetKinematicLine4();
//    TVector3 beta(0,0,-reaction.GetNucleus4()->GetBeta());
//
//    double ThetaGamma = GammaDirection.Angle(BeamDirection)/deg;
//    // Construct LV in lab frame
//    GammaLV.SetPx(Egamma*GammaDirection.X());
//    GammaLV.SetPy(Egamma*GammaDirection.Y());
//    GammaLV.SetPz(Egamma*GammaDirection.Z());
//    GammaLV.SetE(Egamma);
//    // Boost back in CM
//    GammaLV.Boost(beta);
//    // Get EDC
//    EDC.push_back(GammaLV.Energy());
//  }
//  */
//
//  // Agata add back is not always multiplicity 1 ?? NO, not necessarily!
//  for(int i=0; i<nbAdd; i++){
//  //if(nbAdd==1){
//    TLorentzVector GammaLV;
//    // Measured E
//    double Egamma = AddE[i]/1000.; // From keV to MeV 
//    // Gamma detection position
//    // TrackZ1 to be corrected there is a shift of +51mm
//    TVector3 GammaHit(AddX[i],AddY[i],AddZ[i]+agata_zShift); 
//    // TVector3 GammaHit(trackX1[0],trackY1[0],trackZ1[0]); 
//
//
////    cout << "gDir " << GammaHit.X() << " "
////                    << GammaHit.Y() << " "
////                    << GammaHit.Z() << " ";
//    /* TEST adding constant to Phi */
//    //double phitemp = GammaHit.Phi();
//    //double phiconst = -2.0;
//    //GammaHit.SetPhi(phitemp+phiconst);
////    GammaHit.RotateZ(0.0);
//
////    cout << "gDir " << GammaHit.X() << " "
////                    << GammaHit.Y() << " "
////                    << GammaHit.Z() << endl;
//
//    // Gamma Direction 
//    TVector3 GammaDirection = GammaHit-BeamImpact;
//    GammaDirection = GammaDirection.Unit();
//
//
//    /* Beta from Two body kinematic */
//    TVector3 beta = reaction.GetEnergyImpulsionLab_4().BoostVector();
//    //TVector3 beta = reaction->GetEnergyImpulsionLab_4().BoostVector();
//
////    cout << "bDir " << beta.X() << " "
////                    << beta.Y() << " "
////                    << beta.Z() ;
////    beta.RotateX(0.002847); beta.RotateY(3.144869); beta.RotateZ(0.095923);
//    beta.RotateY(M_PI);
////    cout << "bDir " << beta.X() << " "
////                    << beta.Y() << " "
////                    << beta.Z() << endl;
//
//    /* Original beta */
////    TVector3 beta(0,0,-0.1257);
//
//    // For beta rotation minimization
//    AGATA_Theta.push_back(GammaDirection.Theta());
//    AGATA_Phi.push_back(GammaDirection.Phi());
//    AGATA_GammaPx.push_back(Egamma*GammaDirection.X());
//    AGATA_GammaPy.push_back(Egamma*GammaDirection.Y());
//    AGATA_GammaPz.push_back(Egamma*GammaDirection.Z());
//    AGATA_GammaE.push_back(Egamma);
//    AGATA_OrigBetaX.push_back(beta.X());
//    AGATA_OrigBetaY.push_back(beta.Y());
//    AGATA_OrigBetaZ.push_back(beta.Z());
//    AGATA_OrigBetaMag.push_back(beta.Mag());
//
//    /* Other fills */
//    double ThetaGamma = GammaDirection.Angle(BeamDirection)/deg;
//    // Construct LV in lab frame
//    GammaLV.SetPx(Egamma*GammaDirection.X());
//    GammaLV.SetPy(Egamma*GammaDirection.Y());
//    GammaLV.SetPz(Egamma*GammaDirection.Z());
//    GammaLV.SetE(Egamma);
//    // Boost back in CM
//    GammaLV.Boost(beta);
//    // Get EDC
//    AddBack_EDC.push_back(GammaLV.Energy());
//
//    if(i==0){
//      //First event in loop
//      AddBack_EDC_Event1.push_back(GammaLV.Energy());
//    } else {
//      //second, third, fourth...
//      AddBack_EDC_Event2.push_back(GammaLV.Energy());
//    }
//
//  }
//
//  float offset[21] = {0.,27.4,46.7,41.1,71.2,58.7,32.1,40.4,-46.3,15.1,9.9,-39.1,-39.8,-5.3,18.5,0.7,-28.5,-7,-22,-8.5,-8.5};
//  float slope[21] = {1.,0.96,0.93,0.93,0.89,0.91,0.95,0.93,1.06,0.97,0.99,1.05,1.04,1.01,0.97,1.,1.03,0.99,1.,1.02,1.02};
//
//  for(unsigned int i=0; i<MW_Nr; i++){
//    MWT[i] = -1500;
//    for(unsigned short j=0; j<21; j++)
//      if(MW_N[i]==j)
//        MWT[i] = (MW_T[i]-offset[j])/slope[j];
//  }


//cout << "end_TreatEvent" << endl;
}

////////////////////////////////////////////////////////////////////////////////
void FillSolidAngles(TH1F* hSA, TH1F* hDet, TH1F* hEmm){
  if(!filledCline){
    cout << RED << "FILLING CLINE VECTOR! SHOULD ONLY OCCUR ONCE!" << RESET << endl;
    for(int t=0; t<NumThetaAngleBins; t++){
      double angleMin = (t)*(180./NumThetaAngleBins);
      double angleMax = (t+1)*(180./NumThetaAngleBins);
      cout << " Angle " << angleMin 
	      << " to " << angleMax
	      << endl; 
      clineVal.push_back(2.0*M_PI*(cos(angleMin*degtorad) - cos(angleMax*degtorad)));
      clineX.push_back(angleMin+((angleMax-angleMin)/2.0));
      filledCline=true;
    }
  }

  for (int i = 0; i < hSA->GetNbinsX(); ++i){
    double val = hDet->GetBinContent(i) / hEmm->GetBinContent(i);
    double valerr = val * sqrt( 
      pow(hDet->GetBinError(i) / hDet->GetBinContent(i), 2) +
      pow(hEmm->GetBinError(i) / hEmm->GetBinContent(i), 2) );
    if (isnan(val)) { val = 0; valerr = 0; }
    val *= clineVal.at(i);
    valerr *= clineVal.at(i);

    hSA->SetBinContent(i, val);
    hSA->SetBinError(i, valerr);
  }
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::End(){
  cout << endl << "\e[1;32m GATCONF Statistics " << endl ;
  for(int i=0;i<GATCONF_SIZE;i++){ // loop over the bits
    cout << GATCONF_Label[i]  << "\t\t" << GATCONF_Counter[i] << endl ; 
  }
  cout << endl ;

  if(isSim && !isPhaseSpace){
    
    //TObjArray HistList(0);
    TList *HistList = new TList();

    // MUGAST
    auto Efficiency_CM_MG = new TH1F(*ThetaCM_detected_MG);
    Efficiency_CM_MG->SetName("Efficiency_CM_MG");
    Efficiency_CM_MG->SetTitle("Efficiency_CM_MG");
    Efficiency_CM_MG->Sumw2();
    auto Efficiency_Lab_MG = new TH1F(*ThetaLab_detected_MG);
    Efficiency_Lab_MG->SetName("Efficiency_Lab_MG");
    Efficiency_Lab_MG->SetTitle("Efficiency_Lab_MG");
    Efficiency_Lab_MG->Sumw2();
     
    auto SolidAngle_CM_MG = new TH1F(*ThetaCM_detected_MG);
    SolidAngle_CM_MG->SetName("SolidAngle_CM_MG");
    SolidAngle_CM_MG->SetTitle("SolidAngle_CM_MG");
    auto SolidAngle_Lab_MG = new TH1F(*ThetaLab_detected_MG);
    SolidAngle_Lab_MG->SetName("SolidAngle_Lab_MG");
    SolidAngle_Lab_MG->SetTitle("SolidAngle_Lab_MG");

    Efficiency_CM_MG->Divide(ThetaCM_emmitted);
    Efficiency_Lab_MG->Divide(ThetaLab_emmitted);

    /* Testing method for better errors in SolidAngle histograms */
    FillSolidAngles(SolidAngle_CM_MG, ThetaCM_detected_MG, ThetaCM_emmitted);
    FillSolidAngles(SolidAngle_Lab_MG, ThetaLab_detected_MG, ThetaLab_emmitted);

    HistList->Add(ThetaCM_detected_MG);
    HistList->Add(ThetaLab_detected_MG);
    HistList->Add(Efficiency_CM_MG);
    HistList->Add(Efficiency_Lab_MG);
    HistList->Add(SolidAngle_CM_MG);
    HistList->Add(SolidAngle_Lab_MG);

    // MUGAST INDIVIDUAL
    TH1F *SolidAngle_CM_MGX[7];
    string base1 = "SolidAngle_CM_MG";
    for(int i=0;i<7;i++){
      int j=i+1;
      string name = base1 + to_string(j);	  
      SolidAngle_CM_MGX[i] = new TH1F(*ThetaCM_detected_MGX[i]);
      SolidAngle_CM_MGX[i]->SetName(name.c_str());
      SolidAngle_CM_MGX[i]->SetTitle(name.c_str());
      FillSolidAngles(SolidAngle_CM_MGX[i], ThetaCM_detected_MGX[i], ThetaCM_emmitted);
    }

    TH1F *SolidAngle_Lab_MGX[7];
    string base2 = "SolidAngle_Lab_MG";
    for(int i=0;i<7;i++){
      int j=i+1;
      string name = base2 + to_string(j);	  
      SolidAngle_Lab_MGX[i] = new TH1F(*ThetaLab_detected_MGX[i]);
      SolidAngle_Lab_MGX[i]->SetName(name.c_str());
      SolidAngle_Lab_MGX[i]->SetTitle(name.c_str());
      FillSolidAngles(SolidAngle_Lab_MGX[i], ThetaLab_detected_MGX[i], ThetaLab_emmitted);
    }

    for(int i=0; i<7; i++){HistList->Add(ThetaCM_detected_MGX[i]);}
    for(int i=0; i<7; i++){HistList->Add(ThetaLab_detected_MGX[i]);}

    for(int i=0; i<7; i++){HistList->Add(SolidAngle_CM_MGX[i]);}
    for(int i=0; i<7; i++){HistList->Add(SolidAngle_Lab_MGX[i]);}
    
    auto clineValGraph = new TGraph(NumThetaAngleBins);
    clineValGraph->SetName("clineValGraph");
    clineValGraph->SetTitle("clineValGraph");
    for(int b=0; b<NumThetaAngleBins; b++){
      clineValGraph->SetPoint(b,clineX.at(b),clineVal.at(b));
    }
    HistList->Add(clineValGraph);

    auto HistoFile = new TFile("~/Programs/nptoolv3/Projects/e775s/analysis/SolidAngleHistFiles/SAHF_New.root","RECREATE");
    HistList->Write();
    HistoFile->Close();


    cout << "!!! MAKE SURE YOU RENAME THE SOLID ANGLE FILE! !!!" << endl;
  }
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::InitOutputBranch(){
cout << "start_InitOutput" << endl;
  RootOutput::getInstance()->GetTree()->Branch("hasAuTarget",&hasAuTarget);
  

  //RootOutput::getInstance()->GetTree()->Branch("Ex",&Ex,"Ex/D");
  RootOutput::getInstance()->GetTree()->Branch("Ex",&Ex);
  RootOutput::getInstance()->GetTree()->Branch("ExCalib",&ExCalib);
  //RootOutput::getInstance()->GetTree()->Branch("EDC",&EDC,"EDC/D");
  RootOutput::getInstance()->GetTree()->Branch("EDC_VAMOS",&EDC_VAMOS);
  RootOutput::getInstance()->GetTree()->Branch("EDC_MUGAST",&EDC_MUGAST);
  RootOutput::getInstance()->GetTree()->Branch("beta_VAMOS",&beta_VAMOS);
  RootOutput::getInstance()->GetTree()->Branch("beta_MUGAST",&beta_MUGAST);
  RootOutput::getInstance()->GetTree()->Branch("ToF",&ToF);

  RootOutput::getInstance()->GetTree()->Branch("RawGammaEnergy",&RawGammaEnergy);
  RootOutput::getInstance()->GetTree()->Branch("BetaUsedByAddBackEDC",&BetaUsedByAddBackEDC);
  RootOutput::getInstance()->GetTree()->Branch("AddBack_EDC",&AddBack_EDC);
  RootOutput::getInstance()->GetTree()->Branch("AddBack_EDC_Event1",&AddBack_EDC_Event1);
  RootOutput::getInstance()->GetTree()->Branch("AddBack_EDC_Event2",&AddBack_EDC_Event2);
  RootOutput::getInstance()->GetTree()->Branch("gammaEDC",&gammaEDC);
  RootOutput::getInstance()->GetTree()->Branch("trackEDC",&trackEDC);
  RootOutput::getInstance()->GetTree()->Branch("vamosEDC",&vamosEDC);
  RootOutput::getInstance()->GetTree()->Branch("goldEDC",&goldEDC);
  RootOutput::getInstance()->GetTree()->Branch("ThetaG",&ThetaG);
  RootOutput::getInstance()->GetTree()->Branch("radius",&radius);
  RootOutput::getInstance()->GetTree()->Branch("PhiG",&PhiG);		
  RootOutput::getInstance()->GetTree()->Branch("AGATA_Theta",&AGATA_Theta);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_Phi",&AGATA_Phi);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_GammaPx",&AGATA_GammaPx);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_GammaPy",&AGATA_GammaPy);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_GammaPz",&AGATA_GammaPz);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_GammaE",&AGATA_GammaE);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_OrigBetaX",&AGATA_OrigBetaX);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_OrigBetaY",&AGATA_OrigBetaY);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_OrigBetaZ",&AGATA_OrigBetaZ);
  RootOutput::getInstance()->GetTree()->Branch("AGATA_OrigBetaMag",&AGATA_OrigBetaMag);
  RootOutput::getInstance()->GetTree()->Branch("EAgata",&EAgata,"EAgata/D");
  RootOutput::getInstance()->GetTree()->Branch("ELab",&ELab);
  RootOutput::getInstance()->GetTree()->Branch("Ecm",&Ecm);
  RootOutput::getInstance()->GetTree()->Branch("RawEnergy",&RawEnergy);
  RootOutput::getInstance()->GetTree()->Branch("ELoss_Al",&ELoss_Al);
  RootOutput::getInstance()->GetTree()->Branch("ELoss_Target",&ELoss_Target);
  RootOutput::getInstance()->GetTree()->Branch("ELoss",&ELoss);
  RootOutput::getInstance()->GetTree()->Branch("ThetaLab",&ThetaLab);
  RootOutput::getInstance()->GetTree()->Branch("ThetaGamma",&ThetaGamma);
  RootOutput::getInstance()->GetTree()->Branch("Theta4",&Theta4);
  RootOutput::getInstance()->GetTree()->Branch("Phi4",&Phi4);
  RootOutput::getInstance()->GetTree()->Branch("Energy4",&Energy4);
  RootOutput::getInstance()->GetTree()->Branch("ThetaGR",&ThetaGR);
  RootOutput::getInstance()->GetTree()->Branch("PhiLab",&PhiLab);
  RootOutput::getInstance()->GetTree()->Branch("HeavyThetaLab",&HeavyThetaLab);
  RootOutput::getInstance()->GetTree()->Branch("HeavyPhiLab",&HeavyPhiLab);
  RootOutput::getInstance()->GetTree()->Branch("HeavyPosXAtWindow",&HeavyPosXAtWindow);
  RootOutput::getInstance()->GetTree()->Branch("HeavyPosYAtWindow",&HeavyPosYAtWindow);
  RootOutput::getInstance()->GetTree()->Branch("HeavyPosXAtMUST2",&HeavyPosXAtMUST2);
  RootOutput::getInstance()->GetTree()->Branch("HeavyPosYAtMUST2",&HeavyPosYAtMUST2);
  RootOutput::getInstance()->GetTree()->Branch("HeavyAngX",&HeavyAngX);
  RootOutput::getInstance()->GetTree()->Branch("HeavyAngY",&HeavyAngY);
  RootOutput::getInstance()->GetTree()->Branch("ThetaCM",&ThetaCM);
  RootOutput::getInstance()->GetTree()->Branch("Run",&Run,"Run/I");
  RootOutput::getInstance()->GetTree()->Branch("X",&X);
  RootOutput::getInstance()->GetTree()->Branch("Y",&Y);
  RootOutput::getInstance()->GetTree()->Branch("Z",&Z);
  RootOutput::getInstance()->GetTree()->Branch("dE",&dE,"dE/D");
  RootOutput::getInstance()->GetTree()->Branch("MGXStrip",&MGXStrip);
  RootOutput::getInstance()->GetTree()->Branch("MGYStrip",&MGYStrip);
  RootOutput::getInstance()->GetTree()->Branch("MGDet",&MGDet);
  RootOutput::getInstance()->GetTree()->Branch("MG_T",MG_T);
  RootOutput::getInstance()->GetTree()->Branch("MG_E",MG_E);
  RootOutput::getInstance()->GetTree()->Branch("MG_X",MG_X);
  RootOutput::getInstance()->GetTree()->Branch("MG_Y",MG_Y);
  RootOutput::getInstance()->GetTree()->Branch("MG_D",MG_D);

  // Vamos 
  RootOutput::getInstance()->GetTree()->Branch("LTS",&LTS,"LTS/l");
    RootOutput::getInstance()->GetTree()->Branch("T_FPMW_CATS1",&T_FPMW_CATS1,"T_FPMW_CATS1/s");
    RootOutput::getInstance()->GetTree()->Branch("T_FPMW_CATS2_C",&T_FPMW_CATS2_C,"T_FPMW_CATS2_C/F");
    RootOutput::getInstance()->GetTree()->Branch("T_FPMW_HF",&T_FPMW_HF,"T_FPMW_CATS1/s");
    RootOutput::getInstance()->GetTree()->Branch("T_FPMW_HF_C",&T_FPMW_HF_C,"T_FPMW_CATS1/F");
    RootOutput::getInstance()->GetTree()->Branch ("IC",IC, "IC[12]/F");
    RootOutput::getInstance()->GetTree()->Branch ("ICRawPUMult01",ICRawPUMult01, "ICRawPUMult01[7]/s");

    RootOutput::getInstance()->GetTree()->Branch("T_CATS2_HF",&T_CATS2_HF,"T_CATS2_HF/s");
    RootOutput::getInstance()->GetTree()->Branch("T_MUGAST_FPMW",&T_MUGAST_FPMW,"T_MUGAST_FPMW/s");
    RootOutput::getInstance()->GetTree()->Branch("T_FPMW_CATS2",&T_FPMW_CATS2,"T_FPMW_CATS2/s");
    RootOutput::getInstance()->GetTree()->Branch("T_FPMW_HF",&T_FPMW_HF,"T_FPMW_HF/s");

    // DC0
    RootOutput::getInstance()->GetTree()->Branch ("DC0_X", &DC0_X, "DC0_X/F");
    RootOutput::getInstance()->GetTree()->Branch ("DC0_Y", &DC0_Y, "DC0_Y/F");
    // DC1
    RootOutput::getInstance()->GetTree()->Branch ("DC1_X", &DC1_X, "DC1_X/F");
    RootOutput::getInstance()->GetTree()->Branch ("DC1_Y", &DC1_Y, "DC1_Y/F");
    // DC2
    RootOutput::getInstance()->GetTree()->Branch ("DC2_X", &DC2_X, "DC2_X/F");
    RootOutput::getInstance()->GetTree()->Branch ("DC2_Y", &DC2_Y, "DC2_Y/F");
    // DC3
    RootOutput::getInstance()->GetTree()->Branch ("DC3_X", &DC3_X, "DC3_X/F");
    RootOutput::getInstance()->GetTree()->Branch ("DC3_Y", &DC3_Y, "DC3_Y/F");

    RootOutput::getInstance()->GetTree()->Branch ("Xf", &Xf, "Xf/F");
    RootOutput::getInstance()->GetTree()->Branch ("Tf", &Tf, "Tf/F");

    RootOutput::getInstance()->GetTree()->Branch ("Brho", &Brho, "Brho/F");
    RootOutput::getInstance()->GetTree()->Branch ("Theta", &Theta, "Theta/F");
    RootOutput::getInstance()->GetTree()->Branch ("Phi", &Phi, "Phi/F");
    RootOutput::getInstance()->GetTree()->Branch ("Path", &Path, "Path/F");

    RootOutput::getInstance()->GetTree()->Branch ("EWIRE_1_1", &EWIRE_1_1, "EWIRE_1_1/s");
    RootOutput::getInstance()->GetTree()->Branch ("EWIRE_1_2", &EWIRE_1_2, "EWIRE_1_2/s");
    RootOutput::getInstance()->GetTree()->Branch ("EWIRE_2_1", &EWIRE_2_1, "EWIRE_2_1/s");
    RootOutput::getInstance()->GetTree()->Branch ("EWIRE_2_2", &EWIRE_2_2, "EWIRE_2_2/s");

  RootOutput::getInstance()->GetTree()->Branch ("MW_Nr", &MW_Nr, "MW_Nr/I");
  RootOutput::getInstance()->GetTree()->Branch ("MW_T", MW_T, "MW_T[MW_Nr]/F");
  RootOutput::getInstance()->GetTree()->Branch ("MW_N", MW_N, "MW_N[MW_Nr]/s");
  RootOutput::getInstance()->GetTree()->Branch ("MWT", MWT, "MWT[MW_Nr]/F");

  RootOutput::getInstance()->GetTree()->Branch ("AGAVA_VAMOSTS", &AGAVA_VAMOSTS, "AGAVA_VAMOSTS/l");
/*    RootOutput::getInstance()->GetTree()->Branch("mT",&mT,"mT/D");
    RootOutput::getInstance()->GetTree()->Branch("mV",&mV,"mV/D");
    RootOutput::getInstance()->GetTree()->Branch("mD",&mD,"mD/D");
    RootOutput::getInstance()->GetTree()->Branch("mBeta",&mBeta,"mBeta/D");
    RootOutput::getInstance()->GetTree()->Branch("mGamma",&mGamma,"mGamma/D");
    RootOutput::getInstance()->GetTree()->Branch("mM_Q",&mM_Q,"mM_Q/D");
    RootOutput::getInstance()->GetTree()->Branch("mM",&mM,"mM/D");
    RootOutput::getInstance()->GetTree()->Branch("mE",&mE,"mE/D");
    RootOutput::getInstance()->GetTree()->Branch("mdE",&mdE,"mdE/D");
*/

  // DC0->3_X/_Y Xf, Tf, Brho, Path, EWIRE_1_1 all, MW_Nr, MW_T, AGAVA_VAMOSTS, 
  // Agata
  // Time stamp of the agata trigger
  
  RootOutput::getInstance()->GetTree()->Branch("TStrack",&TStrack,"TStrack/l");

  // Array of reconstructed tracks
  RootOutput::getInstance()->GetTree()->Branch("nbTrack",&nbTrack,"nbTrack/I");
  RootOutput::getInstance()->GetTree()->Branch("trackE",trackE,"trackE[nbTrack]/F");
  RootOutput::getInstance()->GetTree()->Branch("trackX1",trackX1,"trackX1[nbTrack]/F");
  RootOutput::getInstance()->GetTree()->Branch("trackY1",trackY1,"trackY1[nbTrack]/F");
  RootOutput::getInstance()->GetTree()->Branch("trackZ1",trackZ1,"trackZ1[nbTrack]/F");
  RootOutput::getInstance()->GetTree()->Branch("trackT",trackT,"trackT[nbTrack]/F");
  RootOutput::getInstance()->GetTree()->Branch("trackCrystalID",trackCrystalID,"trackCrystalID[nbTrack]/I");
  
  //Agata Addback
  RootOutput::getInstance()->GetTree()->Branch("nbAdd",&nbAdd,"nbAdd/I");
  RootOutput::getInstance()->GetTree()->Branch("TSHit",&TSHit,"TSHit/l");
  RootOutput::getInstance()->GetTree()->Branch("AddE",AddE,"AddE[nbAdd]/F");
  RootOutput::getInstance()->GetTree()->Branch("AddX",AddX,"AddX[nbAdd]/F");
  RootOutput::getInstance()->GetTree()->Branch("AddY",AddY,"AddY[nbAdd]/F");
  RootOutput::getInstance()->GetTree()->Branch("AddZ",AddZ,"AddZ[nbAdd]/F");
  
  RootOutput::getInstance()->GetTree()->Branch("EventWeight",&EventWeight,"EventWeight/D");

  //AGATA hits for the tracking optimization
  RootOutput::getInstance()->GetTree()->Branch("nbHits",&nbHits,"nbHits/I");
  RootOutput::getInstance()->GetTree()->Branch("hitE",hitE,"hitE[nbHits]/F");
  RootOutput::getInstance()->GetTree()->Branch("hitX",hitX,"hitX[nbHits]/F");
  RootOutput::getInstance()->GetTree()->Branch("hitY",hitY,"hitY[nbHits]/F");
  RootOutput::getInstance()->GetTree()->Branch("hitZ",hitZ,"hitZ[nbHits]/F");
  RootOutput::getInstance()->GetTree()->Branch("hitId",hitId,"hitId[nbHits]/I");
  RootOutput::getInstance()->GetTree()->Branch("hitSg",hitSg,"hitSg[nbHits]/I");
//  mEDC_PhiAgata = new TH2I("mEDC_PhiAgata",
//			   "Optimization DC for Phi AGATA",
//			   360,-0.5,359.5,
//			   3000,0,3000);
//  RootOutput::getInstance()->GetList()->Add(mEDC_PhiAgata);
//  mEDC_OffsetAgata = new TH2I("mEDC_OffsetAgata",
//			      "Optimization DC for Offset AGATA",
//			      21,-10.5,10.5,
//			      3000,0,3000);
//  RootOutput::getInstance()->GetList()->Add(mEDC_OffsetAgata);
//  backingThick = new TH2I("backingThick",
//			  "Optimization Backing Thickness",
//			  30,10.0,13.0,
//			  3000,0,3000);
//  RootOutput::getInstance()->GetList()->Add(backingThick);
//  kineShift = new TH3I("kineShift","Kinematic for Annular Shift",
//		       11,-0.5,10.5,
//		       1200,0,12,
//		       360,90,180);
//  RootOutput::getInstance()->GetList()->Add(kineShift);
//  for(int i=0; i<7; i++){
//    beamPos[i] = new TH3I(Form("beamPos_%i",i),
//			  "Position of the beam",
//			  21,-10.5,10.5,
//			  21,-10.5,10.5,
//			  280,-1,6);
//    RootOutput::getInstance()->GetList()->Add(beamPos[i]);
//  }

  if(isSim && !isPhaseSpace){
    RootOutput::getInstance()->GetTree()->Branch("OriginalELab",
	      &OriginalELab,"OriginalELab/D");
    RootOutput::getInstance()->GetTree()->Branch("OriginalThetaLab",
	      &OriginalThetaLab,"OriginalThetaLab/D");
    RootOutput::getInstance()->GetTree()->Branch("BeamEnergy",
	      &BeamEnergy,"BeamEnergy/D");
  }
cout << "end_InitOutput" << endl;
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::InitInputBranch(){
  SetBranchStatus();
cout << "start_InitInputBranch (after set status)" << endl;
  // RootInput:: getInstance()->GetChain()->SetBranchAddress("GATCONF",&vGATCONF);
  //
  // Vamos
  RootInput::getInstance()->GetChain()->SetBranchAddress("LTS",&LTS);
  RootInput::getInstance()->GetChain()->SetBranchAddress("VTS",&VTS);
  RootInput::getInstance()->GetChain()->SetBranchAddress("T_FPMW_CATS1",&T_FPMW_CATS1);
  RootInput::getInstance()->GetChain()->SetBranchAddress("T_FPMW_CATS2_C",&T_FPMW_CATS2_C);
  RootInput::getInstance()->GetChain()->SetBranchAddress("T_FPMW_HF_C",&T_FPMW_HF_C);
  RootInput::getInstance()->GetChain()->SetBranchAddress("IC",IC);

  RootInput::getInstance()->GetChain()->SetBranchAddress("T_CATS2_HF",&T_CATS2_HF);
  RootInput::getInstance()->GetChain()->SetBranchAddress("T_MUGAST_FPMW",&T_MUGAST_FPMW);
  RootInput::getInstance()->GetChain()->SetBranchAddress("T_FPMW_CATS2",&T_FPMW_CATS2);
  RootInput::getInstance()->GetChain()->SetBranchAddress("T_FPMW_HF",&T_FPMW_HF);

  // DC0
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC0_X", &DC0_X);
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC0_Y", &DC0_Y);
  // DC1
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC1_X", &DC1_X);
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC1_Y", &DC1_Y);
  // DC2
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC2_X", &DC2_X);
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC2_Y", &DC2_Y);
  // DC3
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC3_X", &DC3_X);
  RootInput::getInstance()->GetChain()->SetBranchAddress("DC3_Y", &DC3_Y);

  RootInput::getInstance()->GetChain()->SetBranchAddress("Xf", &Xf);
  RootInput::getInstance()->GetChain()->SetBranchAddress("Tf", &Tf);

  RootInput::getInstance()->GetChain()->SetBranchAddress("Brho", &Brho);
  RootInput::getInstance()->GetChain()->SetBranchAddress("Theta", &Theta);
  RootInput::getInstance()->GetChain()->SetBranchAddress("Phi", &Phi);
  RootInput::getInstance()->GetChain()->SetBranchAddress("Path", &Path);

  RootInput::getInstance()->GetChain()->SetBranchAddress("EWIRE_1_1", &EWIRE_1_1);
  RootInput::getInstance()->GetChain()->SetBranchAddress("EWIRE_1_2", &EWIRE_1_2);
  RootInput::getInstance()->GetChain()->SetBranchAddress("EWIRE_2_1", &EWIRE_2_1);
  RootInput::getInstance()->GetChain()->SetBranchAddress("EWIRE_2_2", &EWIRE_2_2);

  RootInput::getInstance()->GetChain()->SetBranchAddress("MWTVM", &MW_Nr);
  RootInput::getInstance()->GetChain()->SetBranchAddress("MWTV", MW_T);
  RootInput::getInstance()->GetChain()->SetBranchAddress("MWTVN", MW_N);
  RootInput::getInstance()->GetChain()->SetBranchAddress("AGAVA_VAMOSTS", &AGAVA_VAMOSTS);

  // Agata
     
  RootInput::getInstance()->GetChain()->SetBranchAddress("TStrack",&TStrack);
  RootInput::getInstance()->GetChain()->SetBranchAddress("nbTrack",&nbTrack);
  RootInput::getInstance()->GetChain()->SetBranchAddress("trackE",trackE);
  RootInput::getInstance()->GetChain()->SetBranchAddress("trackX1",trackX1);
  RootInput::getInstance()->GetChain()->SetBranchAddress("trackY1",trackY1);
  RootInput::getInstance()->GetChain()->SetBranchAddress("trackZ1",trackZ1);
  RootInput::getInstance()->GetChain()->SetBranchAddress("trackT",trackT);
  RootInput::getInstance()->GetChain()->SetBranchAddress("trackCrystalID",trackCrystalID);
  
  // We need to add the hits for the tracking optimization
  RootInput::getInstance()->GetChain()->SetBranchAddress("nbHits",&nbHits);
  RootInput::getInstance()->GetChain()->SetBranchAddress("hitE",hitE);
  RootInput::getInstance()->GetChain()->SetBranchAddress("hitX",hitX);
  RootInput::getInstance()->GetChain()->SetBranchAddress("hitY",hitY);
  RootInput::getInstance()->GetChain()->SetBranchAddress("hitZ",hitZ);
  RootInput::getInstance()->GetChain()->SetBranchAddress("hitId",hitId);
  RootInput::getInstance()->GetChain()->SetBranchAddress("hitSg",hitSg);

  RootInput::getInstance()->GetChain()->SetBranchAddress("TSHit",&TSHit);
  RootInput::getInstance()->GetChain()->SetBranchAddress("nbAdd",&nbAdd);
  RootInput::getInstance()->GetChain()->SetBranchAddress("AddE",AddE);
  RootInput::getInstance()->GetChain()->SetBranchAddress("AddX",AddX);
  RootInput::getInstance()->GetChain()->SetBranchAddress("AddY",AddY);
  RootInput::getInstance()->GetChain()->SetBranchAddress("AddZ",AddZ);

  if(isPhaseSpace){
    RootInput:: getInstance()->GetChain()->SetBranchAddress("EventWeight",&EventWeight);
  }

  if(isSim && !isPhaseSpace){
    //RootInput:: getInstance()->GetChain()->SetBranchStatus("InitialConditions",true );
    //RootInput:: getInstance()->GetChain()->SetBranchStatus("fIC_*",true );
    RootInput::getInstance()->GetChain()->SetBranchAddress("InitialConditions",
		    &Initial);
    RootInput::getInstance()->GetChain()->SetBranchAddress("InteractionCoordinates",
		    &Interaction);
    //RootInput:: getInstance()->GetChain()->SetBranchStatus("ReactionConditions",true );
    //RootInput:: getInstance()->GetChain()->SetBranchStatus("fRC_*",true );
    RootInput::getInstance()->GetChain()->SetBranchAddress("ReactionConditions",
		    &ReactionConditions);
  }
cout << "end_InitInputBranch (after set status)" << endl;
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::SetBranchStatus(){
cout << "start_SetBranchStatus" << endl;
  // Set Branch status 
  RootInput::getInstance()->GetChain()->SetBranchStatus("LTS",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("VTS",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("T_FPMW_CATS1",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("T_FPMW_CATS2_C",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("T_FPMW_HF",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("T_FPMW_HF_C",true);
  
  RootInput::getInstance()->GetChain()->SetBranchStatus("T_CATS2_HF",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("T_MUGAST_FPMW",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("T_FPMW_CATS2",true );

  // DC0
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC0_X",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC0_Y",true);
  // DC1
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC1_X",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC1_Y",true);
  // DC2
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC2_X",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC2_Y",true);
  // DC3
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC3_X",true);
  RootInput::getInstance()->GetChain()->SetBranchStatus("DC3_Y",true);

  RootInput::getInstance()->GetChain()->SetBranchStatus("Xf",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("Tf",true );

  RootInput::getInstance()->GetChain()->SetBranchStatus("Brho",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("Path",true );

  RootInput::getInstance()->GetChain()->SetBranchStatus("EWIRE_1_1",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("EWIRE_1_2",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("EWIRE_2_1",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("EWIRE_2_2",true );

  RootInput::getInstance()->GetChain()->SetBranchStatus("MW_Nr",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("MW_T",true  );

  RootInput::getInstance()->GetChain()->SetBranchStatus("MWTV",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("MWTVN",true  );
  RootInput::getInstance()->GetChain()->SetBranchStatus("MWTVM",true  );

  RootInput::getInstance()->GetChain()->SetBranchStatus("AGAVA_VAMOSTS",true  );
  // Agata
    
  RootInput::getInstance()->GetChain()->SetBranchStatus("TStrack",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("nbTrack",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("trackE",true  );
  RootInput::getInstance()->GetChain()->SetBranchStatus("trackX1",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("trackY1",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("trackZ1",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("trackT",true  );
  RootInput::getInstance()->GetChain()->SetBranchStatus("trackCrystalID",true);
  
  RootInput::getInstance()->GetChain()->SetBranchStatus("hitE",1);
  RootInput::getInstance()->GetChain()->SetBranchStatus("hitX",1);
  RootInput::getInstance()->GetChain()->SetBranchStatus("hitY",1);
  RootInput::getInstance()->GetChain()->SetBranchStatus("hitZ",1);
  RootInput::getInstance()->GetChain()->SetBranchStatus("hitId",1);
  RootInput::getInstance()->GetChain()->SetBranchStatus("hitSg",1);
  RootInput::getInstance()->GetChain()->SetBranchStatus("nbHits",1);

  RootInput::getInstance()->GetChain()->SetBranchStatus("TSHit",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("nbAdd",true );
  RootInput::getInstance()->GetChain()->SetBranchStatus("AddE",true  );
  RootInput::getInstance()->GetChain()->SetBranchStatus("AddX",true  );
  RootInput::getInstance()->GetChain()->SetBranchStatus("AddY",true  );
  RootInput::getInstance()->GetChain()->SetBranchStatus("AddZ",true  );

  if(isPhaseSpace){
    RootInput:: getInstance()->GetChain()->SetBranchStatus("EventWeight",true );
  }

  if(isSim && !isPhaseSpace){
    RootInput:: getInstance()->GetChain()->SetBranchStatus("InitialConditions",true );
    RootInput:: getInstance()->GetChain()->SetBranchStatus("fIC_*",true );
    RootInput:: getInstance()->GetChain()->SetBranchStatus("InteractionCoordinates",true );
    RootInput:: getInstance()->GetChain()->SetBranchStatus("ReactionConditions",true );
    RootInput:: getInstance()->GetChain()->SetBranchStatus("fRC_*",true );
  }
cout << "end_SetBranchStatus" << endl;
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::ReInitValue(){
//cout << "start_ReInit" << endl;
  hasAuTarget.clear();

  Ex.clear();
  ExCalib.clear();
  Ecm.clear();
  RawGammaEnergy.clear();
  BetaUsedByAddBackEDC.clear();
  AddBack_EDC.clear();
  AddBack_EDC_Event1.clear();
  AddBack_EDC_Event2.clear();
  AGATA_Theta.clear();
  AGATA_Phi.clear();
  AGATA_GammaPx.clear();
  AGATA_GammaPy.clear();
  AGATA_GammaPz.clear();
  AGATA_GammaE.clear();
  AGATA_OrigBetaX.clear();
  AGATA_OrigBetaY.clear();
  AGATA_OrigBetaZ.clear();
  AGATA_OrigBetaMag.clear();
  EAgata = -1000;
  ELab.clear();
  RawEnergy.clear(); 
  ELoss_Al.clear();
  ELoss_Target.clear();
  ELoss.clear();
  ThetaLab.clear();
  PhiLab.clear();
  ThetaCM.clear();
  HeavyThetaLab.clear();
  HeavyPhiLab.clear();
  HeavyPosXAtWindow.clear();
  HeavyPosYAtWindow.clear();
  HeavyPosXAtMUST2.clear();
  HeavyPosYAtMUST2.clear();
  HeavyAngX.clear();
  HeavyAngY.clear();
  //relative_angle = -1000;
  //sum_angle = -1000;
  X.clear();
  Y.clear();
  Z.clear();
  dE= -1000;
  ParticleMult = 0;
  GammaMult = 0;
  DetectorNumber = 0;
  ThetaNormalTarget = 0;
  ThetaM2Surface = 0;
  ThetaMGSurface = 0;
  Si_E_M2 = 0;
  CsI_E_M2 = 0;
  Energy = 0;
  ThetaGDSurface = 0;
  BeamDirection = TVector3(0,0,1);
  elab_tmp = 0;
  thetalab_tmp = 0;
  philab_tmp = 0;

  filledCline=false;

  MG_T=-1000;
  MG_E=-1000;
  MG_X=-1000;
  MG_Y=-1000;
  MG_D=-1000;

  //EDC= -1000;
  EDC.clear();

  /* Just straight copied from IZanons */
  Ex.clear();
  ExCalib.clear();
  //AddBack_EDC=-1000;
  EAgata = -1000;
  ELab.clear();
  MGDet.clear();
  MGXStrip.clear();
  MGYStrip.clear();
    
  ThetaG.clear();
  radius = 0;
  PhiG.clear();
  ThetaLab.clear();
  ThetaGamma.clear();
  Theta4 = 0;
  Phi4 = 0;
  Theta3.clear();
  Phi3.clear();
  ThetaGR.clear();
  PhiLab.clear();
  ThetaCM.clear();
  gammaEDC.clear();
  trackEDC.clear();
  vamosEDC.clear();
	goldEDC.clear();
  //relative_angle = -1000;
  //sum_angle = -1000;
  X.clear();
  Y.clear();
  Z.clear();
  dE= -1000;
  ParticleMult = 0;
  GammaMult = 0;
  DetectorNumber = 0;
  ThetaNormalTarget = 0;
  ThetaM2Surface = 0;
  ThetaMGSurface = 0;
  Si_E_M2 = 0;
  CsI_E_M2 = 0;
  Energy = 0;
  ThetaGDSurface = 0;
  BeamDirection = TVector3(0,0,1);
  elab_tmp = 0;
  thetalab_tmp = 0;
  philab_tmp = 0;

  EDC_VAMOS.clear();
  EDC_MUGAST.clear();
  beta_VAMOS.clear();
  beta_MUGAST.clear();

//cout << "end_ReInit" << endl;
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::CalculateVamos(){
//cout << "start_CalculateVamos" << endl;
  mT1 = -1500.;
  mT = -1500.;
  mV = -1500.;
  mD2 = -1500;
  mBeta = -1500.;
  mGamma = -1500.;
  mM_Q = -1500.;
  mM = -1500.;
  mQ = -1500.;
  mMrec = -1500.;
  mdE = -1500;
  mE = -1500;

  mT1 = T_FPMW_HF_C;
  mT  = 596.0 - mT1;
  
  Float_t ICTh = 1;
  if(MW_Nr == 1){
    switch (MW_N[0]) {
    case 0:
      mD2 = 826.465 ;
      break;
    case 1:
      mD2 = 825.677 ;
      break;
    case 2:
      mT += -2.5;
      mD2 = 830.012 ;
      break;
    case 3:
      mT += -2.0;
      mD2 = 825.939 ;
      break;
    case 4:
      mD2 = 825.521 ;
      break;
    case 5:
      mT += 1;
      mD2 = 827.253 ;
      break;
    case 6:
      mT += 2;
      mD2 = 826.334 ;
      break;
    case 7:
      mT += 2;
      mD2 = 827.253 ;
      break;
    case 8:
      mT += 1.5;
      mD2 = 833.559 ;
      break;
    case 9:
      mT += 0;
      mD2 = 825.043 ;
      break;
    case 10:
      mT += 0.9;
      mD2 = 826.728 ;
      break;
    case 11:
      mT += 2.2;
      mD2 = 825.414 ;
      break;
    case 12:
      mT += 5;
      mD2 = 817.138 ;
      break;
    case 13:
      mT += 5;
      mD2 = 823.049 ;
      break;
    case 14:
      mT += 4;
      mD2 = 827.779 ;
      break;
    case 15:
      mT += 3 ;
      mD2 = 832.508 ;
      break;
    case 16:
      mT += 2;
      mD2 = 841.966 ;
      break;
    case 17:
      mD2 = 848.403 ;
      break;
    case 18:
      mD2 = 831.194 ;
      break;
    case 19:
      mD2 = 828.698 ;
      break;
    case 20:
      mD2 = 824.232 ;
      break;
    default:
      mT = -1500.;
      mD2 = -1500 ;
      break;
    }
  }
  else{
    mT = -1500.;
    mD2 = -1500 ;
  }

  mV = mD2/mT;
  mBeta = mV/29.9792;
  mGamma = 1./sqrt(1.0-mBeta*mBeta);
  mM_Q = Brho/3.105/mBeta/mGamma;
    
  mE = 0.86*(2.8*IC[0]+IC[1]*(IC[0]>0.1)+IC[2]*(IC[0]>0.1)*(IC[1]>0.1)+IC[3]*(IC[0]>0.1)*(IC[1]>0.1)*(IC[2]>0.1)+IC[4]*(IC[0]>0.1)*(IC[1]>0.1)*(IC[2]>0.1)*(IC[3]>0.1)+IC[5]*(IC[0]>0.1)*(IC[1]>0.1)*(IC[2]>0.1)*(IC[3]>0.1)*(IC[4]>0.1)+IC[6]*(IC[0]>0.1)*(IC[1]>0.1)*(IC[2]>0.1)*(IC[3]>0.1)*(IC[4]>0.1)*(IC[5]>0.1));
   
  mdE = IC[0]+IC[1]*(IC[0]>0.1);

  mM = mE/931.494/(mGamma-1.);
  mQ = mM/mM_Q;
  mMrec = int(mQ+0.5)*mM_Q;

//cout << "end_CalculateVamos" << endl;
}

////////////////////////////////////////////////////////////////////////////////
//            Construct Method to be pass to the AnalysisFactory              //
////////////////////////////////////////////////////////////////////////////////
NPL::VAnalysis* Analysis::Construct(){
  return (NPL::VAnalysis*) new Analysis();
}

////////////////////////////////////////////////////////////////////////////////
//            Registering the construct method to the factory                 //
////////////////////////////////////////////////////////////////////////////////
extern "C"{
  class proxy_analysis{
    public:
      proxy_analysis(){
        NPL::AnalysisFactory::getInstance()->SetConstructor(Analysis::Construct);
      }
  };

  proxy_analysis p_analysis;
}


