#include <math.h>
#include <string>
#include <sstream>
#include "Plots_TWOFNR.h"
#include "Plots_KnownPeakFitter.h"
#include "DefineColours.h"
using namespace std;

//=====================================================================//
//=====================================================================//

vector<string> files;
TChain* tree_cd=NULL;
TChain* tree_au=NULL;
TChain* tree_both=NULL;
TChain* tree_raw=NULL;
//TChain* tree=NULL;
TChain* tree_PSp00=NULL;
TChain* tree_PSp96=NULL;

NPL::Reaction O19dp("19O(d,p)20O@151"); //TEMPORARY BEAM ENERGY!

auto colourR = new TColor((Int_t)TColor::GetFreeColorIndex(), 220./256.,   5./256.,  12./256.);
auto colourB = new TColor((Int_t)TColor::GetFreeColorIndex(),  25./256., 101./256., 176./256.);
auto colourG = new TColor((Int_t)TColor::GetFreeColorIndex(),  78./256., 178./256., 101./256.);
auto colourP = new TColor((Int_t)TColor::GetFreeColorIndex(), 136./256.,  46./256., 114./256.);

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void PlotKinematic(NPL::Reaction r, double Ex, Color_t c, int w, int s)
{
  r.SetExcitation4(Ex);
  TGraph* g = r.GetKinematicLine3();
  g->SetLineColor(c);
  g->SetLineWidth(w);
  g->SetLineStyle(s);
  g->Draw("c same");
}

//////////////////////////////////////////////////////////////////////////////////////
TChain* Chain(string TreeName, vector<string>& file, bool EventList)
{
  TChain* chain = new TChain(TreeName.c_str());
  unsigned int size = file.size();
  for(unsigned int i=0; i<size; i++){
    cout << "Adding to chain: " << file[i] << endl;
    chain->Add(file[i].c_str());
  }
  return chain;
}

//////////////////////////////////////////////////////////////////////////////////////
void LoadChain_Both_PaxmanAnalysis()
{
  files.clear();
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_XYZT_0003_CD.root");  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_XYZT_0003_Au.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-08-28_CD.root"); files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-08-28_Au.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-05_CD.root"); files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-05_Au.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-06_Au.root"); files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-06_CD.root");
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-10_Au.root"); files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-10_CD.root");
  tree_both = Chain("PhysicsTree",files,true);

  files.clear();
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_XYZT_0003_CD.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-08-28_CD.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-05_CD.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-06_CD.root");
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-10_CD.root");
  tree_cd = Chain("PhysicsTree",files,true);

  files.clear();
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_XYZT_0003_Au.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-08-28_Au.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-05_Au.root");
//files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-06_Au.root");
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-09-10_Au.root");
  tree_au = Chain("PhysicsTree",files,true);
}

//////////////////////////////////////////////////////////////////////////////////////
void LoadChain_Raw()
{
  files.clear();
  files.push_back("~/Experiment/e775s/RawData/run_0115/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0116/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0117/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0123/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0124/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0125/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0128/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0129/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0130/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0131/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0132/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0133/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0134/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0135/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0137/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0138/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0139/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0140/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0142/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0150/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0151/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0152/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0153/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0163/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0164/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0165/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0166/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0167/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0168/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0169/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0171/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0172/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0173/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0174/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0178/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0179/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0180/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0181/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0184/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0185/Out/Analysis/Tree_0000.root");
  tree_raw = Chain("TreeMaster",files,true);
}

//////////////////////////////////////////////////////////////////////////////////////
void LoadChain_PhaseSpace()
{
  files.clear();
  files.push_back("../../../Outputs/Analysis/PSp_PhaseSpace_InitialTest_0000.root");
  tree_PSp00 = Chain("PhysicsTree",files,true);
  
  files.clear();
  files.push_back("../../../Outputs/Analysis/PSp_PhaseSpace_InitialTest_0096.root");
  tree_PSp96 = Chain("PhysicsTree",files,true);
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_AddBack(TChain* tree)
{
  TCanvas *c = new TCanvas("cExEg_AddBack","cExEg_AddBack",500,500);
  tree->Draw("Ex:AddBack_EDC>>hExEg(8000,0,8,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && ThetaLab<158","colz");
  TH2F* hExEg = (TH2F*) gDirectory->Get("hExEg");
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExEg->SetName("hExEg_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExEg->SetName("hExEg_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExEg->SetName("hExEg_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_Tracked(TChain* tree)
{
  TCanvas *c = new TCanvas("cExEg_Tracked","cExEg_Tracked",500,500);
  tree->Draw("Ex:trackEDC>>hExEt(8000,0,8,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && ThetaLab<158","colz");
  TH2F* hExEt= (TH2F*) gDirectory->Get("hExEt");
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExEt->SetName("hExEt_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExEt->SetName("hExEt_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExEt->SetName("hExEt_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_Ex(TChain* tree)
{
  TCanvas *c = new TCanvas("cEx","cEx",500,500);
  tree->Draw("Ex>>hExGood(2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && ThetaLab<158","colz");
  TH1F* hExGood = (TH1F*) gDirectory->Get("hExGood");
  tree->Draw("Ex>>hExBad(2000,-5,15)","abs(T_FPMW_HF-10500)<1500 && ThetaLab<158","colz");
  TH1F* hExBad = (TH1F*) gDirectory->Get("hExBad");
  TH1F* hExClean = (TH1F*) hExGood->Clone();
  hExClean->SetTitle("hExClean");
  hExClean->SetName("hExClean");
  hExClean->Add(hExBad,-1);
  hExClean->Draw();
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExClean->SetName("hExClean_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExClean->SetName("hExClean_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExClean->SetName("hExClean_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExThetaLab(TChain* tree)
{ 
  TCanvas *c = new TCanvas("cExThetaLab","cExThetaLab",500,500);
  tree->Draw("Ex:ThetaLab>>hExTLGood(80,100,180,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && ThetaLab<158","colz");
  TH2F* hExTLGood = (TH2F*) gDirectory->Get("hExTLGood");
  tree->Draw("Ex:ThetaLab>>hExTLBad(80,100,180,2000,-5,15)","abs(T_FPMW_HF-10500)<1500 && ThetaLab<158","colz");
  TH2F* hExTLBad = (TH2F*) gDirectory->Get("hExTLBad");
  TH2F* hExTLClean = (TH2F*) hExTLGood->Clone();
  hExTLClean->SetTitle("hExTLClean");
  hExTLClean->SetName("hExTLClean");
  hExTLClean->Add(hExTLBad,-1);
  hExTLClean->Draw();
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExTLClean->SetName("hExTLClean_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExTLClean->SetName("hExTLClean_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExTLClean->SetName("hExTLClean_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_All(TChain* tree)
{
  DrawPlotsToLoad_ExEg_AddBack(tree);
  DrawPlotsToLoad_ExEg_Tracked(tree);
  DrawPlotsToLoad_Ex(tree);
  DrawPlotsToLoad_ExThetaLab(tree);
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_AddBack(){ DrawPlotsToLoad_ExEg_AddBack(tree_both); }
void DrawPlotsToLoad_ExEg_Tracked(){ DrawPlotsToLoad_ExEg_Tracked(tree_both); }
void DrawPlotsToLoad_Ex(){ DrawPlotsToLoad_Ex(tree_both); }
void DrawPlotsToLoad_ExThetaLab(){ DrawPlotsToLoad_ExThetaLab(tree_both); }
void DrawPlotsToLoad_All(){ DrawPlotsToLoad_All(tree_both); }

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
TH1F* Project(TH2F* h2D, const char * XY, Color_t col)
{
  TH1F* h;
  if(string(XY) == "X"){
    string name = "projX_" + to_string(col);
    h2D->ProjectionX(name.c_str(),0,-1,"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else if (string(XY) == "Y"){
    string name = "projY_" + to_string(col);
    h2D->ProjectionY(name.c_str(),0,-1,"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else {
    cout << "!!!! ERROR !!!! XY projection selector is invalid" << endl;
    return 0;
  }
  return (TH1F*) h;
}

//////////////////////////////////////////////////////////////////////////////////////
TH1F* Project(TH2F* h2D, const char * XY, double mean, double width, Color_t col)
{
  TH1F* h;
  if(string(XY) == "X"){
    //X projection is GPSG
    string name = "projX_" + to_string(col);
    h2D->ProjectionX(name.c_str(),h2D->GetYaxis()->FindBin(mean-(0.5*width)),
                                  h2D->GetYaxis()->FindBin(mean+(0.5*width)),"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else if (string(XY) == "Y"){
    //Y projection is GGSP
    string name = "projY_" + to_string(col);
    h2D->ProjectionY(name.c_str(),h2D->GetXaxis()->FindBin(mean-(0.5*width)),
                                  h2D->GetXaxis()->FindBin(mean+(0.5*width)),"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else {
    cout << "!!!! ERROR !!!! XY projection selector is invalid" << endl;
    return 0;
  }
  return (TH1F*) h;
}

//////////////////////////////////////////////////////////////////////////////////////
void Prj_DrawLines(double mean, double width, Color_t col)
{
  TLine* line = new TLine();
  line->SetLineColor(col);
  line->SetLineStyle(kDashed);
  line->SetLineWidth(2);
  line->DrawLine(mean-(0.5*width), 0, mean-(0.5*width), 2000);
  line->DrawLine(mean+(0.5*width), 0, mean+(0.5*width), 2000);
}

//////////////////////////////////////////////////////////////////////////////////////
string AddOrTrack(string input)
{
       if(input == "addback"){ return "hExEg"; }
  else if(input == "track"){   return "hExEt"; }
  else { cout << "ERROR! not 'addback' or 'track' !!!" << endl; return "hExEg";}
}

//////////////////////////////////////////////////////////////////////////////////////
void CheckExEgExists(string histname, string tree, string addbackOrTrack)
{
  if(gDirectory->Get(histname.c_str()) == 0){
    if(addbackOrTrack == "track") { 
      if(tree == "cd")   { DrawPlotsToLoad_ExEg_Tracked(tree_cd);   }
      if(tree == "au")   { DrawPlotsToLoad_ExEg_Tracked(tree_au);   }
      if(tree == "both") { DrawPlotsToLoad_ExEg_Tracked(tree_both); }
    } else { 
      if(tree == "cd")   { DrawPlotsToLoad_ExEg_AddBack(tree_cd);   }
      if(tree == "au")   { DrawPlotsToLoad_ExEg_AddBack(tree_au);   }
      if(tree == "both") { DrawPlotsToLoad_ExEg_AddBack(tree_both); }
    }
  }
}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Eg1, double width1)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  projP1->Draw();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  projP1->Draw();
  projP2->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2,
                                                     double Eg3, double width3)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);
  Prj_DrawLines(Eg3, width3, kGreen);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  TH1F* projP3 = Project(hExEg, "Y", Eg3, width3, kGreen);
  projP1->Draw();
  projP2->Draw("same");
  projP3->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2,
                                                     double Eg3, double width3,
                                                     double Eg4, double width4)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);
  Prj_DrawLines(Eg3, width3, kGreen);
  Prj_DrawLines(Eg4, width4, kViolet);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  TH1F* projP3 = Project(hExEg, "Y", Eg3, width3, kGreen);
  TH1F* projP4 = Project(hExEg, "Y", Eg4, width4, kViolet);
  projP1->Draw();
  projP2->Draw("same");
  projP3->Draw("same");
  projP4->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2,
                                                     double Eg3, double width3,
                                                     double Eg4, double width4,
                                                     double Eg5, double width5)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);
  Prj_DrawLines(Eg3, width3, kGreen);
  Prj_DrawLines(Eg4, width4, kViolet);
  Prj_DrawLines(Eg5, width5, kOrange);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  TH1F* projP3 = Project(hExEg, "Y", Eg3, width3, kGreen);
  TH1F* projP4 = Project(hExEg, "Y", Eg4, width4, kViolet);
  TH1F* projP5 = Project(hExEg, "Y", Eg5, width5, kOrange);
  projP1->Draw();
  projP2->Draw("same");
  projP3->Draw("same");
  projP4->Draw("same");
  projP5->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Ex1, double width1)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  projG1->Draw();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  projG1->Draw();
  projG2->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2,
					             double Ex3, double width3)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);
  Prj_DrawLines(Ex3, width3, kGreen);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  TH1F* projG3 = Project(hExEg, "X", Ex3, width3, kGreen);
  projG1->Draw();
  projG2->Draw("same");
  projG3->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2,
					             double Ex3, double width3,
					             double Ex4, double width4)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);
  Prj_DrawLines(Ex3, width3, kGreen);
  Prj_DrawLines(Ex4, width4, kViolet);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  TH1F* projG3 = Project(hExEg, "X", Ex3, width3, kGreen);
  TH1F* projG4 = Project(hExEg, "X", Ex4, width4, kViolet);
  projG1->Draw();
  projG2->Draw("same");
  projG3->Draw("same");
  projG4->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string addbackOrTrack, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2,
					             double Ex3, double width3,
					             double Ex4, double width4,
					             double Ex5, double width5)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);
  Prj_DrawLines(Ex3, width3, kGreen);
  Prj_DrawLines(Ex4, width4, kViolet);
  Prj_DrawLines(Ex5, width5, kOrange);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  TH1F* projG3 = Project(hExEg, "X", Ex3, width3, kGreen);
  TH1F* projG4 = Project(hExEg, "X", Ex4, width4, kViolet);
  TH1F* projG5 = Project(hExEg, "X", Ex5, width5, kOrange);
  projG1->Draw();
  projG2->Draw("same");
  projG3->Draw("same");
  projG4->Draw("same");
  projG5->Draw("same");
}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_CompareTargets(string addbackOrTrack, double bing, double binp, 
                                                        double Ex1, double width1)
{
  string histname_cd = AddOrTrack(addbackOrTrack);  histname_cd += "_cd";
  CheckExEgExists(histname_cd, "cd", addbackOrTrack);
  
  string histname_au = AddOrTrack(addbackOrTrack);  histname_au += "_au";
  CheckExEgExists(histname_au, "au", addbackOrTrack);

  //Load in & rebin CD
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname_cd.c_str());
  TH2F* hExEg_cd = (TH2F*) hCopyMe->Clone();
  hExEg_cd->Rebin2D(bing/hExEg_cd->GetXaxis()->GetBinWidth(3),
                    binp/hExEg_cd->GetYaxis()->GetBinWidth(3));

  //Load in & rebin Au
        hCopyMe = (TH2F*) gDirectory->Get(histname_au.c_str());
  TH2F* hExEg_au = (TH2F*) hCopyMe->Clone();
  hExEg_au->Rebin2D(bing/hExEg_au->GetXaxis()->GetBinWidth(3),
                    binp/hExEg_au->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPCT = new TCanvas("cGPCT","cGPCT",1000,750);
  cGPCT->Divide(1,2);
  cGPCT->cd(1);

  //Axis being projected
  TH1F* projP_au = Project(hExEg_au, "Y", kOrange+2);
  projP_au->Draw();
  TH1F* projP_cd = Project(hExEg_cd, "Y", kBlack);
  projP_cd->SetFillColor(kBlack); projP_cd->SetFillStyle(3001); 
  projP_cd->Draw("same");

  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);

  //Projected axis
  cGPCT->cd(2);
  TH1F* projG1_au = Project(hExEg_au, "X", Ex1, width1, kRed);
  TH1F* projG1_cd = Project(hExEg_cd, "X", Ex1, width1, kRed+2);
    projG1_cd->SetFillColor(kRed+2); projG1_cd->SetFillStyle(3001);

  projG1_au->Draw();
  projG1_cd->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_CompareTargets(string addbackOrTrack, double bing, double binp, 
                                                        double Ex1, double width1,
                                                        double Ex2, double width2)
{
  string histname_cd = AddOrTrack(addbackOrTrack);  histname_cd += "_cd";
  CheckExEgExists(histname_cd, "cd", addbackOrTrack);
  
  string histname_au = AddOrTrack(addbackOrTrack);  histname_au += "_au";
  CheckExEgExists(histname_au, "au", addbackOrTrack);

  //Load in & rebin CD
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname_cd.c_str());
  TH2F* hExEg_cd = (TH2F*) hCopyMe->Clone();
  hExEg_cd->Rebin2D(bing/hExEg_cd->GetXaxis()->GetBinWidth(3),
                    binp/hExEg_cd->GetYaxis()->GetBinWidth(3));

  //Load in & rebin Au
        hCopyMe = (TH2F*) gDirectory->Get(histname_au.c_str());
  TH2F* hExEg_au = (TH2F*) hCopyMe->Clone();
  hExEg_au->Rebin2D(bing/hExEg_au->GetXaxis()->GetBinWidth(3),
                    binp/hExEg_au->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPCT = new TCanvas("cGPCT","cGPCT",1000,750);
  cGPCT->Divide(1,2);
  cGPCT->cd(1);

  //Axis being projected
  TH1F* projP_au = Project(hExEg_au, "Y", kOrange+2);
  projP_au->Draw();
  TH1F* projP_cd = Project(hExEg_cd, "Y", kBlack);
  projP_cd->SetFillColor(kBlack); projP_cd->SetFillStyle(3001); 
  projP_cd->Draw("same");

  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);

  //Projected axis
  cGPCT->cd(2);
  TH1F* projG1_au = Project(hExEg_au, "X", Ex1, width1, kRed);
  TH1F* projG1_cd = Project(hExEg_cd, "X", Ex1, width1, kRed+2);  
    projG1_cd->SetFillColor(kRed+2); projG1_cd->SetFillStyle(3001);
  TH1F* projG2_au = Project(hExEg_au, "X", Ex2, width2, kBlue);
  TH1F* projG2_cd = Project(hExEg_cd, "X", Ex2, width2, kBlue+2); 
    projG2_cd->SetFillColor(kBlue+2); projG2_cd->SetFillStyle(3001);

  projG1_au->Draw();
  projG1_cd->Draw("same");
  projG2_au->Draw("same");
  projG2_cd->Draw("same");
}



//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle_HighEnergyStaggered(string treesuffix, string addbackOrTrack, 
                                               double bing, double binp)
{
  string histname = AddOrTrack(addbackOrTrack);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, addbackOrTrack);

  gStyle->SetPalette(kCool);

  double width = 8.0;
  //vector<double> energies = {3.5, 4.0, 4.8, 5.0};
  //vector<double> energies = {3.5,3.6,3.7,3.8,3.9,4.0,4.1,4.2,4.3,4.4,4.5,4.6,4.7,4.8,4.9,5.0};
  vector<double> energies = {3.4,3.6,3.8,4.0,4.8};
  auto cols = TColor::GetPalette();

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG_High = new TCanvas("cGPSG_High","cGPSG_High",1000,750);
  cGPSG_High->Divide(1,2);
  cGPSG_High->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gatesi
  for(int i=0; i<energies.size(); i++){
    Prj_DrawLines(energies.at(i)+width/2., width, cols.At(i));
  }

  //Projected axis
  cGPSG_High->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", energies.at(0)+width/2., width, cols.At(0));
  projP1->Draw("plc");
  for(int i=1; i<energies.size(); i++){
    TH1F* projP = Project(hExEg, "Y", energies.at(i)+width/2., width, cols.At(i));
    projP->Draw("plc same");
  } 

}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void DrawExThetaLab_SubPSp(TChain* tree)
{
  LoadChain_PhaseSpace();
  TCanvas* c = new TCanvas("c","c",500,500);

  // ExTL Good
  tree->Draw("Ex:ThetaLab>>hGood(26,104,156,150,-2,13)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<10","colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  // ExTL Bad
  tree->Draw("Ex:ThetaLab>>hBad(26,104,156,150,-2,13)","abs(T_FPMW_HF-10500)<1500 && Mugast.TelescopeNumber<10","colz");
  TH2F* hBad = (TH2F*) gDirectory->Get("hBad");
  
  // ExTL Clean
  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);

  // ExTL PSp
  tree_PSp00->Draw("Ex:ThetaLab>>hPSp(26,104,156,150,-2,13)","","colz");
  TH2F* hPSp = (TH2F*) gDirectory->Get("hPSp");

  double scale = 1.0;
  int minX = hGood->GetXaxis()->FindBin(106);//110);
  int maxX = hGood->GetXaxis()->FindBin(130);//145);
  int minY = hGood->GetYaxis()->FindBin(7.0);
  int maxY = hGood->GetYaxis()->FindBin(9.5);
  int step = (maxX-minX)/3;
   
  cout << minX << " " << maxX << "  ||  " << minY << " " << maxY << endl;

  for(int x = minX; x < maxX; x++){
//  cout << " ---- X = " << x << " ---- " << endl;
    for(int y = minY; y < maxY; y++){
//    cout << " ---- Y = " << y << " ---- " << endl;

      if(hClean->GetYaxis()->GetBinCenter(y) < (-0.0833 * hClean->GetXaxis()->GetBinCenter(x) ) + 17.75){
//        cout << x << " " << y << " || " << hPSp->GetBinContent(x,y) << " vs " << hClean->GetBinContent(x,y) << endl;

        if(hClean->GetBinContent(x,y)>2){
          while(hPSp->GetBinContent(x,y) * scale > hClean->GetBinContent(x,y)){
            scale *= 0.999;
            cout << hPSp->GetXaxis()->GetBinCenter(x) << ", " << hPSp->GetYaxis()->GetBinCenter(y) 
	         << " | " << hPSp->GetBinContent(x,y) * scale << " > " << hClean->GetBinContent(x,y) 
		 << " | " << scale << endl;
          }
        }

      }
    }
  }

  cout << " |||| scale = " << scale << " |||| " << endl;
  delete c;

  TCanvas* cClean = new TCanvas("cClean","cClean",500,500);
  hClean->Draw("colz");

  TCanvas* cPSp = new TCanvas("cPSp","cPSp",500,500);
  hPSp->Draw("colz");

  TCanvas* cFinal = new TCanvas("cFinal","cFinal",500,500);
  TH2F* hFinal = (TH2F*) hClean->Clone();
  hFinal->SetTitle("hFinal");
  hFinal->SetName("hFinal");
  hFinal->Add(hPSp,-scale);
  hFinal->Draw("colz");

  TCanvas* cProj = new TCanvas("cProj","cProj",1000,500);
  TH1F* hLowC = (TH1F*) hClean->ProjectionY("LC", minX,      minX+step);
  TH1F* hMidC = (TH1F*) hClean->ProjectionY("MC", minX+step, maxX-step);
  TH1F* hHghC = (TH1F*) hClean->ProjectionY("HC", maxX-step, maxX     );
 
  hLowC->SetLineColor(kBlack);
  hMidC->SetLineColor(kBlack);
  hHghC->SetLineColor(kBlack);

  TH1F* hLowP = (TH1F*) hPSp->ProjectionY("LP", minX,      minX+step);
  TH1F* hMidP = (TH1F*) hPSp->ProjectionY("MP", minX+step, maxX-step);
  TH1F* hHghP = (TH1F*) hPSp->ProjectionY("HP", maxX-step, maxX     );

  hLowP->SetLineColor(kRed);  hLowP->Scale(scale);
  hMidP->SetLineColor(kRed);  hMidP->Scale(scale);
  hHghP->SetLineColor(kRed);  hHghP->Scale(scale);

  TH1F* hLowF = (TH1F*) hFinal->ProjectionY("LF", minX,      minX+step);
  TH1F* hMidF = (TH1F*) hFinal->ProjectionY("MF", minX+step, maxX-step);
  TH1F* hHghF = (TH1F*) hFinal->ProjectionY("HF", maxX-step, maxX     );

  hLowF->SetLineColor(kBlue);
  hMidF->SetLineColor(kBlue);
  hHghF->SetLineColor(kBlue);

  cProj->Divide(3);
  cProj->cd(1); hLowC->Draw(); hLowP->Draw("same"); hLowF->Draw("same");
  cProj->cd(2); hMidC->Draw(); hMidP->Draw("same"); hMidF->Draw("same");
  cProj->cd(3); hHghC->Draw(); hHghP->Draw("same"); hHghF->Draw("same");

}

//////////////////////////////////////////////////////////////////////////////////////
void DrawEx_SubPSp(TChain* tree)
{
  LoadChain_PhaseSpace();
  TCanvas* c = new TCanvas("c","c",1000,1000);

  // ExTL Good
  tree->Draw("Ex>>hGood(2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<10","colz");
  TH1F* hGood = (TH1F*) gDirectory->Get("hGood");
  
  // ExTL Bad
  tree->Draw("Ex>>hBad(2000,-5,15)","abs(T_FPMW_HF-10500)<1500 && Mugast.TelescopeNumber<10","colz");
  TH1F* hBad = (TH1F*) gDirectory->Get("hBad");
  
  // ExTL Clean
  TH1F* hClean = (TH1F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);

  // ExTL PSp
  tree_PSp00->Draw("Ex>>hPSp(2000,-5,15)","","colz");
  TH1F* hPSp = (TH1F*) gDirectory->Get("hPSp");

  double scale = 0.134256;
  cout << " || SUBTRACTING PHASE SPACE WITH SCALE = " << scale << endl;

  TH1F* hFinal = (TH1F*) hClean->Clone();
  hFinal->SetTitle("hFinal");
  hFinal->SetName("hFinal");
  hFinal->Add(hPSp,-scale);

  hClean->SetLineColor(kBlack);                       hClean->Draw();
  hPSp  ->SetLineColor(kRed);    hPSp->Scale(scale);  hPSp  ->Draw("same");
  hFinal->SetLineColor(kBlue);                        hFinal->Draw("same");

}

//////////////////////////////////////////////////////////////////////////////////////
void AddKnownStateLines(TH1F* hist, double ymax)
{
  hist->Draw();

  TLine* Sn = new TLine(7.608, 0.0,7.608, ymax);
  Sn->SetLineColor(kOrange);
  Sn->SetLineWidth(4);
  Sn->SetLineStyle(kDashed);
  Sn->Draw();

  for(int i=0; i<numPeaks; i++){
    TLine* l = new TLine(means.at(i), 0.0, means.at(i), ymax);
    l->SetLineColor(kBlack);
    l->SetLineWidth(2);
    l->SetLineStyle(kDotted);
    l->Draw();
  }
}

void DrawGammas_AddBackVersusTracked()
{
  TCanvas* cGammaCompare = new TCanvas("cGammaCompare","cGammaCompare",1600,800);

  tree_both->Draw("trackEDC>>hTracked(1000,-1,9)",
                  "Mugast.TelescopeNumber<9 && abs(T_FPMW_HF-13300)<1500",
		  "");
  TH1F* hTracked = (TH1F*) gDirectory->Get("hTracked");
  hTracked->SetLineColor(kRed);
  hTracked->SetLineWidth(2);

  
  tree_both->Draw("AddBack_EDC>>hAddBack(1000,-1,9)",
                  "Mugast.TelescopeNumber<9 && abs(T_FPMW_HF-13300)<1500",
		  "");
  TH1F* hAddBack = (TH1F*) gDirectory->Get("hAddBack");
  hAddBack->SetLineColor(kBlue);
  hAddBack->SetLineWidth(2);

  hTracked->Draw(); hAddBack->Draw("same");

}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void Diagnostics_StripNumber(TTree* tree)
{
  TCanvas* c = new TCanvas("c","c",1000,1000);

  c->Divide(4,4);
  
  c->cd(1);
  tree->Draw("Mugast.DSSD_X>>hX1(130,0,130)","Mugast.TelescopeNumber==1","");
  c->cd(2);
  tree->Draw("Mugast.DSSD_Y>>hY1(130,0,130)","Mugast.TelescopeNumber==1","");
  
  c->cd(3);
  tree->Draw("Mugast.DSSD_X>>hX5(130,0,130)","Mugast.TelescopeNumber==5","");
  c->cd(4);
  tree->Draw("Mugast.DSSD_Y>>hY5(130,0,130)","Mugast.TelescopeNumber==5","");
  
  c->cd(5);
  tree->Draw("Mugast.DSSD_X>>hX2(130,0,130)","Mugast.TelescopeNumber==2","");
  c->cd(6);
  tree->Draw("Mugast.DSSD_Y>>hY2(130,0,130)","Mugast.TelescopeNumber==2","");

  c->cd(7);
  tree->Draw("Mugast.DSSD_X>>hX6(130,0,130)","Mugast.TelescopeNumber==6","");
  c->cd(8);
  tree->Draw("Mugast.DSSD_Y>>hY6(130,0,130)","Mugast.TelescopeNumber==6","");

  c->cd(9);
  tree->Draw("Mugast.DSSD_X>>hX3(130,0,130)","Mugast.TelescopeNumber==3","");
  c->cd(10);
  tree->Draw("Mugast.DSSD_Y>>hY3(130,0,130)","Mugast.TelescopeNumber==3","");

  c->cd(11);
  tree->Draw("Mugast.DSSD_X>>hX7(130,0,130)","Mugast.TelescopeNumber==7","");
  c->cd(12);
  tree->Draw("Mugast.DSSD_Y>>hY7(130,0,130)","Mugast.TelescopeNumber==7","");

  c->cd(13);
  tree->Draw("Mugast.DSSD_X>>hX4(130,0,130)","Mugast.TelescopeNumber==4","");
  c->cd(14);
  tree->Draw("Mugast.DSSD_Y>>hY4(130,0,130)","Mugast.TelescopeNumber==4","");



}

//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
		    
  tree_both->Draw(drawgood.c_str(),"abs(T_FPMW_HF-13300)<1500","colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str(), "abs(T_FPMW_HF-10500)<1500","colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Experiment.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite_SubPSp(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  LoadChain_PhaseSpace();
  TChain* treePSp = tree_PSp00;

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
  string drawpsp  = "Ex:ThetaLab>>hPSp("  + drawbase;
		    
  tree_both->Draw(drawgood.c_str(),"abs(T_FPMW_HF-13300)<1500","colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str(), "abs(T_FPMW_HF-10500)<1500","colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  treePSp->Draw(drawpsp.c_str(),"","colz");
  TH2F* hPSp  = (TH2F*) gDirectory->Get("hPSp");  hPSp->SetTitle("hPSp");  hPSp->SetName("hPSp");
  double scale = 0.134256; cout << GREEN << " USING SCALING FROM DrawEx_SubPSp, SCALE =  " << scale << RESET << endl;
  
  hClean->Add(hPSp, -scale);

  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Experiment.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}

//////////////////////////////////////////////////////////////////////////////////////
void GatePhaseSpace_MultiWrite(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  LoadChain_PhaseSpace();
  double finishTheta = startTheta + (numGates * gatesize);

  string draw = "Ex:ThetaLab>>h(" 
                + to_string(numGates) 
		+ "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		+ to_string(20.0/ExBinsize)
		+ ",-5,15)";
  
  tree_PSp00->Draw(draw.c_str(),"","colz");
  TH2F* h = (TH2F*) gDirectory->Get("h");
  h->Draw("colz");
  
  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) h->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_PhaseSpace.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}



//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite_ParallelCuts(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  string baseParallelCuts = "(ELab < (0.05458*ThetaLab)-3.456) && (ELab > (0.05458*ThetaLab)-5.696) && Ex < 8.7";

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
  string drawpsp  = "Ex:ThetaLab>>hPSp("  + drawbase;
		   
  string gategood = "abs(T_FPMW_HF-13300)<1500 &&" + baseParallelCuts;
  string gatebad  = "abs(T_FPMW_HF-10500)<1500 &&" + baseParallelCuts;

  tree_both->Draw(drawgood.c_str(), gategood.c_str(), "colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str() , gatebad.c_str() , "colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Parallel_Experiment.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}



//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite_ParallelCuts_SubPSp(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  string baseParallelCuts = "(ELab < (0.05458*ThetaLab)-3.456) && (ELab > (0.05458*ThetaLab)-5.696) && Ex < 8.7";

  LoadChain_PhaseSpace();
  TChain* treePSp = tree_PSp00;

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
  string drawpsp  = "Ex:ThetaLab>>hPSp("  + drawbase;
		   
  string gategood = "abs(T_FPMW_HF-13300)<1500 &&" + baseParallelCuts;
  string gatebad  = "abs(T_FPMW_HF-10500)<1500 &&" + baseParallelCuts;

  tree_both->Draw(drawgood.c_str(), gategood.c_str(), "colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str() , gatebad.c_str() , "colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  treePSp->Draw(drawpsp.c_str(),baseParallelCuts.c_str(),"colz");
  TH2F* hPSp  = (TH2F*) gDirectory->Get("hPSp");  hPSp->SetTitle("hPSp");  hPSp->SetName("hPSp");
  double scale = 1.0;
  for(int t=0; t<numGates; t++){
    for(int b=hClean->GetYaxis()->FindBin(7.7); b<hClean->GetYaxis()->FindBin(8.5);b++){
      while(hClean->GetBinContent(t,b)>2 && hClean->GetBinContent(t,b)<(hPSp->GetBinContent(t,b)*scale)){
        scale *= 0.999;
      }
    }
  }
  cout << "scale = " << scale << endl;

  hClean->Add(hPSp,-scale);

  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Parallel_Experiment_SubPSp.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}






//==================================================================================//

void TWOFNR_ApproachingUnbound()
{
  TCanvas* cTWOFNR = new TCanvas("cTWOFNR","cTWOFNR",1000,1000);
  cTWOFNR->Divide(2,2);
  cTWOFNR->cd(1)->SetLogy();
  cTWOFNR->cd(2)->SetLogy();
  cTWOFNR->cd(3)->SetLogy();
  cTWOFNR->cd(4)->SetLogy();
  
  cTWOFNR->cd(1);
  TGraph* s12_7500 = TWOFNR(7.500,2,1,0,0.5,2.5); s12_7500->SetLineColor(colourR->GetNumber()); s12_7500->SetLineWidth(4); s12_7500->SetLineStyle(1); s12_7500->GetXaxis()->SetRangeUser(100,160); s12_7500->GetYaxis()->SetRangeUser(0.004,2); s12_7500->Draw();
  TGraph* s12_7520 = TWOFNR(7.520,2,1,0,0.5,2.5); s12_7520->SetLineColor(colourB->GetNumber()); s12_7520->SetLineWidth(2); s12_7520->SetLineStyle(7); s12_7520->GetXaxis()->SetRangeUser(100,160); s12_7520->GetYaxis()->SetRangeUser(0.004,2); s12_7520->Draw("same");
  TGraph* s12_7540 = TWOFNR(7.540,2,1,0,0.5,2.5); s12_7540->SetLineColor(colourB->GetNumber()); s12_7540->SetLineWidth(2); s12_7540->SetLineStyle(7); s12_7540->GetXaxis()->SetRangeUser(100,160); s12_7540->GetYaxis()->SetRangeUser(0.004,2); s12_7540->Draw("same");
  TGraph* s12_7560 = TWOFNR(7.560,2,1,0,0.5,2.5); s12_7560->SetLineColor(colourB->GetNumber()); s12_7560->SetLineWidth(2); s12_7560->SetLineStyle(7); s12_7560->GetXaxis()->SetRangeUser(100,160); s12_7560->GetYaxis()->SetRangeUser(0.004,2); s12_7560->Draw("same");
  TGraph* s12_7580 = TWOFNR(7.580,2,1,0,0.5,2.5); s12_7580->SetLineColor(colourB->GetNumber()); s12_7580->SetLineWidth(2); s12_7580->SetLineStyle(7); s12_7580->GetXaxis()->SetRangeUser(100,160); s12_7580->GetYaxis()->SetRangeUser(0.004,2); s12_7580->Draw("same");
  TGraph* s12_7600 = TWOFNR(7.600,2,1,0,0.5,2.5); s12_7600->SetLineColor(colourG->GetNumber()); s12_7600->SetLineWidth(4); s12_7600->SetLineStyle(1); s12_7600->GetXaxis()->SetRangeUser(100,160); s12_7600->GetYaxis()->SetRangeUser(0.004,2); s12_7600->Draw("same");

  cTWOFNR->cd(2);
  TGraph* p12_7500 = TWOFNR(7.500,2,0,1,0.5,2.5); p12_7500->SetLineColor(colourR->GetNumber()); p12_7500->SetLineWidth(4); p12_7500->SetLineStyle(1); p12_7500->GetXaxis()->SetRangeUser(100,160); p12_7500->GetYaxis()->SetRangeUser(0.004,2); p12_7500->Draw();
  TGraph* p12_7520 = TWOFNR(7.520,2,0,1,0.5,2.5); p12_7520->SetLineColor(colourB->GetNumber()); p12_7520->SetLineWidth(2); p12_7520->SetLineStyle(7); p12_7520->GetXaxis()->SetRangeUser(100,160); p12_7520->GetYaxis()->SetRangeUser(0.004,2); p12_7520->Draw("same");
  TGraph* p12_7540 = TWOFNR(7.540,2,0,1,0.5,2.5); p12_7540->SetLineColor(colourB->GetNumber()); p12_7540->SetLineWidth(2); p12_7540->SetLineStyle(7); p12_7540->GetXaxis()->SetRangeUser(100,160); p12_7540->GetYaxis()->SetRangeUser(0.004,2); p12_7540->Draw("same");
  TGraph* p12_7560 = TWOFNR(7.560,2,0,1,0.5,2.5); p12_7560->SetLineColor(colourB->GetNumber()); p12_7560->SetLineWidth(2); p12_7560->SetLineStyle(7); p12_7560->GetXaxis()->SetRangeUser(100,160); p12_7560->GetYaxis()->SetRangeUser(0.004,2); p12_7560->Draw("same");
  TGraph* p12_7580 = TWOFNR(7.580,2,0,1,0.5,2.5); p12_7580->SetLineColor(colourB->GetNumber()); p12_7580->SetLineWidth(2); p12_7580->SetLineStyle(7); p12_7580->GetXaxis()->SetRangeUser(100,160); p12_7580->GetYaxis()->SetRangeUser(0.004,2); p12_7580->Draw("same");
  TGraph* p12_7600 = TWOFNR(7.600,2,0,1,0.5,2.5); p12_7600->SetLineColor(colourG->GetNumber()); p12_7600->SetLineWidth(4); p12_7600->SetLineStyle(1); p12_7600->GetXaxis()->SetRangeUser(100,160); p12_7600->GetYaxis()->SetRangeUser(0.004,2); p12_7600->Draw("same");

  cTWOFNR->cd(3);
  TGraph* d52_7500 = TWOFNR(7.500,2,0,2,2.5,2.5); d52_7500->SetLineColor(colourR->GetNumber()); d52_7500->SetLineWidth(4); d52_7500->SetLineStyle(1); d52_7500->GetXaxis()->SetRangeUser(100,160); d52_7500->GetYaxis()->SetRangeUser(0.004,2); d52_7500->Draw();
  TGraph* d52_7520 = TWOFNR(7.520,2,0,2,2.5,2.5); d52_7520->SetLineColor(colourB->GetNumber()); d52_7520->SetLineWidth(2); d52_7520->SetLineStyle(7); d52_7520->GetXaxis()->SetRangeUser(100,160); d52_7520->GetYaxis()->SetRangeUser(0.004,2); d52_7520->Draw("same");
  TGraph* d52_7540 = TWOFNR(7.540,2,0,2,2.5,2.5); d52_7540->SetLineColor(colourB->GetNumber()); d52_7540->SetLineWidth(2); d52_7540->SetLineStyle(7); d52_7540->GetXaxis()->SetRangeUser(100,160); d52_7540->GetYaxis()->SetRangeUser(0.004,2); d52_7540->Draw("same");
  TGraph* d52_7560 = TWOFNR(7.560,2,0,2,2.5,2.5); d52_7560->SetLineColor(colourB->GetNumber()); d52_7560->SetLineWidth(2); d52_7560->SetLineStyle(7); d52_7560->GetXaxis()->SetRangeUser(100,160); d52_7560->GetYaxis()->SetRangeUser(0.004,2); d52_7560->Draw("same");
  TGraph* d52_7580 = TWOFNR(7.580,2,0,2,2.5,2.5); d52_7580->SetLineColor(colourB->GetNumber()); d52_7580->SetLineWidth(2); d52_7580->SetLineStyle(7); d52_7580->GetXaxis()->SetRangeUser(100,160); d52_7580->GetYaxis()->SetRangeUser(0.004,2); d52_7580->Draw("same");
  TGraph* d52_7600 = TWOFNR(7.600,2,0,2,2.5,2.5); d52_7600->SetLineColor(colourG->GetNumber()); d52_7600->SetLineWidth(4); d52_7600->SetLineStyle(1); d52_7600->GetXaxis()->SetRangeUser(100,160); d52_7600->GetYaxis()->SetRangeUser(0.004,2); d52_7600->Draw("same");

  cTWOFNR->cd(4);
  TGraph* f72_7500 = TWOFNR(7.500,3,0,3,3.5,2.5); f72_7500->SetLineColor(colourR->GetNumber()); f72_7500->SetLineWidth(4); f72_7500->SetLineStyle(1); f72_7500->GetXaxis()->SetRangeUser(100,160); f72_7500->GetYaxis()->SetRangeUser(0.004,2); f72_7500->Draw();
  TGraph* f72_7520 = TWOFNR(7.520,3,0,3,3.5,2.5); f72_7520->SetLineColor(colourB->GetNumber()); f72_7520->SetLineWidth(2); f72_7520->SetLineStyle(7); f72_7520->GetXaxis()->SetRangeUser(100,160); f72_7520->GetYaxis()->SetRangeUser(0.004,2); f72_7520->Draw("same");
  TGraph* f72_7540 = TWOFNR(7.540,3,0,3,3.5,2.5); f72_7540->SetLineColor(colourB->GetNumber()); f72_7540->SetLineWidth(2); f72_7540->SetLineStyle(7); f72_7540->GetXaxis()->SetRangeUser(100,160); f72_7540->GetYaxis()->SetRangeUser(0.004,2); f72_7540->Draw("same");
  TGraph* f72_7560 = TWOFNR(7.560,3,0,3,3.5,2.5); f72_7560->SetLineColor(colourB->GetNumber()); f72_7560->SetLineWidth(2); f72_7560->SetLineStyle(7); f72_7560->GetXaxis()->SetRangeUser(100,160); f72_7560->GetYaxis()->SetRangeUser(0.004,2); f72_7560->Draw("same");
  TGraph* f72_7580 = TWOFNR(7.580,3,0,3,3.5,2.5); f72_7580->SetLineColor(colourB->GetNumber()); f72_7580->SetLineWidth(2); f72_7580->SetLineStyle(7); f72_7580->GetXaxis()->SetRangeUser(100,160); f72_7580->GetYaxis()->SetRangeUser(0.004,2); f72_7580->Draw("same");
  TGraph* f72_7600 = TWOFNR(7.600,3,0,3,3.5,2.5); f72_7600->SetLineColor(colourG->GetNumber()); f72_7600->SetLineWidth(4); f72_7600->SetLineStyle(1); f72_7600->GetXaxis()->SetRangeUser(100,160); f72_7600->GetYaxis()->SetRangeUser(0.004,2); f72_7600->Draw("same");

}

void TWOFNR_1MeVJumps()
{
  TCanvas* cTWOFNR = new TCanvas("cTWOFNR","cTWOFNR",1000,1000);
  cTWOFNR->Divide(2,2);
  cTWOFNR->cd(1)->SetLogy();
  cTWOFNR->cd(2)->SetLogy();
  cTWOFNR->cd(3)->SetLogy();
  cTWOFNR->cd(4)->SetLogy();
  
  cTWOFNR->cd(1);
  TGraph* s12_7500 = TWOFNR(2.000,2,1,0,0.5,2.5); s12_7500->SetLineColor(colourR->GetNumber()); s12_7500->SetLineWidth(4); s12_7500->SetLineStyle(1); s12_7500->GetXaxis()->SetRangeUser(100,160); s12_7500->GetYaxis()->SetRangeUser(0.004,2); s12_7500->Draw();
  TGraph* s12_7520 = TWOFNR(3.000,2,1,0,0.5,2.5); s12_7520->SetLineColor(colourB->GetNumber()); s12_7520->SetLineWidth(2); s12_7520->SetLineStyle(7); s12_7520->GetXaxis()->SetRangeUser(100,160); s12_7520->GetYaxis()->SetRangeUser(0.004,2); s12_7520->Draw("same");
  TGraph* s12_7540 = TWOFNR(4.000,2,1,0,0.5,2.5); s12_7540->SetLineColor(colourB->GetNumber()); s12_7540->SetLineWidth(2); s12_7540->SetLineStyle(7); s12_7540->GetXaxis()->SetRangeUser(100,160); s12_7540->GetYaxis()->SetRangeUser(0.004,2); s12_7540->Draw("same");
  TGraph* s12_7560 = TWOFNR(5.000,2,1,0,0.5,2.5); s12_7560->SetLineColor(colourB->GetNumber()); s12_7560->SetLineWidth(2); s12_7560->SetLineStyle(7); s12_7560->GetXaxis()->SetRangeUser(100,160); s12_7560->GetYaxis()->SetRangeUser(0.004,2); s12_7560->Draw("same");
  TGraph* s12_7580 = TWOFNR(6.000,2,1,0,0.5,2.5); s12_7580->SetLineColor(colourB->GetNumber()); s12_7580->SetLineWidth(2); s12_7580->SetLineStyle(7); s12_7580->GetXaxis()->SetRangeUser(100,160); s12_7580->GetYaxis()->SetRangeUser(0.004,2); s12_7580->Draw("same");
  TGraph* s12_7600 = TWOFNR(7.000,2,1,0,0.5,2.5); s12_7600->SetLineColor(colourG->GetNumber()); s12_7600->SetLineWidth(4); s12_7600->SetLineStyle(1); s12_7600->GetXaxis()->SetRangeUser(100,160); s12_7600->GetYaxis()->SetRangeUser(0.004,2); s12_7600->Draw("same");

  cTWOFNR->cd(2);
  TGraph* p12_7500 = TWOFNR(2.000,2,0,1,0.5,2.5); p12_7500->SetLineColor(colourR->GetNumber()); p12_7500->SetLineWidth(4); p12_7500->SetLineStyle(1); p12_7500->GetXaxis()->SetRangeUser(100,160); p12_7500->GetYaxis()->SetRangeUser(0.004,2); p12_7500->Draw();
  TGraph* p12_7520 = TWOFNR(3.000,2,0,1,0.5,2.5); p12_7520->SetLineColor(colourB->GetNumber()); p12_7520->SetLineWidth(2); p12_7520->SetLineStyle(7); p12_7520->GetXaxis()->SetRangeUser(100,160); p12_7520->GetYaxis()->SetRangeUser(0.004,2); p12_7520->Draw("same");
  TGraph* p12_7540 = TWOFNR(4.000,2,0,1,0.5,2.5); p12_7540->SetLineColor(colourB->GetNumber()); p12_7540->SetLineWidth(2); p12_7540->SetLineStyle(7); p12_7540->GetXaxis()->SetRangeUser(100,160); p12_7540->GetYaxis()->SetRangeUser(0.004,2); p12_7540->Draw("same");
  TGraph* p12_7560 = TWOFNR(5.000,2,0,1,0.5,2.5); p12_7560->SetLineColor(colourB->GetNumber()); p12_7560->SetLineWidth(2); p12_7560->SetLineStyle(7); p12_7560->GetXaxis()->SetRangeUser(100,160); p12_7560->GetYaxis()->SetRangeUser(0.004,2); p12_7560->Draw("same");
  TGraph* p12_7580 = TWOFNR(6.000,2,0,1,0.5,2.5); p12_7580->SetLineColor(colourB->GetNumber()); p12_7580->SetLineWidth(2); p12_7580->SetLineStyle(7); p12_7580->GetXaxis()->SetRangeUser(100,160); p12_7580->GetYaxis()->SetRangeUser(0.004,2); p12_7580->Draw("same");
  TGraph* p12_7600 = TWOFNR(7.000,2,0,1,0.5,2.5); p12_7600->SetLineColor(colourG->GetNumber()); p12_7600->SetLineWidth(4); p12_7600->SetLineStyle(1); p12_7600->GetXaxis()->SetRangeUser(100,160); p12_7600->GetYaxis()->SetRangeUser(0.004,2); p12_7600->Draw("same");

  cTWOFNR->cd(3);
  TGraph* d52_7500 = TWOFNR(2.000,2,0,2,2.5,2.5); d52_7500->SetLineColor(colourR->GetNumber()); d52_7500->SetLineWidth(4); d52_7500->SetLineStyle(1); d52_7500->GetXaxis()->SetRangeUser(100,160); d52_7500->GetYaxis()->SetRangeUser(0.004,2); d52_7500->Draw();
  TGraph* d52_7520 = TWOFNR(3.000,2,0,2,2.5,2.5); d52_7520->SetLineColor(colourB->GetNumber()); d52_7520->SetLineWidth(2); d52_7520->SetLineStyle(7); d52_7520->GetXaxis()->SetRangeUser(100,160); d52_7520->GetYaxis()->SetRangeUser(0.004,2); d52_7520->Draw("same");
  TGraph* d52_7540 = TWOFNR(4.000,2,0,2,2.5,2.5); d52_7540->SetLineColor(colourB->GetNumber()); d52_7540->SetLineWidth(2); d52_7540->SetLineStyle(7); d52_7540->GetXaxis()->SetRangeUser(100,160); d52_7540->GetYaxis()->SetRangeUser(0.004,2); d52_7540->Draw("same");
  TGraph* d52_7560 = TWOFNR(5.000,2,0,2,2.5,2.5); d52_7560->SetLineColor(colourB->GetNumber()); d52_7560->SetLineWidth(2); d52_7560->SetLineStyle(7); d52_7560->GetXaxis()->SetRangeUser(100,160); d52_7560->GetYaxis()->SetRangeUser(0.004,2); d52_7560->Draw("same");
  TGraph* d52_7580 = TWOFNR(6.000,2,0,2,2.5,2.5); d52_7580->SetLineColor(colourB->GetNumber()); d52_7580->SetLineWidth(2); d52_7580->SetLineStyle(7); d52_7580->GetXaxis()->SetRangeUser(100,160); d52_7580->GetYaxis()->SetRangeUser(0.004,2); d52_7580->Draw("same");
  TGraph* d52_7600 = TWOFNR(7.000,2,0,2,2.5,2.5); d52_7600->SetLineColor(colourG->GetNumber()); d52_7600->SetLineWidth(4); d52_7600->SetLineStyle(1); d52_7600->GetXaxis()->SetRangeUser(100,160); d52_7600->GetYaxis()->SetRangeUser(0.004,2); d52_7600->Draw("same");

  cTWOFNR->cd(4);
  TGraph* f72_7500 = TWOFNR(2.000,3,0,3,3.5,2.5); f72_7500->SetLineColor(colourR->GetNumber()); f72_7500->SetLineWidth(4); f72_7500->SetLineStyle(1); f72_7500->GetXaxis()->SetRangeUser(100,160); f72_7500->GetYaxis()->SetRangeUser(0.004,2); f72_7500->Draw();
  TGraph* f72_7520 = TWOFNR(3.000,3,0,3,3.5,2.5); f72_7520->SetLineColor(colourB->GetNumber()); f72_7520->SetLineWidth(2); f72_7520->SetLineStyle(7); f72_7520->GetXaxis()->SetRangeUser(100,160); f72_7520->GetYaxis()->SetRangeUser(0.004,2); f72_7520->Draw("same");
  TGraph* f72_7540 = TWOFNR(4.000,3,0,3,3.5,2.5); f72_7540->SetLineColor(colourB->GetNumber()); f72_7540->SetLineWidth(2); f72_7540->SetLineStyle(7); f72_7540->GetXaxis()->SetRangeUser(100,160); f72_7540->GetYaxis()->SetRangeUser(0.004,2); f72_7540->Draw("same");
  TGraph* f72_7560 = TWOFNR(5.000,3,0,3,3.5,2.5); f72_7560->SetLineColor(colourB->GetNumber()); f72_7560->SetLineWidth(2); f72_7560->SetLineStyle(7); f72_7560->GetXaxis()->SetRangeUser(100,160); f72_7560->GetYaxis()->SetRangeUser(0.004,2); f72_7560->Draw("same");
  TGraph* f72_7580 = TWOFNR(6.000,3,0,3,3.5,2.5); f72_7580->SetLineColor(colourB->GetNumber()); f72_7580->SetLineWidth(2); f72_7580->SetLineStyle(7); f72_7580->GetXaxis()->SetRangeUser(100,160); f72_7580->GetYaxis()->SetRangeUser(0.004,2); f72_7580->Draw("same");
  TGraph* f72_7600 = TWOFNR(7.000,3,0,3,3.5,2.5); f72_7600->SetLineColor(colourG->GetNumber()); f72_7600->SetLineWidth(4); f72_7600->SetLineStyle(1); f72_7600->GetXaxis()->SetRangeUser(100,160); f72_7600->GetYaxis()->SetRangeUser(0.004,2); f72_7600->Draw("same");


cout << "FROM ENERGY 2 MeV TO 7 MeV" << endl;
}

//==================================================================================//

double AGATAefficiency(double gammaE_MeV){
  double A = 1.0;
  double B = 1.0;
  double C = 0.000; //Fixed to 0, hence not included in IZanon's thesis equation
  double D = 1.0;   //IZ: C
  double E = 1.0;   //IZ: D
  double F = 1.0;   //IZ: E
  double G = 1.0;   //IZ: F
  
  double constX = log(gammaE_MeV / 0.1); //x = log(Eg/100 keV)
  double constY = log(gammaE_MeV / 1.0); //x = log(Eg/  1 MeV)

  double exponent1 = pow(A + (B * constX) + (C * constX * constX) , -G);
  double exponent2 = pow(D + (E * constY) + (F * constY * constY) , -G);

  double exponentF = pow(exponent1+exponent2, -1/G);

  double effic = exp(exponentF);


  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;


  return effic;
}
