#include "TMath.h"
#include "math.h"
#include <cmath>
#include "stdlib.h"

double pi = M_PI;

//-------------------------------------------------------
vector<double> meansGauss = { 0.000,
                              1.675,
		              3.572,
		              4.071,
		              4.456,
		              5.004,
		              5.228,
		              5.629,
		              7.252,
		              7.622,
		              7.855,
		              8.313,
		            //8.561, //MOVED TO VOIGT
		              8.962			 
		            };

vector<double> meansVoigt = { 8.561
		            };

vector<double> means; //meansGauss + meansVoigt, added together in Plots_Experiment()
//means.reserve(meansGauss.size() + meansVoigt.size());
//means.insert(means.end(), meansGauss.begin(),meansGauss.end());
//means.insert(means.end(), meansVoigt.begin(),meansVoigt.end());

//-------------------------------------------------------

int numGauss;// = 13; 
int numVoigt;// = 1;
int numPeaks;// = numGauss + numVoigt; 

double GetSigmaFromFunction(double Ex){ return (0.000969*Ex*Ex - 0.009116*Ex + 0.128396); }

vector<vector<double>> FitKnownPeaks_RtrnArray(TH1F* hist){
  //double minfit = -1.0, maxfit = 7.0;
  double minfit = -1.0, maxfit = 8.7;
  double binwidth = hist->GetXaxis()->GetBinWidth(5);

  // Build individual peak functions
  string nameBase = "Peak";
  string function = "gaus";
//TF1 **allPeaks = new TF1*[numPeaks];
  TF1 **allPeaks = new TF1*[numPeaks];
  for(int i=0; i<numGauss; i++){
    string nameHere = nameBase;
    nameHere += to_string(i);
    allPeaks[i] = new TF1(nameHere.c_str(), function.c_str(), minfit, maxfit);
    allPeaks[i]->SetLineColor(4001);
    allPeaks[i]->SetLineStyle(7);
    allPeaks[i]->SetNpx(500);
  }
  for(int i=numGauss; i<numPeaks; i++){
    string nameHere = nameBase;
    nameHere += to_string(i);
    allPeaks[i] = new TF1(nameHere.c_str(), "[3] * TMath::Voigt(x-[0],[1],[2])", minfit, maxfit);
    allPeaks[i]->SetLineColor(kCyan+2);
    allPeaks[i]->SetLineStyle(7);
    allPeaks[i]->SetNpx(500);
  }

  // Build background function (SIMPLE FOR NOW!)
  TF1 *bg = new TF1("bg","[0]",minfit, maxfit);
  bg->SetLineColor(kGreen);
  bg->SetLineStyle(9);
  bg->SetParNames("Background");

//// Build total function: Original
//int bgRefNum = ((numGauss-1)*3)+3;
//string s_full = "gaus(0)";
//for(int i=1; i<numGauss; i++){
//  s_full += "+ gaus("+ to_string(3*i) + ")"; 
//}
//TF1 *full = new TF1("fitAllPeaks",s_full.c_str(),minfit, maxfit);
//full->SetLineColor(kRed);
//full->SetNpx(500);

  // Build total function: with Voigt functions
  int bgRefNum = ((numGauss-1)*3)+3;
  string s_full = "gaus(0)";
  for(int i=1; i<numGauss; i++){
    s_full += "+ gaus("+ to_string(3*i) + ")"; 
  }
  for(int i=0; i<numVoigt; i++){
    s_full += "+ [" + to_string(3*numGauss + i + 3) 
                    + "] * TMath::Voigt(x-[" 
		    + to_string(3*numGauss + i + 0) 
		    + "],["
                    + to_string(3*numGauss + i + 1) 
		    + "],["
                    + to_string(3*numGauss + i + 2) 
		    + "])";
  }
  cout << s_full << endl;
  TF1 *full = new TF1("fitAllPeaks",s_full.c_str(),minfit, maxfit);
  full->SetLineColor(kRed);
  full->SetNpx(500);

  // Set and fix some parameters
  for(int i=0; i<numGauss; i++){
    full->FixParameter((i*3)+1, means.at(i));
    full->FixParameter((i*3)+2, GetSigmaFromFunction(means.at(i)));
    full->SetParameter((i*3)+0, 1e1);
    full->SetParLimits((i*3)+0, 0.0,1e8);
  }
  for(int i=0; i<numVoigt; i++){
    full->FixParameter((3*numGauss)+i+0, meansVoigt.at(i));
    full->FixParameter((3*numGauss)+i+1, GetSigmaFromFunction(meansVoigt.at(i)));
    full->SetParameter((3*numGauss)+i+2, 0.5);
    full->SetParLimits((3*numGauss)+i+2, 0.0,1.0);
    full->SetParameter((3*numGauss)+i+3, 1e2);
    full->SetParLimits((3*numGauss)+i+3, 0.0,1e8);
    cout << "fixing " << (3*numGauss)+i+ 0 << " to " << meansVoigt.at(i) << " and " << (3*numGauss)+i+1 << " to " << GetSigmaFromFunction(meansVoigt.at(i)) << endl;
  }


cout << " BEFORE FIT" << endl;
  // Fit full function
  hist->Fit(full, "RWQB", "", minfit, maxfit);
cout << " AFTER FIT" << endl;
  
  // Extract parameters
  const Double_t* finalPar = full->GetParameters();
  const Double_t* finalErr = full->GetParErrors();
  for(int i=0; i<numGauss; i++){
    allPeaks[i]->SetParameters(finalPar[0+(i*3)],
                               finalPar[1+(i*3)],
			       finalPar[2+(i*3)]);
    allPeaks[i]->SetLineColor(kOrange);
  }
cout << " MID EXTRACTION" << endl;
  for(int i=0; i<numVoigt; i++){
    allPeaks[numGauss+i]->SetParameters(finalPar[0+i+(numGauss*3)],
                                        finalPar[1+i+(numGauss*3)],
			                finalPar[2+i+(numGauss*3)],
			                finalPar[3+i+(numGauss*3)]);
//  allPeaks[i]->SetLineColor(kOrange);
  }
cout << " AFTER EXTRACTION" << endl;
 
  //Draw all peaks
  hist->Draw();
  for(int i=0; i<numPeaks; i++){
    allPeaks[i]->Draw("same");
  }

  //Write to screen
  cout << "===========================" << endl;
  cout << "== PEAK =========== AREA ==" << endl;
  
  //Loop over every peak
  vector<vector<double>> allpeaks;
  for(int i=0; i<numGauss; i++){
    //For AREA = HEIGHT * SIGMA * SQRT(2*PI)
    double A = (finalPar[0+(i*3)] * finalPar[2+(i*3)] * sqrt(2*pi)) / binwidth;

    double deltaA = A * sqrt( pow( finalErr[0+(i*3)]/finalPar[0+(i*3)] ,2) + pow( finalErr[2+(i*3)]/finalPar[2+(i*3)] ,2) );
    cout << "DELTAAREA = " 
	 << finalErr[0+(i*3)] << " / " 
	 << finalPar[0+(i*3)] << " and "
	 << finalErr[2+(i*3)] << " / " 
	 << finalPar[2+(i*3)] << " = " 
	 << deltaA << endl;

    cout << fixed << setprecision(3) 
	 << " #" << i << "  " 
	 << finalPar[(i*3)+1] << "\t" << setprecision(0)
	 << A << "\t+- " 
	 << deltaA << setprecision(3);
    cout << "    SQRT: " << sqrt(A) << endl;

    vector<double> onepeak; //energy, area and error for one peak
    onepeak.push_back(finalPar[(i*3)+1]);
    onepeak.push_back(A);
    onepeak.push_back(deltaA);
    allpeaks.push_back(onepeak);
    cout << "-------------" << endl;
  }
  for(int i=0; i<numVoigt; i++){
    //For AREA = ?????????????????
    //double A = 

    cout << "voigt funciton is area of unity" << endl;
    cout << "therefore area of peaks is multiplier/binwidth = " << finalPar[(numGauss*3)+i+3]/binwidth << endl;

    cout << fixed << setprecision(3) 
	 << " #" << numGauss+i << "  " 
	 << finalPar[(numGauss*3)+i+0] << "\t" << setprecision(0)
	 << finalPar[(numGauss*3)+i+3] / binwidth << "\t+- " 
	 << finalErr[(numGauss*3)+i+3] / binwidth << setprecision(3);
    cout << "    SQRT: " << sqrt(finalPar[(numGauss*3)+i+3]) << endl;

    vector<double> onepeak; //energy, area and error for one peak
    onepeak.push_back(finalPar[(numGauss*3)+i+0]);
    onepeak.push_back(finalPar[(numGauss*3)+i+3]/binwidth);
    onepeak.push_back(finalErr[(numGauss*3)+i+3]/binwidth);
    allpeaks.push_back(onepeak);

    cout << " VOIGT PEAK #" << i << " FUNCTION VALUES: [0] =" << finalPar[(numGauss*3)+i+0] << " , [1] = "
                                                              << finalPar[(numGauss*3)+i+1] << " , [2] = " 
                                                              << finalPar[(numGauss*3)+i+2] << " , [3] = " 
                                                              << finalPar[(numGauss*3)+i+3] << endl; 
    cout << "-------------" << endl;
  }

  cout << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! " << allpeaks.size() << endl;

  //if(removing){
  //  for(int b=1; b<hist->GetNbinsX()-1; b++){
  //    cout << b << " " << hist->GetBinContent(b) << " " << full->Eval(hist->GetBinCenter(b)) << endl;
  //    fitContByBin.push_back(full->Eval(hist->GetBinCenter(b)));
  //  }
  //}

  return allpeaks;
}

int FitKnownPeaks(TH1F* hist){auto dump = FitKnownPeaks_RtrnArray(hist); return 0; }

