void ElasticsFitAngleGates(string ddpp, double MinThetaCM, double MaxThetaCM, double StepWidthThetaCM){
  double Amp, PShft;  
  int waitpoint;
  TList* list = new TList();

  auto t = time(nullptr);
  auto tm = *localtime(&t);

  cout << " Elastics Equation::  " << endl;
  cout << "   Amplitude / PShift?  " << endl;

  if(ddpp == "dd"){
    cout << GREEN << "  (d,d)   T = 11Jul22" << RESET << endl;
    Amp = 27.817908; 
    PShft = 27.817762; 
  } else if (ddpp == "pp"){
    cout << GREEN << "  (p,p)   T = 11Jul22" << RESET << endl;
    Amp = 14.947797; 
    PShft = 14.947895; 
  }

  int nIter = (int) (MaxThetaCM-MinThetaCM)/StepWidthThetaCM;
  cout << "Number of steps: " << nIter << endl;

  ostringstream ossText;
  ossText << "ElasticsFitGates/outputElasticsAreas_" 
      << ddpp << "_"
      << put_time(&tm, "%Y-%m-%d_%H%M") 
      << ".root";
  auto strText = ossText.str();

  ofstream outfile;
  outfile.open (strText.c_str());

  for(int i=0; i<nIter; i++){
    double MinELab = (Amp * cos((180.-(MinThetaCM+(i*StepWidthThetaCM))) * (M_PI/180.))) + PShft;
    double MaxELab = (Amp * cos((180.-(MinThetaCM+((i+1)*StepWidthThetaCM))) * (M_PI/180.))) + PShft;

    cout << RESET;
    cout <<  "==========================================================" << endl;
    cout << MinELab << "  " << MaxELab << endl;
    cout << "----------------------------------------------------------" << endl;
    cout << "Gate: " << MinThetaCM+(i*StepWidthThetaCM)
         << " to "   << MinThetaCM+((i+1)*StepWidthThetaCM)
	 << endl;
    cout << "----------------------------------------------------------" << endl;
    cout << "Constructing histogram..." << endl;

    string histname = "Gate_" + to_string(MinThetaCM+(i*StepWidthThetaCM)) + "-" + to_string(MinThetaCM+((i+1)*StepWidthThetaCM));

    ElasticsGate(MinELab, MaxELab);
    TH1F* hist = (TH1F*) gDirectory->Get("hist");
    hist->Draw();
    hist->SetName(histname.c_str());
    gPad->Update();
    list->Add(hist);
    cout << "----------------------------------------------------------" << endl;

    double ddLine, ppLine;
    if(ddpp == "dd"){
      ddLine = 81.-5.5*log(MaxELab);
      ppLine = 79.-7.5*log(MaxELab);
    }else if (ddpp == "pp"){
      ddLine = 83.-7.5*log(MaxELab);
      ppLine = 82.-10.5*log(MaxELab);
    }
    
    cout << "Fitting with dd = " << ddLine << " and pp = " << ppLine << endl;
    vector<double> areas = DoubleGausNumbs(hist, 52., 87., ppLine, ddLine, true);

    outfile << areas.at(0) << "\t"
            << areas.at(1) << "\t"
            << areas.at(2) << "\t"
            << areas.at(3) << endl;

    gPad->Update();

  }

  outfile.close();

  ostringstream oss;
  oss << "ElasticsFitGates/ElasticsFitGates_" 
      << ddpp << "_"
      << put_time(&tm, "%Y-%m-%d_%H%M") 
      << ".root";
  auto str = oss.str();

  TFile* file = new TFile(str.c_str(),"RECREATE");
  list->Write("ElasticsGateFits",TObject::kSingleKey);
  file->ls();

}
