TGraphErrors* GetYield(string);
TGraphErrors* GetCorrectedYield(string);

double GetMean(TGraphErrors* gerr);
double GetManualRMS(TGraphErrors* gerr);
void LoadEfficiency(string name);

void Draw(TString isotope, TGraphErrors* ge);

TH2F* h2;

double m_Efficiency[100];

////////////////////////////////////////////
void ShowIndividualYield(string nucleus="182Hg")
{
  gROOT->SetStyle("pierre_style");
  h2 = new TH2F("h2","h2",500,25,60,500,0,25);
  h2->GetXaxis()->SetTitle("Z");
  h2->GetYaxis()->SetTitle("Y(Z) (%)");
  h2->GetXaxis()->CenterTitle();
  h2->GetYaxis()->CenterTitle();

  TLatex* iso;
  TLatex* text_rms;

  //***  ***//
  string name = "dat/yield_" + nucleus + ".dat";
  TGraphErrors* gerr = GetYield(name) ;
  gerr->SetLineColor(2);
  gerr->SetMarkerColor(2);
  gerr->SetMarkerStyle(22);
  gerr->SetName("yield");

  gerr->GetXaxis()->SetTitle("Nuclear Charge Z");
  gerr->GetYaxis()->SetTitle("Y(Z) %");
  gerr->GetXaxis()->CenterTitle();
  gerr->GetYaxis()->CenterTitle();

  LoadEfficiency(nucleus);
  TGraphErrors* gcorr = GetCorrectedYield(name) ;
  gcorr->SetLineStyle(9); 
  gcorr->SetMarkerColor(4); 
  gcorr->SetLineColor(4); 
  gcorr->SetName("yield_corr");

  TCanvas* c1 = new TCanvas("c1","c1",800,800);
  //c1->Divide(2,1);
  c1->cd();
  gerr->Draw("apl");
  gcorr->Draw("plsame");
  TLegend* leg = new TLegend(0.15,0.75,0.4,0.93);
  leg->SetBorderSize(0);
  leg->AddEntry("yield","Y(Z) before corr.","pl");
  leg->AddEntry("yield_corr","Y(Z) after corr.","pl");
  leg->Draw();


  int npoint = gerr->GetN();
  double Z;
  double Y, Ycorr;
  for(int i=0; i<npoint; i++){
    gerr->GetPoint(i,Z,Y);
    gcorr->GetPoint(i,Z,Ycorr);

    cout << Z << " " << Y << " " << Ycorr << " / " << (Y-Ycorr)/Y*100 << endl;
  }

  //Draw("",gerr);
  //c1->cd(2);
  //Draw("",gcorr);
}

////////////////////////////////////////////
void Draw(TString isotope, TGraphErrors* ge){
  double rms = GetManualRMS(ge)/2;
  TLatex* iso = new TLatex(48,25,isotope);
  TLatex* text_rms = new TLatex(28,25,Form("RMS=%f",rms));

  h2->Draw();
  //iso->Draw();
  //text_rms->Draw();
  ge->Draw("plsame");
}

////////////////////////////////////////////
TGraphErrors* GetYield(string filename)
{
  ifstream ifile;

  ifile.open(filename.c_str());

  int Z;
  double yield;
  double yield_err;
  TGraphErrors* g = new TGraphErrors();
  int i=0;
  while(!ifile.eof()){
    ifile >> Z >> yield >> yield_err;
    //cout << Z << " " << yield << " " << yield_err << endl;
    if(yield>0 && yield_err>0){
      g->SetPoint(i,Z,yield);
      g->SetPointError(i,0,yield_err);
      i++;
    }
  }

  g->SetMarkerStyle(8);

  ifile.close();
  return g;
}

////////////////////////////////////////////
TGraphErrors* GetCorrectedYield(string filename)
{
  ifstream ifile;

  ifile.open(filename.c_str());

  int Z;
  double yield[100];
  double yield_err[100];
  for(int i=0; i<100; i++){
    yield[i]=0;
    yield_err[i]=0;
  }

  double sum_yield = 0;
  TGraphErrors* g = new TGraphErrors();
  int i=0;
  while(!ifile.eof()){
    ifile >> Z >> yield[Z] >> yield_err[Z];
    if(yield[Z]>0 && yield_err[Z]>0 && m_Efficiency[Z]>0){
      yield[Z] = yield[Z]/m_Efficiency[Z];
      sum_yield += yield[Z];
      g->SetPoint(i,Z,yield[Z]);
      g->SetPointError(i,0,yield_err[Z]);
      i++;
    }
  }

  cout << "sum= " << sum_yield << endl;

  double scale = 200./sum_yield;

  for(int i=0; i<g->GetN(); i++){
    g->SetPoint(i,g->GetPointX(i),g->GetPointY(i)*scale);
  }

  g->SetMarkerStyle(8);

  ifile.close();
  return g;
}


////////////////////////////////////////////
void LoadEfficiency(string name)
{
  for(int i=0; i<100;i++)
    m_Efficiency[i] = 0;

  ifstream ifile;
  string input_filename = "efficiency/eff_" + name + ".dat";
  cout << "Loadfing efficiency filename: " << input_filename << endl;

  ifile.open(input_filename.c_str());

  int Z=0;
  double eff=0;
  while(!ifile.eof()){
    ifile >> Z >> eff;

    m_Efficiency[Z] = eff;
  }

}
////////////////////////////////////////////
double GetMean(TGraphErrors* gerr)
{
  double Y;
  double Z;
  double weight;
  double sum=0;
  double sumY=0;
  for(int i=0; i<gerr->GetN(); i++){
    gerr->GetPoint(i,Z,Y);
    if(Y>0){
      weight = Z*Y;
      sumY += Y;
      sum += weight;
    }
  }

  double mean;
  mean = sum/sumY;

  return mean;
}


////////////////////////////////////////////
double GetManualRMS(TGraphErrors* gerr)
{
  double mean = GetMean(gerr);
  double Y;
  double Z;
  double weight;
  double sum2=0;
  double sumY=0;
  for(int i=0; i<gerr->GetN(); i++){
    gerr->GetPoint(i,Z,Y);
    if(Y>0){
      weight = Z*Y;
      sumY += Y;
      sum2 += pow(Y*(Z-mean),2);
    }
  }

  double rms;
  rms = sqrt(sum2/sumY);

  return rms;
}


